@@ -3,7 +3,7 @@
 *                  F i l e - A s s o c i a t i o n   T a b l e                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,16 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFileDict.h,v 1.14 2001/08/23 16:24:51 jeroen Exp $                     *
+* $Id: FXFileDict.h,v 1.17 2002/01/23 18:53:57 jeroen Exp $                     *
 ********************************************************************************/
 #ifndef FXFILEDICT_H
 #define FXFILEDICT_H
 
+#ifndef FXDICT_H
+#include "FXDict.h"
+#endif
+
+
 
 /// Registers stuff to know about the extension
 struct FXFileAssoc {
@@ -46,15 +51,21 @@ private:
   FXApp        *app;            // Application object
   FXString      path;           // Where to search icons
 protected:
+  FXIconDict(){}
   virtual void *createData(const void*);
   virtual void deleteData(void*);
 private:
   FXIconDict(const FXIconDict&);
   FXIconDict &operator=(const FXIconDict&);
 public:
+  
+  /// Default icon search path
+  static const FXchar defaultIconPath[];
 
-  /// Construct an icon dictionary
-  FXIconDict(FXApp* a=NULL);
+public:
+
+  /// Construct an icon dictionary, with given path
+  FXIconDict(FXApp* a,const FXString& p=defaultIconPath);
 
   /// Get application
   FXApp* getApp() const { return app; }
@@ -97,8 +108,9 @@ class FXAPI FXFileDict : public FXDict {
 private:
   FXApp        *app;            // Application object
   FXSettings   *settings;       // Settings database where to get bindings
-  FXIconDict    icons;          // Icon table
+  FXIconDict   *icons;          // Icon table
 protected:
+  FXFileDict(){}
   virtual void *createData(const void*);
   virtual void deleteData(void*);
 private:
@@ -120,7 +132,7 @@ public:
   * Construct a dictionary mapping file-extension to file associations,
   * using the application registry settings as a source for the bindings.
   */
-  FXFileDict(FXApp* a=NULL);
+  FXFileDict(FXApp* a);
 
   /**
   * Construct a dictionary mapping file-extension to file associations,
