@@ -3,7 +3,7 @@
 *                         I c o n   L i s t   W i d g e t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,15 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXIconList.h,v 1.48 2001/08/23 16:24:51 jeroen Exp $                     *
+* $Id: FXIconList.h,v 1.54 2002/01/18 22:42:53 jeroen Exp $                     *
 ********************************************************************************/
 #ifndef FXICONLIST_H
 #define FXICONLIST_H
 
+#ifndef FXSCROLLAREA_H
+#include "FXScrollArea.h"
+#endif
+
 
 
 /// Icon list styles
@@ -69,7 +73,6 @@ protected:
   virtual void drawBigIcon(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
   virtual void drawMiniIcon(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
   virtual void drawDetails(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
-  virtual void drawFocus(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
 protected:
   enum {
     SELECTED      = 1,
@@ -237,6 +240,12 @@ public:
   /// Icon list can receive focus
   virtual FXbool canFocus() const;
 
+  /// Move the focus to this window
+  virtual void setFocus();
+
+  /// Remove the focus from this window
+  virtual void killFocus();
+
   /// Return viewport size
   virtual FXint getViewportHeight();
 
