@@ -3,7 +3,7 @@
 *                 S h u t t e r   C o n t a i n e r   W i d g e t               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,16 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXShutter.h,v 1.13 2001/08/23 16:24:51 jeroen Exp $                      *
+* $Id: FXShutter.h,v 1.15 2002/01/18 22:42:54 jeroen Exp $                      *
 ********************************************************************************/
 #ifndef FXSHUTTER_H
 #define FXSHUTTER_H
 
+#ifndef FXVERTICALFRAME_H
+#include "FXVerticalFrame.h"
+#endif
+
+
 
 struct FXTimer;
 class FXShutter;
