@@ -3,7 +3,7 @@
 *                           S l i d e r   W i d g e t                           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,25 +19,33 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXSlider.h,v 1.21 2001/08/28 22:52:25 jeroen Exp $                       *
+* $Id: FXSlider.h,v 1.25 2002/01/18 22:42:54 jeroen Exp $                       *
 ********************************************************************************/
 #ifndef FXSLIDER_H
 #define FXSLIDER_H
 
+#ifndef FXFRAME_H
+#include "FXFrame.h"
+#endif
+
 
 struct FXTimer;
 
 
 /// Slider Control styles
 enum {
-  SLIDER_HORIZONTAL  = 0,                          /// Slider shown horizontally
-  SLIDER_VERTICAL    = 0x00008000,                 /// Slider shown vertically
-  SLIDER_ARROW_UP    = 0x00010000,                 /// Slider has arrow head pointing up
-  SLIDER_ARROW_DOWN  = 0x00020000,                 /// Slider has arrow head pointing down
-  SLIDER_ARROW_LEFT  = SLIDER_ARROW_UP,            /// Slider has arrow head pointing left
-  SLIDER_ARROW_RIGHT = SLIDER_ARROW_DOWN,          /// Slider has arrow head pointing right
-  SLIDER_INSIDE_BAR  = 0x00040000,                 /// Slider is inside the slot rather than overhanging
-  SLIDER_NORMAL      = SLIDER_HORIZONTAL
+  SLIDER_HORIZONTAL   = 0,                        /// Slider shown horizontally
+  SLIDER_VERTICAL     = 0x00008000,               /// Slider shown vertically
+  SLIDER_ARROW_UP     = 0x00010000,               /// Slider has arrow head pointing up
+  SLIDER_ARROW_DOWN   = 0x00020000,               /// Slider has arrow head pointing down
+  SLIDER_ARROW_LEFT   = SLIDER_ARROW_UP,          /// Slider has arrow head pointing left
+  SLIDER_ARROW_RIGHT  = SLIDER_ARROW_DOWN,        /// Slider has arrow head pointing right
+  SLIDER_INSIDE_BAR   = 0x00040000,               /// Slider is inside the slot rather than overhanging
+  SLIDER_TICKS_TOP    = 0x00080000,               /// Ticks on the top of horizontal slider
+  SLIDER_TICKS_BOTTOM = 0x00100000,               /// Ticks on the bottom of horizontal slider
+  SLIDER_TICKS_LEFT   = SLIDER_TICKS_TOP,         /// Ticks on the left of vertical slider
+  SLIDER_TICKS_RIGHT  = SLIDER_TICKS_BOTTOM,      /// Ticks on the right of vertical slider
+  SLIDER_NORMAL       = SLIDER_HORIZONTAL
   };
 
 
@@ -57,18 +65,23 @@ protected:
   FXint         range[2];                 // Reported data range
   FXint         pos;                      // Reported data position
   FXint         incr;                     // Increment when auto-sliding
+  FXint         delta;                    // Interval between ticks
   FXint         headpos;                  // Head position
   FXint         headsize;                 // Head size
   FXint         slotsize;                 // Slot size
   FXColor       slotColor;                // Color of slot the head moves in
   FXTimer      *timer;                    // Timer for auto-sliding
-  FXint         dragpoint;                // Where on the head it's grabbed
+  FXint         dragpoint;                // Where the head is grabbed
   FXString      help;                     // Help string
   FXString      tip;                      // Tip string
 protected:
   FXSlider();
   virtual void layout();
+  FXint headPos(FXint v) const;
+  FXint headVal(FXint p) const;
   void drawSliderHead(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h);
+  void drawHorzTicks(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h);
+  void drawVertTicks(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint h);
 private:
   FXSlider(const FXSlider&);
   FXSlider &operator=(const FXSlider&);
@@ -152,6 +165,12 @@ public:
   /// Change the slider's auto-increment/decrement value
   void setIncrement(FXint inc);
 
+  /// Change the delta between ticks
+  void setTickDelta(FXint dist);
+
+  /// Get delta between ticks
+  FXint getTickDelta() const { return delta; }
+
   /// Change the color of the slot the slider head moves in
   void setSlotColor(FXColor clr);
 
