@@ -3,7 +3,7 @@
 *                          T I F F   I c o n   O b j e c t                      *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2001 Eric Gillet <egillet@ibelgique.com>. All Rights Reserved.  *
+* Copyright (C) 2001,2002 Eric Gillet.   All Rights Reserved.                   *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,11 +19,15 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTIFIcon.h,v 1.5 2001/07/24 21:49:27 jeroen Exp $                       *
+* $Id: FXTIFIcon.h,v 1.7 2002/01/18 22:55:03 jeroen Exp $                       *
 ********************************************************************************/
 #ifndef FXTIFICON_H
 #define FXTIFICON_H
 
+#ifndef FXICON_H
+#include "FXIcon.h"
+#endif
+
 
 /// TIFF Icon class
 class FXAPI FXTIFIcon : public FXIcon {
