@@ -3,7 +3,7 @@
 *           D e v i c e   C o n t e x t   F o r   P r i n t i n g               *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by celer@ipro.lug.usf.edu                                         *
 *********************************************************************************
@@ -21,7 +21,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDCPrint.cpp,v 1.22 2001/08/23 16:24:52 jeroen Exp $                    *
+* $Id: FXDCPrint.cpp,v 1.26.4.1 2003/03/27 20:34:43 fox Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -32,8 +32,6 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
@@ -701,11 +699,24 @@ void FXDCPrint::fillComplexPolygonRel(const FXPoint*,FXuint){
 
 
 // Draw string (only foreground bits)
-void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint){
+// Contributed by S. Ancelot <sancelot@online.fr>
+void FXDCPrint::drawText(FXint x,FXint y,const FXchar* string,FXuint len){
   FXfloat xx,yy;
   tfm(xx,yy,(FXfloat)x,(FXfloat)y);
-
-  outf("(%s) %g %g %d /Courier drawText\n",string,xx,yy,15);
+  bbox(xx,yy);
+  FXFontDesc fontdesc;
+  font->getFontDesc(fontdesc);
+  outf("gsave /%s findfont\n",font->getName().text());
+  outf("%d scalefont\n",font->getSize()/10);
+  outf("setfont\n");
+  outf("newpath\n%g %g moveto\n(",xx,yy);
+  for(FXuint i=0; i<len; i++){ 
+    if(string[i]=='(') outf("\\050"); 
+    else if(string[i]==')') outf("\\051"); 
+    else outf("%c",string[i]);
+    } 
+  outf(") show\n");
+  outf("grestore\n");
   }
 
 
@@ -720,7 +731,34 @@ void FXDCPrint::drawArea(const FXDrawable*,FXint,FXint,FXint,FXint,FXint,FXint){
 
 
 // Draw image
-void FXDCPrint::drawImage(const FXImage*,FXint,FXint){
+// Contibuted by dwalz@cs.uni-magdeburg.de
+void FXDCPrint::drawImage(const FXImage *img,FXint dx,FXint dy){
+  FXuint opts=img->getOptions();
+  if(opts&IMAGE_OWNED){
+    FXint    width  = img->getWidth();
+    FXint    height = img->getHeight();
+    FXuchar *buffer = img->getData();
+
+    outf("/picstr %d string def\n",width*3);
+    outf("%d %d translate\n",dx,height-dy);
+    outf("%d %d scale\n",width,-height);
+    outf("%d %d %d\n",width,height,8);
+    outf("[%d 0 0 -%d 0 %d]\n",width,height,height);
+    outf("{currentfile picstr readhexstring pop}\n");
+    outf("false %d\n",3);
+    outf("colorimage\n");
+
+    int bpp=3;
+    if(opts&IMAGE_ALPHA) bpp=4;
+
+    int end=width*height*bpp;
+    for(int i=0; i<end ; i+=bpp){
+      outhex(buffer[i]);
+      outhex(buffer[i+1]);
+      outhex(buffer[i+2]);
+      }
+    outf("\n");
+    }
   }
 
 
