@@ -3,7 +3,7 @@
 *                     D i r e c t o r y   B o x   O b j e c t                   *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1999,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1999,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDirBox.cpp,v 1.17 2001/08/22 04:01:55 jeroen Exp $                     *
+* $Id: FXDirBox.cpp,v 1.25.4.1 2002/08/05 16:14:30 fox Exp $                        *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,16 +30,12 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
 #include "FXFile.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXObjectList.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDrawable.h"
 #include "FXImage.h"
 #include "FXIcon.h"
 #include "FXGIFIcon.h"
@@ -69,9 +65,10 @@
   - When setting path, it adds all directories from the top down to
     the lowest directory.
   - It also adds common places in the file system.
-  - Add API's to set current path [from root down to whereever]
-  - Add code to read path of selected item
-  - Add some better icons
+  - Add API's to set current path [from root down to whereever].
+  - Add code to read path of selected item.
+  - Add some better icons.
+  - Should use FXFileDict for icons.
 */
 
 
@@ -271,22 +268,7 @@ FXDirBox::FXDirBox(FXComposite *p,FXint nvis,FXObject* tgt,FXSelector sel,FXuint
   computericon=new FXBMPIcon(getApp(),mycomputer,0,IMAGE_ALPHAGUESS);
   nethoodicon=new FXBMPIcon(getApp(),nethood,0,IMAGE_ALPHAGUESS);
   zipdiskicon=new FXBMPIcon(getApp(),zipdisk,0,IMAGE_ALPHAGUESS);
-#ifndef WIN32
-  addItemLast(NULL,"/",foldericon,foldericon);
-  setCurrentItem(getFirstItem());
-  directory="/";
-#else
-  /*
-  FXchar result[1024];
-  fxgetcurrentdir(result);
-  directory=FXString(result).left(2)+PATHSEPSTRING;
-  addItemLast(NULL,directory,harddiskicon,harddiskicon);
-  setCurrentItem(getFirstItem());
-  */
-  FXchar result[1024];
-  fxgetcurrentdir(result);
-  setDirectory(FXString(result).left(2)+PATHSEPSTRING);
-#endif
+  setDirectory(PATHSEPSTRING);
   }
 
 
@@ -344,7 +326,6 @@ long FXDirBox::onCmdSetValue(FXObject*,FXSelector,void* ptr){
 
 // Change value
 long FXDirBox::onCmdSetStringValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdSetStringValue: NULL pointer.\n",getClassName()); }
   setDirectory(*((FXString*)ptr));
   return 1;
   }
@@ -352,8 +333,7 @@ long FXDirBox::onCmdSetStringValue(FXObject*,FXSelector,void* ptr){
 
 // Obtain value
 long FXDirBox::onCmdGetStringValue(FXObject*,FXSelector,void* ptr){
-  if(ptr==NULL){ fxerror("%s::onCmdGetStringValue: NULL pointer.\n",getClassName()); }
-  *((FXString*)ptr)=directory;
+  *((FXString*)ptr)=getDirectory();
   return 1;
   }
 
@@ -393,7 +373,6 @@ long FXDirBox::onCommand(FXObject*,FXSelector,void* ptr){
 
 // Set directory
 void FXDirBox::setDirectory(const FXString& pathname){
-  /*
   FXTreeItem *item;
   FXString    string;
   FXint       part;
@@ -412,142 +391,65 @@ void FXDirBox::setDirectory(const FXString& pathname){
       while(1){
         string=directory.extract(part,PATHSEP);
         if(string.empty()) break;
-        item=addItemLast(item,string,foldericon,foldericon);
+        item=addItemLast(item,string,foldericon,foldericon);            // FIXME should simply find icons via FXFileDict
         part++;
         }
       setCurrentItem(item);
 #else
+      DWORD drivemask=GetLogicalDrives();
+      char drivename[10];
+      drivename[1]=':';
+      drivename[2]='\\';
+      drivename[3]=0;
+      for(drivename[0]='a'; drivename[0]<='z'; drivename[0]++){
+        if(drivemask&1){
+          switch(GetDriveType(drivename)){
+            case DRIVE_REMOVABLE:
+              if(drivename[0]=='a' || drivename[0]=='b'){
+                item=addItemLast(NULL,drivename,floppyicon,floppyicon);
+                }
+              else{
+                item=addItemLast(NULL,drivename,zipdiskicon,zipdiskicon);
+                }
+              break;
+            case DRIVE_FIXED:
+              item=addItemLast(NULL,drivename,harddiskicon,harddiskicon);
+              break;
+            case DRIVE_REMOTE:
+              item=addItemLast(NULL,drivename,networkicon,networkicon);
+              break;
+            case DRIVE_CDROM:
+              item=addItemLast(NULL,drivename,cdromicon,cdromicon);
+              break;
+            case DRIVE_RAMDISK:
+              item=addItemLast(NULL,drivename,harddiskicon,harddiskicon);
+              break;
+            case DRIVE_UNKNOWN:
+            case DRIVE_NO_ROOT_DIR:
+            default:
+              goto x;
+            }
+          if(islower((FXuchar)drivename[0]))
+            directory[0]=tolower((FXuchar)directory[0]);
+          else
+            directory[0]=toupper((FXuchar)directory[0]);
+          if(directory[0]==drivename[0]){   // FIXME what about network neighborhood?
+            part=1;
+            while(1){
+              string=directory.extract(part,PATHSEP);
+              if(string.empty()) break;
+              item=addItemLast(item,string,foldericon,foldericon);      // FIXME should simply find icons via FXFileDict
+              part++;
+              }
+            setCurrentItem(item);
+            }
+          }
+x:      drivemask>>=1;
+        }
 #endif
       recalc();
       }
     }
-  */
-
-  FXchar abspath[MAXPATHLEN+1];
-  FXString string;
-  FXTreeItem *item;
-  FXuint part=1;
-
-  // Empty path:- ignore
-  if(pathname.empty()) return;
-
-  // Exactly equal already
-  if(directory!=pathname){
-    fxabspath(abspath,NULL,pathname.text());
-
-    // Equal
-    if(directory!=abspath){
-
-      // Perhaps got trailing filename?
-      while(!fxistopdir(abspath) && !fxisdir(abspath)){
-        fxupdir(abspath,abspath);
-        }
-
-      // Switch
-#ifndef WIN32
-      if(directory!=abspath){
-        directory=abspath;
-        clearItems();
-        item=addItemFirst(NULL,PATHSEPSTRING,foldericon,foldericon);    // Add root
-        while(1){
-          string=directory.extract(part,PATHSEP);
-          if(string.empty()) break;
-          item=addItemLast(item,string,foldericon,foldericon);
-          part++;
-          }
-        setCurrentItem(item);
-        recalc();
-        }
-#else
-      if(directory!=abspath){
-        directory=abspath;
-        clearItems();
-
-        //FXTreeItem *desktopitem=addItemLast(NULL,"Desktop",desktopicon,desktopicon);
-	//FXTreeItem *computeritem=addItemLast(desktopitem,"My Computer",computericon,computericon);
-	FXTreeItem *computeritem=NULL;
-
-	LPTSTR lpDriveStrings=0;
-	DWORD dwCount=GetLogicalDriveStrings(0,lpDriveStrings); // Get required size
-	if(dwCount>0){
-	  FXMALLOC(&lpDriveStrings,TCHAR,dwCount+1);
-	  if(dwCount>=GetLogicalDriveStrings(dwCount,lpDriveStrings)){
-	    DWORD dwDriveMask=GetLogicalDrives();
-	    TCHAR szRootPathName[]=TEXT("?:\\");
-	    LPTSTR lpParse=lpDriveStrings;
-	    for(szRootPathName[0]=TEXT('a'); szRootPathName[0]<=TEXT('z'); szRootPathName[0]++){
-	      if(dwDriveMask&1){
-		UINT drivetype=GetDriveType(lpParse);
-		FXString caption;
-		switch(drivetype){
-		  case DRIVE_REMOVABLE:
-		    if(szRootPathName[0]==TEXT('a') || szRootPathName[0]==TEXT('b')){
-		      //caption=FXString("Floppy (")+FXString(lpParse[0],1)+FXString(":)");
-		      caption=lpParse;
-		      item=addItemLast(computeritem,caption,floppyicon,floppyicon);
-		      }
-		    else{
-		      //caption=FXString("Removable Disk (")+FXString(lpParse[0],1)+FXString(":)");
-		      caption=lpParse;
-		      item=addItemLast(computeritem,caption,zipdiskicon,zipdiskicon);
-		      }
-		    break;
-		  case DRIVE_FIXED:
-		    //caption=FXString("(")+FXString(lpParse[0],1)+":)";
-		    caption=lpParse;
-    		    item=addItemLast(computeritem,caption,harddiskicon,harddiskicon);
-		    break;
-		  case DRIVE_REMOTE:
-		    //caption=FXString("Network Disk (")+FXString(lpParse[0],1)+":)";
-		    caption=lpParse;
-    		    item=addItemLast(computeritem,caption,networkicon,networkicon);
-		    break;
-		  case DRIVE_CDROM:
-		    //caption=FXString("CD-ROM (")+FXString(lpParse[0],1)+":)";
-		    caption=lpParse;
-		    item=addItemLast(computeritem,caption,cdromicon,cdromicon);
-		    break;
-		  case DRIVE_UNKNOWN:
-		  case DRIVE_NO_ROOT_DIR:
-		  case DRIVE_RAMDISK:
-		    FXASSERT(FALSE);
-		    break;
-		  default:
-		    FXASSERT(FALSE);
-		  }
-
-                // Fix from John Burnell <j.burnell@irl.cri.nz> lpParse is NOT always uppercase.
-                if(islower((FXuchar)lpParse[0]))
-                  directory[0]=tolower((FXuchar)directory[0]);
-                else
-                  directory[0]=toupper((FXuchar)directory[0]);
-
-		// Is the selected directory on this drive?
-		if(directory[0]==lpParse[0]){
-		  while(TRUE){
-		    string=directory.extract(part,PATHSEP);
-		    if(string.empty()) break;
-		    item=addItemLast(item,string,foldericon,foldericon);
-		    part++;
-		    }
-		  setCurrentItem(item);
-		  }
-
-		// Move on to the next drive
-		lpParse+=lstrlen(lpParse)+1;
-		}
-	      dwDriveMask>>=1;
-	      }
-	    }
-	  FXFREE(&lpDriveStrings);
-	  }
-	//addItemLast(desktopitem,"Network Neighborhood",nethoodicon,nethoodicon);
-	//addItemLast(desktopitem,"My Briefcase",briefcaseicon,briefcaseicon);
-        recalc();
-        }
-#endif
-      }
-    }
   }
 
 
@@ -585,6 +487,7 @@ void FXDirBox::load(FXStream& store){
 
 // Delete it
 FXDirBox::~FXDirBox(){
+  clearItems();
   delete foldericon;
   delete cdromicon;
   delete desktopicon;
