@@ -3,7 +3,7 @@
 *                     D i r e c t o r y   L i s t   O b j e c t                 *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXDirList.cpp,v 1.59 2001/07/05 22:34:29 jeroen Exp $                    *
+* $Id: FXDirList.cpp,v 1.63.4.5 2003/05/01 18:02:01 fox Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,25 +30,16 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
 #include "FXFile.h"
 #include "FXURL.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXApp.h"
-#include "FXId.h"
 #include "FXFont.h"
-#include "FXDrawable.h"
-#include "FXImage.h"
 #include "FXIcon.h"
 #include "FXBMPIcon.h"
 #include "FXGIFIcon.h"
-#include "FXWindow.h"
-#include "FXComposite.h"
 #include "FXScrollbar.h"
-#include "FXScrollArea.h"
-#include "FXTreeList.h"
 #include "FXDirList.h"
 #include "FXFileDict.h"
 #ifdef WIN32
@@ -274,6 +265,7 @@ FXIMPLEMENT(FXDirList,FXTreeList,FXDirListMap,ARRAYNUMBER(FXDirListMap))
 
 // For serialization
 FXDirList::FXDirList(){
+  flags|=FLAG_ENABLED|FLAG_DROPTARGET;
   refresh=NULL;
   counter=0;
   associations=NULL;
@@ -284,6 +276,7 @@ FXDirList::FXDirList(){
 // Directory List Widget
 FXDirList::FXDirList(FXComposite *p,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXTreeList(p,nvis,tgt,sel,opts,x,y,w,h),pattern("*"){
+  flags|=FLAG_ENABLED|FLAG_DROPTARGET;
   associations=NULL;
   open_folder=new FXGIFIcon(getApp(),minifolderopen);
   closed_folder=new FXGIFIcon(getApp(),minifolderclosed);
@@ -322,7 +315,6 @@ void FXDirList::create(){
   floppyicon->create();
   zipdiskicon->create();
   scanRootDir(FALSE);
-  dropEnable();
   }
 
 
@@ -989,7 +981,8 @@ FXbool FXDirList::scanRootDir(FXbool relist){
 
     // Refresh subitems of root if necessary
     if(stat(pathname,&info)==0){
-      filetime=FXMAX(info.st_mtime,info.st_ctime);
+      filetime=info.st_mtime;
+      if(filetime<0) filetime=0;
       if(relist || (item->date!=filetime) || ((filetime==0) && (counter==0))){
         if(listSubDir(item,pathname)){
           sortChildItems(item);
@@ -1031,7 +1024,8 @@ FXbool FXDirList::scanSubDir(FXDirItem *par,FXchar *pathname,FXbool relist){
 
         // Refresh subitems of item if necessary
         if(stat(pathname,&info)==0){
-          filetime=FXMAX(info.st_mtime,info.st_ctime);
+          filetime=info.st_mtime;
+          if(filetime<0) filetime=0;
           if(relist || (item->date!=filetime) || ((filetime==0) && (counter==0))){
             if(listSubDir(item,pathname)){
               sortChildItems(item);
@@ -1085,7 +1079,12 @@ FXbool FXDirList::listSubDir(FXDirItem *par,FXchar *pathname){
     if(!ISPATHSEP(*(pathtail-1))) *pathtail++=PATHSEP;
 
     // Process directory entries
+#ifdef FOX_THREAD_SAFE
+    struct fxdirent dirresult;
+    while(readdir_r(dirp,&dirresult,&dp)==0 && dp){
+#else
     while((dp=readdir(dirp))!=NULL){
+#endif
       name=dp->d_name;
 
       // A dot special file?
@@ -1108,7 +1107,8 @@ FXbool FXDirList::listSubDir(FXDirItem *par,FXchar *pathname){
       if(!S_ISDIR(info.st_mode) && !((options&DIRLIST_SHOWFILES) && fxfilematch(pattern.text(),name,matchmode))) continue;
 
       // File change/mod time
-      filetime=FXMAX(info.st_mtime,info.st_ctime);
+      filetime=info.st_mtime;
+      if(filetime<0) filetime=0;
 
       // Find it
       for(item=before; item; item=item->inext){
@@ -1118,9 +1118,9 @@ FXbool FXDirList::listSubDir(FXDirItem *par,FXchar *pathname){
             before=before->inext;
             if(it->prev) it->prev->next=it->next; else it->parent->first=it->next;
             if(it->next) it->next->prev=it->prev; else it->parent->last=it->prev;
+            removeItems(it->first,it->last);
             if(currentitem==it) currentitem=NULL;
             if(anchoritem==it) anchoritem=NULL;
-            removeItems(it->first,it->last);
             changed=TRUE;
             delete it;
             }
@@ -1225,9 +1225,9 @@ fnd:  item->iprev=after;
     before=before->inext;
     if(it->prev) it->prev->next=it->next; else it->parent->first=it->next;
     if(it->next) it->next->prev=it->prev; else it->parent->last=it->prev;
+    removeItems(it->first,it->last);
     if(currentitem==it) currentitem=NULL;
     if(anchoritem==it) anchoritem=NULL;
-    removeItems(it->first,it->last);
     changed=TRUE;
     delete it;
     }
@@ -1267,16 +1267,16 @@ FXbool FXDirList::listRoots(){
 
   // FIXME we want to move this logic into FXFileDict
   DWORD drivemask=GetLogicalDrives();
-  TCHAR drivename[10];
-  drivename[1]=TEXT(':');
-  drivename[2]=TEXT('\\');
+  char drivename[10];
+  drivename[1]=':';
+  drivename[2]='\\';
   drivename[3]=0;
-  for(drivename[0]=TEXT('a'); drivename[0]<=TEXT('z'); drivename[0]++){
+  for(drivename[0]='a'; drivename[0]<='z'; drivename[0]++){
     if(drivemask&1){
       UINT drivetype=GetDriveType(drivename);
       switch(drivetype){
         case DRIVE_REMOVABLE:
-          if(drivename[0]==TEXT('a') || drivename[0]==TEXT('b')){
+          if(drivename[0]=='a' || drivename[0]=='b'){
             addItemLast(NULL,createItem(drivename,floppyicon,floppyicon,NULL));
             }
           else{
@@ -1366,20 +1366,20 @@ FXbool FXDirList::scanRootDir(FXbool relist){
     // Refresh subitems of root if necessary
     hFile=CreateFile(pathname,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_FLAG_BACKUP_SEMANTICS,NULL);
     if(hFile!=INVALID_HANDLE_VALUE){
-      if(GetFileTime(hFile,NULL,NULL,&ftLastWriteTime)){
-
-        // Convert
-        filetime=fxfiletime(ftLastWriteTime);
-
-        if(relist || (item->date!=filetime) || (filetime==0) && (counter==0)){
-          if(listSubDir(item,pathname)){
-            sortChildItems(item);
-            changed=TRUE;
-            }
-          }
-        item->date=filetime;
-        }
+      GetFileTime(hFile,NULL,NULL,&ftLastWriteTime);
       CloseHandle(hFile);
+
+      // Convert
+      filetime=fxfiletime(ftLastWriteTime);
+      if(filetime<0) filetime=0;
+
+      if(relist || (item->date!=filetime) || (filetime==0) && (counter==0)){
+        if(listSubDir(item,pathname)){
+          sortChildItems(item);
+          changed=TRUE;
+          }
+        }
+      item->date=filetime;
       }
 
     // Then check subdirectories
@@ -1415,20 +1415,20 @@ FXbool FXDirList::scanSubDir(FXDirItem *par,FXchar *pathname,FXbool relist){
         // Refresh subitems of item if necessary
         hFile=CreateFile(pathname,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_FLAG_BACKUP_SEMANTICS,NULL);
         if(hFile!=INVALID_HANDLE_VALUE){
-          if(GetFileTime(hFile,NULL,NULL,&ftLastWriteTime)){
-
-            // Convert
-            filetime=fxfiletime(ftLastWriteTime);
-
-            if(relist || (item->date!=filetime) || ((filetime==0) && (counter==0))){
-              if(listSubDir(item,pathname)){
-                sortChildItems(item);
-                changed=TRUE;
-                }
-              }
-            item->date=filetime;
-            }
+          GetFileTime(hFile,NULL,NULL,&ftLastWriteTime);
           CloseHandle(hFile);
+
+          // Convert
+          filetime=fxfiletime(ftLastWriteTime);
+          if(filetime<0) filetime=0;
+
+          if(relist || (item->date!=filetime) || ((filetime==0) && (counter==0))){
+            if(listSubDir(item,pathname)){
+              sortChildItems(item);
+              changed=TRUE;
+              }
+            }
+          item->date=filetime;
           }
 
         // Then check subdirectories
@@ -1491,6 +1491,7 @@ FXbool FXDirList::listSubDir(FXDirItem *par,FXchar *pathname){
 
       // Convert it
       filetime=fxfiletime(ffData.ftLastWriteTime);
+      if(filetime<0) filetime=0;
 
       // Find it
       for(item=before; item; item=item->inext){
@@ -1500,9 +1501,9 @@ FXbool FXDirList::listSubDir(FXDirItem *par,FXchar *pathname){
             before=before->inext;
             if(it->prev) it->prev->next=it->next; else it->parent->first=it->next;
             if(it->next) it->next->prev=it->prev; else it->parent->last=it->prev;
+            removeItems(it->first,it->last);
             if(currentitem==it) currentitem=NULL;
             if(anchoritem==it) anchoritem=NULL;
-            removeItems(it->first,it->last);
             changed=TRUE;
             delete it;
             }
@@ -1610,9 +1611,9 @@ fnd:  item->iprev=after;
     before=before->inext;
     if(it->prev) it->prev->next=it->next; else it->parent->first=it->next;
     if(it->next) it->next->prev=it->prev; else it->parent->last=it->prev;
+    removeItems(it->first,it->last);
     if(currentitem==it) currentitem=NULL;
     if(anchoritem==it) anchoritem=NULL;
-    removeItems(it->first,it->last);
     changed=TRUE;
     delete it;
     }
