@@ -3,7 +3,7 @@
 *      F i l e   I n f o r m a t i o n   a n d   M a n i p u l a t i o n        *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * Contributed by: Sean Hubbell                                                  *
 *********************************************************************************
@@ -21,7 +21,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFile.cpp,v 1.97 2001/08/29 04:12:48 jeroen Exp $                       *
+* $Id: FXFile.cpp,v 1.131.4.10 2003/05/02 17:36:58 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -29,8 +29,13 @@
 #include "FXStream.h"
 #include "FXString.h"
 #include "FXFile.h"
+#ifdef WIN32
+#include <shellapi.h>
+#endif
 
-
+#ifdef __SC__
+using namespace FXFile;
+#endif
 
 /*
   Notes:
@@ -58,6 +63,10 @@
       -  Uppercase only filename.
 
   - Perhaps use GetEnvironmentVariable instead of getenv?
+
+  - FXFile::search() what if some paths are quoted, like
+
+      \this\dir;"\that\dir with a ;";\some\other\dir
 */
 
 
@@ -67,7 +76,7 @@
 
 /*******************************************************************************/
 
-  
+
 // Return value of environment variable name
 FXString FXFile::getEnvironment(const FXString& name){
   return FXString(getenv(name.text()));
@@ -110,10 +119,11 @@ FXString FXFile::getCurrentDrive(){
   }
 
 
+#ifdef WIN32
+
 // Change current drive prefix "a:"
 // This is the same method as used in VC++ CRT.
 FXbool FXFile::setCurrentDrive(const FXString& prefix){
-#ifdef WIN32
   FXchar buffer[3];
   if(!prefix.empty() && isalpha((FXuchar)prefix[0]) && prefix[1]==':'){
     buffer[0]=prefix[0];
@@ -122,15 +132,36 @@ FXbool FXFile::setCurrentDrive(const FXString& prefix){
     return SetCurrentDirectory(buffer);
     }
   return FALSE;
-#else
-  return TRUE;
-#endif
   }
 
+#else
+
+// Change current drive prefix "a:"
+FXbool FXFile::setCurrentDrive(const FXString&){
+  return TRUE;
+  }
+
+#endif
+
 
 // Get home directory for a given user
 FXString FXFile::getUserDirectory(const FXString& user){
 #ifndef WIN32
+#ifdef FOX_THREAD_SAFE
+  struct passwd pwdresult,*pwd;
+  char buffer[1024];
+  if(user.empty()){
+    register const FXchar* str;
+    if((str=getenv("HOME"))!=NULL) return str;
+    if((str=getenv("USER"))!=NULL || (str=getenv("LOGNAME"))!=NULL){
+      if(getpwnam_r(str,&pwdresult,buffer,sizeof(buffer),&pwd)==0 && pwd) return pwd->pw_dir;
+      }
+    if(getpwuid_r(getuid(),&pwdresult,buffer,sizeof(buffer),&pwd)==0 && pwd) return pwd->pw_dir;
+    return PATHSEPSTRING;
+    }
+  if(getpwnam_r(user.text(),&pwdresult,buffer,sizeof(buffer),&pwd)==0 && pwd) return pwd->pw_dir;
+  return PATHSEPSTRING;
+#else
   register struct passwd *pwd;
   if(user.empty()){
     register const FXchar* str;
@@ -143,18 +174,19 @@ FXString FXFile::getUserDirectory(const FXString& user){
     }
   if((pwd=getpwnam(user.text()))!=NULL) return pwd->pw_dir;
   return PATHSEPSTRING;
+#endif
 #else
   if(user.empty()){
     register const FXchar *str1,*str2;
     if((str1=getenv("HOME"))!=NULL) return str1;
-    if((str2=getenv("HOMEPATH"))!=NULL){      // This should be good for WinNT, Win2K according to MSDN       
+    if((str2=getenv("HOMEPATH"))!=NULL){      // This should be good for WinNT, Win2K according to MSDN
       if((str1=getenv("HOMEDRIVE"))==NULL) str1="c:";
       return FXString(str1,str2);
       }
-//    FXchar buffer[MAX_PATH]
-//    if(SHGetFolderPath(NULL,CSIDL_PERSONAL|CSIDL_FLAG_CREATE,NULL,O,buffer)==S_OK){
-//      return buffer; 
-//      }
+//  FXchar buffer[MAX_PATH]
+//  if(SHGetFolderPath(NULL,CSIDL_PERSONAL|CSIDL_FLAG_CREATE,NULL,O,buffer)==S_OK){
+//    return buffer;
+//    }
     HKEY hKey;
     if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",0,KEY_READ,&hKey)==ERROR_SUCCESS){
       FXchar home[MAXPATHLEN];
@@ -182,6 +214,19 @@ FXString FXFile::getExecPath(){
   }
 
 
+// Get temporary directory
+//FXString FXFile::getTempDirectory(){
+//   FXString tmp=getCurrentDirectory();
+// #ifndef win32
+//   if exist /tmp and isWritable then tmp=/tmp
+//   else if exist /var/tmp and isWritable then tmp=/var/tmp
+// #else
+//   tmp=getEnvironment("TEMP") // or TMP etc
+// #endif
+//   return isWritable(tmp)?tmp:FXString::null;
+//  }
+
+
 // Return directory part of pathname, assuming full pathname.
 // Note that directory("/bla/bla/") is "/bla/bla" and NOT "/bla".
 // However, directory("/bla/bla") is "/bla" as we expect!
@@ -205,7 +250,7 @@ FXString FXFile::directory(const FXString& file){
 
 
 // Return name and extension part of pathname.
-// Note that name("/bla/bla/") is "" and NOT "bla". 
+// Note that name("/bla/bla/") is "" and NOT "bla".
 // However, name("/bla/bla") is "bla" as we expect!
 FXString FXFile::name(const FXString& file){
   register FXint f,n;
@@ -313,9 +358,10 @@ FXString FXFile::stripExtension(const FXString& file){
   }
 
 
+#ifdef WIN32
+
 // Return drive letter prefix "c:"
 FXString FXFile::drive(const FXString& file){
-#ifdef WIN32
   FXchar buffer[3];
   if(isalpha((FXuchar)file[0]) && file[1]==':'){
     buffer[0]=tolower((FXuchar)file[0]);
@@ -323,10 +369,17 @@ FXString FXFile::drive(const FXString& file){
     buffer[2]='\0';
     return FXString(buffer,2);
     }
-#endif
   return FXString::null;
   }
 
+#else
+
+// Return drive letter prefix "c:"
+FXString FXFile::drive(const FXString&){
+  return FXString::null;
+  }
+
+#endif
 
 // Perform tilde or environment variable expansion
 FXString FXFile::expand(const FXString& file){
@@ -372,51 +425,75 @@ FXString FXFile::expand(const FXString& file){
   }
 
 
-// Simplify a file path; the path will remain relative if it was relative
-// For example, simplify("..//aaa/./bbb//../c/") becomes "../aaa/c/".
-// Note that a single trailing "/" is not removed!
+
+// Simplify a file path; the path will remain relative if it was relative,
+// or absolute if it was absolute.  Also, a trailing "/" will be preserved
+// as this is important in other functions.
+//
+// Examples:
+//
+//  /aa/bb/../cc    -> /aa/cc
+//  /aa/bb/../cc/   -> /aa/cc/
+//  /aa/bb/../..    -> /
+//  ../../bb        -> ../../bb
+//  ../../bb/       -> ../../bb/
+//  /../            -> /
+//  ./aa/bb/../../  -> ./
+//  a/..            -> .
+//  a/../           -> ./
+//  ./a             -> ./a
+//  /////./././     -> /
 FXString FXFile::simplify(const FXString& file){
   if(!file.empty()){
     FXString result=file;
-    FXint beg,end,head,cur;
-    head=0;
+    register FXint p,q,s;
+    p=q=0;
 #ifndef WIN32
-    if(ISPATHSEP(result[0])){
-      head++;
-      }
-    else{
-      while(result[head]=='.' && result[head+1]=='.' && ISPATHSEP(result[head+2])) head+=3;
-      if(result[head]=='.' && result[head+1]=='.' && result[head+2]==0) head+=2;
+    if(ISPATHSEP(result[q])){
+      result[p++]=PATHSEP;
+      while(ISPATHSEP(result[q])) q++;
       }
 #else
-    if(ISPATHSEP(result[0])){
-      head++;
-      if(ISPATHSEP(result[1])) head++;      // UNC
+    if(ISPATHSEP(result[q])){         // UNC
+      result[p++]=PATHSEP;
+      q++;
+      if(ISPATHSEP(result[q])){
+        result[p++]=PATHSEP;
+        while(ISPATHSEP(result[q])) q++;
+        }
       }
-    else if(isalpha((FXuchar)result[0]) && result[1]==':'){
-      head+=2;
-      if(ISPATHSEP(result[2])) head++;
-      }
-    else{
-      while(result[head]=='.' && result[head+1]=='.' && ISPATHSEP(result[head+2])) head+=3;
-      if(result[head]=='.' && result[head+1]=='.' && result[head+2]==0) head+=2;
+    else if(isalpha((FXuchar)result[q]) && result[q+1]==':'){
+      result[p++]=result[q++];
+      result[p++]=':';
+      q++;
+      if(ISPATHSEP(result[q])){
+        result[p++]=PATHSEP;
+        while(ISPATHSEP(result[q])) q++;
+        }
       }
 #endif
-    for(beg=end=cur=head; result[beg]; beg=end){
-      while(result[end] && !ISPATHSEP(result[end])) end++;
-      if(beg+1==end && result[beg]=='.'){
+    s=p;
+    while(result[q]){
+      while(result[q] && !ISPATHSEP(result[q])){
+        result[p++]=result[q++];
         }
-      else if(beg+2==end && result[beg]=='.' && result[beg+1]=='.'){
-        if(head<cur) cur--;
-        while(head<cur && !ISPATHSEP(result[cur-1])) cur--;
+      if(2<=p && result[p-1]=='.' && ISPATHSEP(result[p-2]) && ISPATHSEP(result[q])){
+        p-=2;
         }
-      else if(beg<end){
-        while(beg<end){ result[cur++]=result[beg++]; }
-        if(result[end]){ result[cur++]=PATHSEP; }
+      else if(3<=p && result[p-1]=='.' && result[p-2]=='.' && ISPATHSEP(result[p-3]) && !(5<=p && result[p-4]=='.' && result[p-5]=='.')){
+        p-=2;
+        if(s+2<=p){
+          p-=2;
+          while(s<p && !ISPATHSEP(result[p])) p--;
+          if(p==0) result[p++]='.';
+          }
+        }
+      if(ISPATHSEP(result[q])){
+        while(ISPATHSEP(result[q])) q++;
+        if(!ISPATHSEP(result[p-1])) result[p++]=PATHSEP;
         }
-      while(result[end] && ISPATHSEP(result[end])) end++;
       }
-    result[cur]=0;
+    result[p]=0;
     return result;
     }
   return FXString::null;
@@ -478,7 +555,7 @@ FXString FXFile::absolute(const FXString& base,const FXString& file){
 FXString FXFile::relative(const FXString& base,const FXString& file){
   register FXint p,q,b;
   FXString result;
-  
+
   // Find branch point
 #ifndef WIN32
   for(p=b=0; base[p] && base[p]==file[p]; p++){
@@ -489,30 +566,30 @@ FXString FXFile::relative(const FXString& base,const FXString& file){
     if(ISPATHSEP(file[p])) b=p;
     }
 #endif
-  
+
   // Paths are equal
   if((base[p]=='\0' || (ISPATHSEP(base[p]) && base[p+1]=='\0')) && (file[p]=='\0' || (ISPATHSEP(file[p]) && file[p+1]=='\0'))){
     return ".";
     }
-    
+
   // Directory base is prefix of file
   if((base[p]=='\0' && ISPATHSEP(file[p])) || (file[p]=='\0' && ISPATHSEP(base[p]))){
     b=p;
     }
-  
+
   // Up to branch point
   for(p=q=b; base[p]; p=q){
     while(base[q] && !ISPATHSEP(base[q])) q++;
     if(q>p) result.append(".." PATHSEPSTRING);
     while(base[q] && ISPATHSEP(base[q])) q++;
     }
-  
+
   // Strip leading path character off, if any
   while(ISPATHSEP(file[b])) b++;
-  
+
   // Append tail end
   result.append(&file[b]);
-  
+
   return result;
   }
 
@@ -607,7 +684,7 @@ FXString FXFile::upLevel(const FXString& file){
   return file;
   }
 
- 
+
 // Check if file represents absolute pathname
 FXbool FXFile::isAbsolute(const FXString& file){
   if(!file.empty()){
@@ -650,12 +727,48 @@ FXbool FXFile::isFile(const FXString& file){
 FXbool FXFile::isLink(const FXString& file){
 #ifndef WIN32
   struct stat status;
-  return !file.empty() && (::stat(file.text(),&status)==0) && S_ISLNK(status.st_mode);
+  return !file.empty() && (::lstat(file.text(),&status)==0) && S_ISLNK(status.st_mode);
 #else
   return FALSE;
 #endif
   }
 
+
+/*
+/// Return true if input path represents a file share
+FXbool FXAPI isShare(const FXString& file);
+
+
+// Return true if input path represents a file share of the form "\\" or "\\server"
+FXbool FXFile::isShare(const FXString& file){
+#ifndef WIN32
+  return FALSE;
+#else
+  if(ISPATHSEP(file[0]) && ISPATHSEP(file[1]) && file.findf(PATHSEP,2)<0){
+    HANDLE hEnum;
+    NETRESOURCE host;
+    host.dwScope=RESOURCE_GLOBALNET;
+    host.dwType=RESOURCETYPE_DISK;
+    host.dwDisplayType=RESOURCEDISPLAYTYPE_GENERIC;
+    host.dwUsage=RESOURCEUSAGE_CONTAINER;
+    host.lpLocalName=NULL;
+    host.lpRemoteName=(char*)file.text();
+    host.lpComment=NULL;
+    host.lpProvider=NULL;
+
+    // This shit thows "First-chance exception in blabla.exe (KERNEL32.DLL): 0x000006BA: (no name)"
+    // when non-existing server name is passed in.  Don't know if this is dangerous...
+    if(WNetOpenEnum((file[2]?RESOURCE_GLOBALNET:RESOURCE_CONTEXT),RESOURCETYPE_DISK,0,(file[2]?&host:NULL),&hEnum)==NO_ERROR){
+      WNetCloseEnum(hEnum);
+      return TRUE;
+      }
+    }
+  return FALSE;
+#endif
+  }
+*/
+
+
 // Check if file represents a directory
 FXbool FXFile::isDirectory(const FXString& file){
 #ifndef WIN32
@@ -682,7 +795,12 @@ FXbool FXFile::isWritable(const FXString& file){
 
 // Return true if file is executable (thanks to gehriger@linkcad.com)
 FXbool FXFile::isExecutable(const FXString& file){
+#ifndef WIN32
   return !file.empty() && access(file.text(),X_OK)==0;
+#else
+  SHFILEINFO sfi;
+  return !file.empty() && SHGetFileInfo(file.text(),0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)!=0;
+#endif
   }
 
 
@@ -727,7 +845,8 @@ FXbool FXFile::isOwnerExecutable(const FXString& file){
   struct stat status;
   return !file.empty() && (::stat(file.text(),&status)==0) && (status.st_mode&S_IXUSR);
 #else
-  return TRUE;
+  SHFILEINFO sfi;
+  return !file.empty() && SHGetFileInfo(file.text(),0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)!=0;
 #endif
   }
 
@@ -773,7 +892,8 @@ FXbool FXFile::isGroupExecutable(const FXString& file){
   struct stat status;
   return !file.empty() && (::stat(file.text(),&status)==0) && (status.st_mode&S_IXGRP);
 #else
-  return TRUE;
+  SHFILEINFO sfi;
+  return !file.empty() && SHGetFileInfo(file.text(),0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)!=0;
 #endif
   }
 
@@ -819,7 +939,8 @@ FXbool FXFile::isOtherExecutable(const FXString& file){
   struct stat status;
   return !file.empty() && (::stat(file.text(),&status)==0) && (status.st_mode&S_IXOTH);
 #else
-  return TRUE;
+  SHFILEINFO sfi;
+  return !file.empty() && SHGetFileInfo(file.text(),0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)!=0;
 #endif
   }
 
@@ -936,41 +1057,66 @@ FXTime FXFile::touched(const FXString& file){
   }
 
 
+
 #ifndef WIN32                 // UNIX
 
 
 // List all the files in directory
 FXint FXFile::listFiles(FXString*& list,const FXString& path,const FXString& pattern,FXuint flags){
-  register const char *name;
-  register struct dirent *dp;
-  register FXString *newlist;
-  register DIR *dirp;
-  register FXint count=0;
-  register FXint size=0;
-  
+  FXuint matchmode=FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE;
+  FXchar pathname[MAXPATHLEN],*pathtail,*name;
+  struct dirent *dp;
+  FXString *newlist;
+  FXint count=0;
+  FXint size=0;
+  DIR *dirp;
+  struct stat inf;
+
   // Initialize to empty
   list=NULL;
-    
-  // One single root on UNIX
+/*
+  // One single root under Unix
   if(path.empty()){
     list=new FXString[2];
-    list[0]=PATHSEPSTRING;
-    return 1;
+    list[count++]=PATHSEPSTRING;
+    return count;
     }
-  
+*/
+  // Folding case
+  if(flags&LIST_CASEFOLD) matchmode|=FILEMATCH_CASEFOLD;
+
+  // Copy directory name
+  strcpy(pathname,path.text());
+
   // Get directory stream pointer
-  dirp=opendir(path.text());
+  dirp=opendir(pathname);
   if(!dirp) return 0;
 
+  // Built directory part
+  pathtail=pathname+strlen(pathname)-1;
+  if(!ISPATHSEP(*pathtail)) *++pathtail=PATHSEP;
+  ++pathtail;
+
   // Loop over directory entries
+#ifdef FOX_THREAD_SAFE
+  struct fxdirent dirresult;
+  while(readdir_r(dirp,&dirresult,&dp)==0 && dp){
+#else
   while((dp=readdir(dirp))!=NULL){
+#endif
     name=dp->d_name;
 
-    // Skip '.' and '..'
-    //if(name[0]=='.' && (name[1]==0 || (name[1]=='.' && name[2]==0))) continue;
+    // Build full pathname
+    strcpy(pathtail,name);
 
-    // Is it a directory or does it match the pattern?
-    if(!fxfilematch(pattern.text(),name,flags)) continue;
+    // Get info on file
+    if(::stat(pathname,&inf)!=0) continue;
+
+    // Filter out files; a bit tricky...
+    if(!S_ISDIR(inf.st_mode) && ((flags&LIST_NO_FILES) || (name[0]=='.' && !(flags&LIST_HIDDEN_FILES)) || (!(flags&LIST_ALL_FILES) && !fxfilematch(pattern.text(),name,matchmode)))) continue;
+
+    // Filter out directories; even more tricky!
+    if(S_ISDIR(inf.st_mode) && ((flags&LIST_NO_DIRS) || (name[0]=='.' && (name[1]==0 || (name[1]=='.' && name[2]==0 && (flags&LIST_NO_PARENT)) || (name[1]!='.' && !(flags&LIST_HIDDEN_DIRS)))) || (!(flags&LIST_ALL_DIRS) && !fxfilematch(pattern.text(),name,matchmode)))) continue;
 
     // Grow list
     if(count+1>=size){
@@ -994,52 +1140,40 @@ FXint FXFile::listFiles(FXString*& list,const FXString& path,const FXString& pat
 
 // List all the files in directory
 FXint FXFile::listFiles(FXString*& list,const FXString& path,const FXString& pattern,FXuint flags){
-  register const char *name;
-  register FXString *newlist;
-  register FXint count=0;
-  register FXint size=0;
+  FXuint matchmode=FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE;
+  FXchar pathname[MAXPATHLEN],*pathtail,*name;
+  FXString *newlist;
+  FXint count=0;
+  FXint size=0;
   WIN32_FIND_DATA ffData;
-  DWORD drivemask,mask,nCount,nSize,i,j;
+  DWORD nCount,nSize,i,j;
   HANDLE hFindFile,hEnum;
-  NETRESOURCE host;
-  FXchar letter[4],server[200];
-  
+  FXchar server[200];
+
   // Initialize to empty
   list=NULL;
-    
+
+/*
   // Each drive is a root on windows
   if(path.empty()){
-    
-    // Get drive letter mask
-    drivemask=GetLogicalDrives();
-    
-    // Count drives
-    for(mask=drivemask; mask; mask>>=1){ if(mask&1) size++; }
-  
-    // Allocate root list
-    list=new FXString[size+2];
-    
-    // Initialize string to "a:\"
+    FXchar letter[4];
     letter[0]='a';
     letter[1]=':';
     letter[2]=PATHSEP;
     letter[3]='\0';
-    
-    // Add drive letters
-    for(mask=drivemask; mask; mask>>=1,letter[0]++){
+    list=new FXString[28];
+    for(DWORD mask=GetLogicalDrives(); mask; mask>>=1,letter[0]++){
       if(mask&1) list[count++]=letter;
       }
-    
-    // Add top level for file shares "\\"
     list[count++]=PATHSEPSTRING PATHSEPSTRING;    // UNC for file shares
-    
-    // Return number of roots
     return count;
     }
-
+*/
+/*
   // A UNC name was given of the form "\\" or "\\server"
   if(ISPATHSEP(path[0]) && ISPATHSEP(path[1]) && path.findf(PATHSEP,2)<0){
-    
+    NETRESOURCE host;
+
     // Fill in
     host.dwScope=RESOURCE_GLOBALNET;
     host.dwType=RESOURCETYPE_DISK;
@@ -1049,27 +1183,27 @@ FXint FXFile::listFiles(FXString*& list,const FXString& path,const FXString& pat
     host.lpRemoteName=(char*)path.text();
     host.lpComment=NULL;
     host.lpProvider=NULL;
-    
+
     // Open network enumeration
     if(WNetOpenEnum((path[2]?RESOURCE_GLOBALNET:RESOURCE_CONTEXT),RESOURCETYPE_DISK,0,(path[2]?&host:NULL),&hEnum)==NO_ERROR){
       NETRESOURCE resource[16384/sizeof(NETRESOURCE)];
-      FXTRACE((100,"Enumerating=%s\n",path.text()));
+      FXTRACE((1,"Enumerating=%s\n",path.text()));
       while(1){
         nCount=-1;    // Read as many as will fit
         nSize=sizeof(resource);
         if(WNetEnumResource(hEnum,&nCount,resource,&nSize)!=NO_ERROR) break;
         for(i=0; i<nCount; i++){
- 
+
           // Dump what we found
-          FXTRACE((100,"dwScope=%s\n",resource[i].dwScope==RESOURCE_CONNECTED?"RESOURCE_CONNECTED":resource[i].dwScope==RESOURCE_GLOBALNET?"RESOURCE_GLOBALNET":resource[i].dwScope==RESOURCE_REMEMBERED?"RESOURCE_REMEMBERED":"?"));
-          FXTRACE((100,"dwType=%s\n",resource[i].dwType==RESOURCETYPE_ANY?"RESOURCETYPE_ANY":resource[i].dwType==RESOURCETYPE_DISK?"RESOURCETYPE_DISK":resource[i].dwType==RESOURCETYPE_PRINT?"RESOURCETYPE_PRINT":"?"));
-          FXTRACE((100,"dwDisplayType=%s\n",resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_DOMAIN?"RESOURCEDISPLAYTYPE_DOMAIN":resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_SERVER?"RESOURCEDISPLAYTYPE_SERVER":resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_SHARE?"RESOURCEDISPLAYTYPE_SHARE":resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_GENERIC?"RESOURCEDISPLAYTYPE_GENERIC":resource[i].dwDisplayType==6?"RESOURCEDISPLAYTYPE_NETWORK":resource[i].dwDisplayType==7?"RESOURCEDISPLAYTYPE_ROOT":resource[i].dwDisplayType==8?"RESOURCEDISPLAYTYPE_SHAREADMIN":resource[i].dwDisplayType==9?"RESOURCEDISPLAYTYPE_DIRECTORY":resource[i].dwDisplayType==10?"RESOURCEDISPLAYTYPE_TREE":resource[i].dwDisplayType==11?"RESOURCEDISPLAYTYPE_NDSCONTAINER":"?"));
-          FXTRACE((100,"dwUsage=%s\n",resource[i].dwUsage==RESOURCEUSAGE_CONNECTABLE?"RESOURCEUSAGE_CONNECTABLE":resource[i].dwUsage==RESOURCEUSAGE_CONTAINER?"RESOURCEUSAGE_CONTAINER":"?"));
-          FXTRACE((100,"lpLocalName=%s\n",resource[i].lpLocalName));
-          FXTRACE((100,"lpRemoteName=%s\n",resource[i].lpRemoteName));
-          FXTRACE((100,"lpComment=%s\n",resource[i].lpComment));
-          FXTRACE((100,"lpProvider=%s\n\n",resource[i].lpProvider));
- 
+          FXTRACE((1,"dwScope=%s\n",resource[i].dwScope==RESOURCE_CONNECTED?"RESOURCE_CONNECTED":resource[i].dwScope==RESOURCE_GLOBALNET?"RESOURCE_GLOBALNET":resource[i].dwScope==RESOURCE_REMEMBERED?"RESOURCE_REMEMBERED":"?"));
+          FXTRACE((1,"dwType=%s\n",resource[i].dwType==RESOURCETYPE_ANY?"RESOURCETYPE_ANY":resource[i].dwType==RESOURCETYPE_DISK?"RESOURCETYPE_DISK":resource[i].dwType==RESOURCETYPE_PRINT?"RESOURCETYPE_PRINT":"?"));
+          FXTRACE((1,"dwDisplayType=%s\n",resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_DOMAIN?"RESOURCEDISPLAYTYPE_DOMAIN":resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_SERVER?"RESOURCEDISPLAYTYPE_SERVER":resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_SHARE?"RESOURCEDISPLAYTYPE_SHARE":resource[i].dwDisplayType==RESOURCEDISPLAYTYPE_GENERIC?"RESOURCEDISPLAYTYPE_GENERIC":resource[i].dwDisplayType==6?"RESOURCEDISPLAYTYPE_NETWORK":resource[i].dwDisplayType==7?"RESOURCEDISPLAYTYPE_ROOT":resource[i].dwDisplayType==8?"RESOURCEDISPLAYTYPE_SHAREADMIN":resource[i].dwDisplayType==9?"RESOURCEDISPLAYTYPE_DIRECTORY":resource[i].dwDisplayType==10?"RESOURCEDISPLAYTYPE_TREE":resource[i].dwDisplayType==11?"RESOURCEDISPLAYTYPE_NDSCONTAINER":"?"));
+          FXTRACE((1,"dwUsage=%s\n",resource[i].dwUsage==RESOURCEUSAGE_CONNECTABLE?"RESOURCEUSAGE_CONNECTABLE":resource[i].dwUsage==RESOURCEUSAGE_CONTAINER?"RESOURCEUSAGE_CONTAINER":"?"));
+          FXTRACE((1,"lpLocalName=%s\n",resource[i].lpLocalName));
+          FXTRACE((1,"lpRemoteName=%s\n",resource[i].lpRemoteName));
+          FXTRACE((1,"lpComment=%s\n",resource[i].lpComment));
+          FXTRACE((1,"lpProvider=%s\n\n",resource[i].lpProvider));
+
           // Grow list
           if(count+1>=size){
             size=size?(size<<1):256;
@@ -1080,32 +1214,44 @@ FXint FXFile::listFiles(FXString*& list,const FXString& path,const FXString& pat
             }
 
           // Add remote name to list
-          list[count++]=resource[i].lpRemoteName;
+          list[count]=resource[i].lpRemoteName;
+          count++;
           }
         }
       WNetCloseEnum(hEnum);
       }
     return count;
     }
+*/
+  // Folding case
+  if(flags&LIST_CASEFOLD) matchmode|=FILEMATCH_CASEFOLD;
 
-  // Open directory 
-  hFindFile=FindFirstFile((path+PATHSEPSTRING "*").text(),&ffData);
+  // Copy directory name
+  strcpy(pathname,path.text());
+  pathtail=pathname+strlen(pathname)-1;
+  if(!ISPATHSEP(*pathtail)) *++pathtail=PATHSEP;
+  ++pathtail;
+
+  // Open directory
+  strcpy(pathtail,"*");
+  hFindFile=FindFirstFile(pathname,&ffData);
   if(hFindFile==INVALID_HANDLE_VALUE) return 0;
 
   // Loop over directory entries
   do{
     name=ffData.cFileName;
 
-    // Skip '.' and '..'
-    //if(name[0]=='.' && (name[1]==0 || (name[1]=='.' && name[2]==0))) continue;
-    // Is it a directory or does it match the pattern?
-    if(!fxfilematch(pattern.text(),name,flags)) continue;
+    // Filter out files; a bit tricky...
+    if(!(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) && ((flags&LIST_NO_FILES) || ((ffData.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN) && !(flags&LIST_HIDDEN_FILES)) || (!(flags&LIST_ALL_FILES) && !fxfilematch(pattern.text(),name,matchmode)))) continue;
+
+    // Filter out directories; even more tricky!
+    if((ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) && ((flags&LIST_NO_DIRS) || ((ffData.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN) && !(flags&LIST_HIDDEN_DIRS)) || (name[0]=='.' && (name[1]==0 || (name[1]=='.' && name[2]==0 && (flags&LIST_NO_PARENT)))) || (!(flags&LIST_ALL_DIRS) && !fxfilematch(pattern.text(),name,matchmode)))) continue;
 
     // Grow list
     if(count+1>=size){
       size=size?(size<<1):256;
       newlist=new FXString[size];
-      for(i=0; i<count; i++) newlist[i]=list[i];
+      for(int f=0; f<count; f++) newlist[f]=list[f];
       delete [] list;
       list=newlist;
       }
@@ -1123,11 +1269,25 @@ FXint FXFile::listFiles(FXString*& list,const FXString& path,const FXString& pat
 
 // Convert file time to string as per strftime format
 FXString FXFile::time(const FXchar *format,FXTime filetime){
-  time_t tmp=(time_t)filetime;
+#ifndef WIN32
+#ifdef FOX_THREAD_SAFE
+  time_t tmp=(time_t)FXMAX(filetime,0);
+  struct tm tmresult;
   FXchar buffer[512];
-  FXint len;
-  len=strftime(buffer,sizeof(buffer),format,localtime(&tmp));
+  FXint len=strftime(buffer,sizeof(buffer),format,localtime_r(&tmp,&tmresult));
   return FXString(buffer,len);
+#else
+  time_t tmp=(time_t)FXMAX(filetime,0);
+  FXchar buffer[512];
+  FXint len=strftime(buffer,sizeof(buffer),format,localtime(&tmp));
+  return FXString(buffer,len);
+#endif
+#else
+  time_t tmp=(time_t)FXMAX(filetime,0);
+  FXchar buffer[512];
+  FXint len=strftime(buffer,sizeof(buffer),format,localtime(&tmp));
+  return FXString(buffer,len);
+#endif
   }
 
 
@@ -1144,11 +1304,11 @@ FXTime FXFile::now(){
 
 
 // Get file info
-FXbool FXFile::info(const FXString& file,struct stat& info){
+FXbool FXFile::info(const FXString& file,struct stat& inf){
 #ifndef WIN32
-  return !file.empty() && (::stat(file.text(),&info)==0);
+  return !file.empty() && (::stat(file.text(),&inf)==0);
 #else
-  return !file.empty() && (::stat(file.text(),&info)==0);
+  return !file.empty() && (::stat(file.text(),&inf)==0);
 #endif
   }
 
@@ -1257,12 +1417,6 @@ FXString FXFile::enquote(const FXString& file,FXbool forcequotes){
   register FXint i,c;
   for(i=0; (c=file[i])!='\0'; i++){
     switch(c){
-      case '"':               // Quote needs to be escaped
-        result+="\\\"";
-        break;
-      case '\\':              // Backspace needs to be escaped, of course
-        result+="\\\\";
-        break;
       case '<':               // Redirections
       case '>':
       case '|':
@@ -1284,21 +1438,19 @@ FXString FXFile::enquote(const FXString& file,FXbool forcequotes){
 
 // Decode filename to get original again
 FXString FXFile::dequote(const FXString& file){
-  FXString result;
   register FXint i,c;
+  FXString result;
   i=0;
   while((c=file[i])!='\0' && isspace((FXuchar)c)) i++;
   if(file[i]=='"'){
     i++;
     while((c=file[i])!='\0' && c!='"'){
-      if(c=='\\' && file[i+1]!='\0') c=file[++i];
       result+=c;
       i++;
       }
     }
   else{
     while((c=file[i])!='\0' && !isspace((FXuchar)c)){
-      if(c=='\\' && file[i+1]!='\0') c=file[++i];
       result+=c;
       i++;
       }
@@ -1595,7 +1747,12 @@ static FXbool copydir(const FXString& oldfile,const FXString& newfile,FXbool ove
   inode.next=inodes;
 
   // Copy stuff
+#ifdef FOX_THREAD_SAFE
+  struct fxdirent dirresult;
+  while(readdir_r(dirp,&dirresult,&dp)==0 && dp){
+#else
   while((dp=readdir(dirp))!=NULL){
+#endif
     if(dp->d_name[0]!='.' || (dp->d_name[1]!='\0' && (dp->d_name[1]!='.' || dp->d_name[2]!='\0'))){
       oldchild=oldfile;
       if(!ISPATHSEP(oldchild[oldchild.length()-1])) oldchild.append(PATHSEP);
@@ -1696,7 +1853,7 @@ FXbool FXFile::concatenate(const FXString& srcfile1,const FXString& srcfile2,con
           if(!WriteFile(dst,buffer,nread,&nwritten,NULL)) goto err;
           }
         while(1){
-          if(!ReadFile(src1,buffer,sizeof(buffer),&nread,NULL)) goto err;
+          if(!ReadFile(src2,buffer,sizeof(buffer),&nread,NULL)) goto err;
           if(nread==0) break;
           if(!WriteFile(dst,buffer,nread,&nwritten,NULL)) goto err;
           }
@@ -1769,7 +1926,7 @@ static FXbool copyrec(const FXString& oldfile,const FXString& newfile,FXbool ove
     if(!(atts2&FILE_ATTRIBUTE_DIRECTORY)){
       if(!overwrite) return FALSE;
       FXTRACE((100,"DeleteFile(%s)\n",newfile.text()));
-      if(DeleteFile(newfile.text())!=0) return FALSE;
+      if(DeleteFile(newfile.text())==0) return FALSE;
       }
     }
 
@@ -1815,7 +1972,12 @@ FXbool FXFile::remove(const FXString& file){
       if(dirp){
         struct dirent *dp;
         FXString child;
-        while((dp=::readdir(dirp))!=NULL){
+#ifdef FOX_THREAD_SAFE
+        struct fxdirent dirresult;
+        while(readdir_r(dirp,&dirresult,&dp)==0 && dp){
+#else
+        while((dp=readdir(dirp))!=NULL){
+#endif
           if(dp->d_name[0]!='.' || (dp->d_name[1]!='\0' && (dp->d_name[1]!='.' || dp->d_name[2]!='\0'))){
             child=file;
             if(!ISPATHSEP(child[child.length()-1])) child.append(PATHSEP);
@@ -1918,17 +2080,21 @@ FXbool FXFile::link(const FXString& oldfile,const FXString& newfile,FXbool overw
     FXTRACE((100,"link(%s,%s)\n",oldfile.text(),newfile.text()));
     return ::link(oldfile.text(),newfile.text())==0;
 #else
-#if (_WIN32_WINNT >=0x0500)   // WinNT, Win2K, and later
+    typedef BOOL (WINAPI *PFN_CHL)(LPCTSTR,LPCTSTR,LPSECURITY_ATTRIBUTES);
+    static PFN_CHL chl=NULL;
+    if(!chl){
+      HMODULE hkernel=LoadLibraryA("Kernel32");
+      if(!hkernel) return FALSE;
+      chl=(PFN_CHL)::GetProcAddress(hkernel,"CreateHardLinkA");
+      FreeLibrary(hkernel);
+      }
     if(!FXFile::exists(oldfile)) return FALSE;
     if(FXFile::exists(newfile)){
       if(!overwrite) return FALSE;
       if(!FXFile::remove(newfile)) return FALSE;
       }
     FXTRACE((100,"CreateHardLink(%s,%s)\n",oldfile.text(),newfile.text()));
-    return ::CreateHardLink(newfile.text(),oldfile.text(),NULL)!=0;
-#else
-    return FALSE;
-#endif
+    return chl && (*chl)(newfile.text(),oldfile.text(),NULL)!=0;
 #endif
     }
   return FALSE;
