@@ -3,7 +3,7 @@
 *                 F i l e  - A s s o c i a t i o n   T a b l e                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFileDict.cpp,v 1.27 2001/07/17 21:59:28 jeroen Exp $                   *
+* $Id: FXFileDict.cpp,v 1.33.4.1 2003/05/21 15:09:58 fox Exp $                   *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,22 +30,25 @@
 #include "FXPoint.h"
 #include "FXRectangle.h"
 #include "FXFile.h"
-#include "FXObject.h"
-#include "FXDict.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDrawable.h"
-#include "FXImage.h"
+#include "FXFileDict.h"
 #include "FXIcon.h"
 #include "FXGIFIcon.h"
 #include "FXBMPIcon.h"
-#include "FXJPEGIcon.h"
-#include "FXPNGIcon.h"
 #include "FXXPMIcon.h"
+
+// Additional image formats may be turned on
+#ifdef ALL_IMAGE_FORMATS
 #include "FXPCXIcon.h"
-#include "FXFileDict.h"
+#include "FXTGAIcon.h"
+#include "FXICOIcon.h"
+#include "FXRGBIcon.h"
+#include "FXJPGIcon.h"
+#include "FXPNGIcon.h"
+#include "FXTIFIcon.h"
+#endif
 
 
 /*
@@ -80,10 +83,10 @@
     file type detection [I'm not a big fan of this, but it may sometimes
     be necessary].
 
-  - Refer to RFC 2045, 2046, 2047, 2048, and 2077.  
+  - Refer to RFC 2045, 2046, 2047, 2048, and 2077.
     The Internet media type registry is at:
     ftp://ftp.iana.org/in-notes/iana/assignments/media-types/
-  
+
   - We should at least organize things so that enough info is passed in
     so we can read a fragment of the content and do it (so that means
     the full pathname, and perhaps some flags).
@@ -93,12 +96,12 @@
     command string ';' extension string ';' bigicon [ ':' bigiconopen ] ';' miniicon [ ':' miniiconopen ] ';' mimetype
 
   - An empty binding like:
-  
+
       ext=""
-    
+
     Can be used to override a global binding with an empty one, i.e. it is
     as if no binding exists; you will get the default or fallback in this case.
-  
+
   - Obtaining icons for files, folders, executable, etc. under Windows:
 
     1) Get icon sizes:
@@ -170,8 +173,12 @@
 FXIMPLEMENT(FXIconDict,FXDict,NULL,0)
 
 
+// Default icon path
+const FXchar FXIconDict::defaultIconPath[]=DEFAULTICONPATH;
+
+
 // Build icon table
-FXIconDict::FXIconDict(FXApp* a):app(a){
+FXIconDict::FXIconDict(FXApp* a,const FXString& p):app(a),path(p){
   FXTRACE((100,"FXIconDict::FXIconDict\n"));
   }
 
@@ -200,19 +207,35 @@ void *FXIconDict::createData(const void* ptr){
     else if(comparecase(".xpm",ext)==0){
       icon=new FXXPMIcon(getApp());
       }
+#ifdef ALL_IMAGE_FORMATS
     else if(comparecase(".pcx",ext)==0){
       icon=new FXPCXIcon(getApp());
       }
-// #ifdef HAVE_JPEG_H
-//     else if(comparecase(".jpg",ext)==0){
-//       icon=new FXJPEGIcon(getApp());
-//       }
-// #endif
-// #ifdef HAVE_PNG_H
-//     else if(comparecase(".png",ext)==0){
-//       icon=new FXPNGIcon(getApp());
-//       }
-// #endif
+    else if(comparecase(".ico",ext)==0){
+      icon=new FXICOIcon(getApp());
+      }
+    else if(comparecase(".tga",ext)==0){
+      icon=new FXTGAIcon(getApp());
+      }
+    else if(comparecase(".rgb",ext)==0){
+      icon=new FXRGBIcon(getApp());
+      }
+#ifdef HAVE_JPEG_H
+    else if(comparecase(".jpg",ext)==0){
+      icon=new FXJPGIcon(getApp());
+      }
+#endif
+#ifdef HAVE_PNG_H
+    else if(comparecase(".png",ext)==0){
+      icon=new FXPNGIcon(getApp());
+      }
+#endif
+#ifdef HAVE_TIFF_H
+    else if(comparecase(".tif",ext)==0){
+      icon=new FXTIFIcon(getApp());
+      }
+#endif
+#endif
 
     // Got icon
     if(icon){
@@ -291,16 +314,16 @@ FXIMPLEMENT(FXFileDict,FXDict,NULL,0)
 
 
 // Construct an file-extension association table
-FXFileDict::FXFileDict(FXApp* a):app(a),settings(&a->reg()),icons(a){
+FXFileDict::FXFileDict(FXApp* a):app(a),settings(&a->reg()){
   FXTRACE((100,"FXFileDict::FXFileDict\n"));
-  icons.setIconPath(settings->readStringEntry("SETTINGS","iconpath",DEFAULTICONPATH));
+  icons=new FXIconDict(a,settings->readStringEntry("SETTINGS","iconpath",FXIconDict::defaultIconPath));
   }
 
 
-// Construct an file-extension association table, and alternative 
-FXFileDict::FXFileDict(FXApp* a,FXSettings* db):app(a),settings(db),icons(a){
+// Construct an file-extension association table, and alternative settings database
+FXFileDict::FXFileDict(FXApp* a,FXSettings* db):app(a),settings(db){
   FXTRACE((100,"FXFileDict::FXFileDict\n"));
-  icons.setIconPath(settings->readStringEntry("SETTINGS","iconpath",DEFAULTICONPATH));
+  icons=new FXIconDict(a,settings->readStringEntry("SETTINGS","iconpath",FXIconDict::defaultIconPath));
   }
 
 
@@ -325,37 +348,37 @@ void *FXFileDict::createData(const void* ptr){
   // Scan entry
   p=(FXchar*)ptr;
   q=command;
-  while(*p && *p!=';' && (p<&command[COMMANDLEN-1])) *q++=*p++;
+  while(*p && *p!=';' && (q<&command[COMMANDLEN-1])) *q++=*p++;
   *q='\0';
   if(*p==';') p++;
 
   q=extension;
-  while(*p && *p!=';' && (p<&command[EXTENSIONLEN-1])) *q++=*p++;
+  while(*p && *p!=';' && (q<&command[EXTENSIONLEN-1])) *q++=*p++;
   *q='\0';
   if(*p==';') p++;
 
   q=bigname;
-  while(*p && *p!=';' && *p!=':' && (p<&command[ICONNAMELEN-1])) *q++=*p++;
+  while(*p && *p!=';' && *p!=':' && (q<&command[ICONNAMELEN-1])) *q++=*p++;
   *q='\0';
   if(*p==':') p++;
 
   q=bignameopen;
-  while(*p && *p!=';' && (p<&command[ICONNAMELEN-1])) *q++=*p++;
+  while(*p && *p!=';' && (q<&command[ICONNAMELEN-1])) *q++=*p++;
   *q='\0';
   if(*p==';') p++;
 
   q=mininame;
-  while(*p && *p!=';' && *p!=':' && (p<&command[ICONNAMELEN-1])) *q++=*p++;
+  while(*p && *p!=';' && *p!=':' && (q<&command[ICONNAMELEN-1])) *q++=*p++;
   *q='\0';
   if(*p==':') p++;
 
   q=mininameopen;
-  while(*p && *p!=';' && (p<&command[ICONNAMELEN-1])) *q++=*p++;
+  while(*p && *p!=';' && (q<&command[ICONNAMELEN-1])) *q++=*p++;
   *q='\0';
   if(*p==';') p++;
 
   q=mimetype;
-  while(*p && *p!=';' && (p<&command[MIMETYPELEN-1])) *q++=*p++;
+  while(*p && *p!=';' && (q<&command[MIMETYPELEN-1])) *q++=*p++;
   *q='\0';
 
   FXTRACE((300,"FXFileDict: command=\"%s\" extension=\"%s\" mimetype=\"%s\" big=\"%s\"|\"%s\" mini=\"%s\"|\"%s\"\n",command,extension,mimetype,bigname,bignameopen,mininame,mininameopen));
@@ -372,12 +395,12 @@ void *FXFileDict::createData(const void* ptr){
   fileassoc->flags=0;
 
   // Insert icons into icon dictionary
-  if(bigname[0]){ fileassoc->bigicon=fileassoc->bigiconopen=icons.insert(bigname); }
-  if(mininame[0]){ fileassoc->miniicon=fileassoc->miniiconopen=icons.insert(mininame); }
+  if(bigname[0]){ fileassoc->bigicon=fileassoc->bigiconopen=icons->insert(bigname); }
+  if(mininame[0]){ fileassoc->miniicon=fileassoc->miniiconopen=icons->insert(mininame); }
 
   // Add open icons also; we will fall back on the regular icons in needed
-  if(bignameopen[0]){ fileassoc->bigiconopen=icons.insert(bignameopen); }
-  if(mininameopen[0]){ fileassoc->miniiconopen=icons.insert(mininameopen); }
+  if(bignameopen[0]){ fileassoc->bigiconopen=icons->insert(bignameopen); }
+  if(mininameopen[0]){ fileassoc->miniiconopen=icons->insert(mininameopen); }
 
   // Return the binding
   return fileassoc;
@@ -397,13 +420,13 @@ void FXFileDict::setIconPath(const FXString& path){
   settings->writeStringEntry("SETTINGS","iconpath",path.text());
 
   // Change it in icon dictionary
-  icons.setIconPath(path);
+  icons->setIconPath(path);
   }
 
 
 // Return current icon search path
 FXString FXFileDict::getIconPath() const {
-  return icons.getIconPath();
+  return icons->getIconPath();
   }
 
 
@@ -521,7 +544,7 @@ FXFileAssoc* FXFileDict::findExecBinding(const FXchar* pathname){
 void FXFileDict::save(FXStream& store) const {
   FXDict::save(store);
   store << app;
-  icons.save(store);
+  store << icons;
   }
 
 
@@ -529,13 +552,15 @@ void FXFileDict::save(FXStream& store) const {
 void FXFileDict::load(FXStream& store){
   FXDict::load(store);
   store >> app;
-  icons.load(store);
+  store >> icons;
   }
 
 
 // Destructor
 FXFileDict::~FXFileDict(){
   FXTRACE((100,"FXFileDict::~FXFileDict\n"));
-  app=(FXApp*)-1;
+  delete icons;
   clear();
+  app=(FXApp*)-1;
+  icons=(FXIconDict*)-1;
   }
