@@ -3,7 +3,7 @@
 *                        F i l e    L i s t   O b j e c t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFileList.cpp,v 1.71 2001/06/28 06:12:56 jeroen Exp $                   *
+* $Id: FXFileList.cpp,v 1.89.4.13 2003/06/20 19:02:07 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,24 +30,15 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
 #include "FXFile.h"
 #include "FXURL.h"
 #include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXApp.h"
-#include "FXId.h"
 #include "FXFont.h"
-#include "FXDrawable.h"
-#include "FXImage.h"
 #include "FXIcon.h"
 #include "FXGIFIcon.h"
-#include "FXWindow.h"
-#include "FXFrame.h"
-#include "FXComposite.h"
 #include "FXScrollbar.h"
-#include "FXScrollArea.h"
 #include "FXFileDict.h"
 #include "FXHeader.h"
 #include "FXIconList.h"
@@ -70,12 +61,10 @@
     FXFileList...
   - Clipboard of filenames.
   - Clipboard, DND, etc. support.
-  - Current item is index, should be name!
   - When being dragged over, if hovering over a directory item for some
     time we need to open it.
   - We should generate SEL_INSERTED, SEL_DELETED, SEL_REPLACED, SEL_CHANGED
     messages as the FXFileList updates itself from the file system.
-  - Under MS-Windows, always pass FILEMATCH_NOESCAPE setting match mode!
 */
 
 
@@ -270,6 +259,7 @@ FXIMPLEMENT(FXFileList,FXIconList,FXFileListMap,ARRAYNUMBER(FXFileListMap))
 
 // For serialization
 FXFileList::FXFileList(){
+  flags|=FLAG_ENABLED|FLAG_DROPTARGET;
   refreshtimer=NULL;
   opentimer=NULL;
   timestamp=0;
@@ -282,6 +272,7 @@ FXFileList::FXFileList(){
 // File List
 FXFileList::FXFileList(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXIconList(p,tgt,sel,opts,x,y,w,h),directory(PATHSEPSTRING),orgdirectory(PATHSEPSTRING),pattern("*"){
+  flags|=FLAG_ENABLED|FLAG_DROPTARGET;
   associations=NULL;
   appendHeader("Name",NULL,200);
   appendHeader("Type",NULL,100);
@@ -323,7 +314,6 @@ void FXFileList::create(){
   mini_app->create();
   listDirectory();    // Do we have to do this here?
   sortItems();
-  dropEnable();
   }
 
 
@@ -363,7 +353,6 @@ void FXFileList::destroy(){
 // Open up folder when howvering long over a folder
 long FXFileList::onOpenTimer(FXObject*,FXSelector,void*){
   FXint xx,yy,index; FXuint state;
-  FXTRACE((100,"FXFileList::onOpenTimer\n"));
   opentimer=NULL;
   getCursorPosition(xx,yy,state);
   index=getItemAt(xx,yy);
@@ -708,12 +697,6 @@ long FXFileList::onUpdSetDirectory(FXObject* sender,FXSelector,void*){
 // Sort by name
 long FXFileList::onCmdSortByName(FXObject*,FXSelector,void*){
   sortfunc=(sortfunc==cmpFName) ? cmpRName : cmpFName;
-//  header->setArrowDir(0,sortfunc==cmpFName ? 1 : -1);    // Name
-//   header->setArrowDir(1,0);    // Type
-//   header->setArrowDir(2,0);    // Size
-//   header->setArrowDir(3,0);    // Date
-//   header->setArrowDir(4,0);    // User
-//   header->setArrowDir(5,0);    // Group
   sortItems();
   return 1;
   }
@@ -730,12 +713,6 @@ long FXFileList::onUpdSortByName(FXObject* sender,FXSelector,void* ptr){
 // Sort by type
 long FXFileList::onCmdSortByType(FXObject*,FXSelector,void*){
   sortfunc=(sortfunc==cmpFType) ? cmpRType : cmpFType;
-//   header->setArrowDir(0,0);    // Name
-//   header->setArrowDir(1,sortfunc==cmpFType ? 1 : -1);    // Type
-//   header->setArrowDir(2,0);    // Size
-//   header->setArrowDir(3,0);    // Date
-//   header->setArrowDir(4,0);    // User
-//   header->setArrowDir(5,0);    // Group
   sortItems();
   return 1;
   }
@@ -752,12 +729,6 @@ long FXFileList::onUpdSortByType(FXObject* sender,FXSelector,void* ptr){
 // Sort by size
 long FXFileList::onCmdSortBySize(FXObject*,FXSelector,void*){
   sortfunc=(sortfunc==cmpFSize) ? cmpRSize : cmpFSize;
-//   header->setArrowDir(0,0);    // Name
-//   header->setArrowDir(1,0);    // Type
-//   header->setArrowDir(2,sortfunc==cmpFSize ? 1 : -1);    // Size
-//   header->setArrowDir(3,0);    // Date
-//   header->setArrowDir(4,0);    // User
-//   header->setArrowDir(5,0);    // Group
   sortItems();
   return 1;
   }
@@ -774,12 +745,6 @@ long FXFileList::onUpdSortBySize(FXObject* sender,FXSelector,void* ptr){
 // Sort by time
 long FXFileList::onCmdSortByTime(FXObject*,FXSelector,void*){
   sortfunc=(sortfunc==cmpFTime) ? cmpRTime : cmpFTime;
-//   header->setArrowDir(0,0);    // Name
-//   header->setArrowDir(1,0);    // Type
-//   header->setArrowDir(2,0);    // Size
-//   header->setArrowDir(3,sortfunc==cmpFTime ? 1 : -1);    // Date
-//   header->setArrowDir(4,0);    // User
-//   header->setArrowDir(5,0);    // Group
   sortItems();
   return 1;
   }
@@ -796,12 +761,6 @@ long FXFileList::onUpdSortByTime(FXObject* sender,FXSelector,void* ptr){
 // Sort by user
 long FXFileList::onCmdSortByUser(FXObject*,FXSelector,void*){
   sortfunc=(sortfunc==cmpFUser) ? cmpRUser : cmpFUser;
-//   header->setArrowDir(0,0);    // Name
-//   header->setArrowDir(1,0);    // Type
-//   header->setArrowDir(2,0);    // Size
-//   header->setArrowDir(3,0);    // Date
-//   header->setArrowDir(4,sortfunc==cmpFUser ? 1 : -1);    // User
-//   header->setArrowDir(5,0);    // Group
   sortItems();
   return 1;
   }
@@ -818,12 +777,6 @@ long FXFileList::onUpdSortByUser(FXObject* sender,FXSelector,void* ptr){
 // Sort by group
 long FXFileList::onCmdSortByGroup(FXObject*,FXSelector,void*){
   sortfunc=(sortfunc==cmpFGroup) ? cmpRGroup : cmpFGroup;
-//   header->setArrowDir(0,0);    // Name
-//   header->setArrowDir(1,0);    // Type
-//   header->setArrowDir(2,0);    // Size
-//   header->setArrowDir(3,0);    // Date
-//   header->setArrowDir(4,0);    // User
-//   header->setArrowDir(5,sortfunc==cmpFGroup ? 1 : -1);    // Group
   sortItems();
   return 1;
   }
@@ -872,7 +825,7 @@ long FXFileList::onUpdSortReverse(FXObject* sender,FXSelector,void* ptr){
 
 // Clicked header button
 long FXFileList::onCmdHeader(FXObject*,FXSelector,void* ptr){
-  if(((FXuint)(long)ptr)<6) handle(this,MKUINT((ID_SORT_BY_NAME+(FXuint)(long)ptr),SEL_COMMAND),NULL);
+  if(((FXuint)(FXuval)ptr)<6) handle(this,MKUINT((ID_SORT_BY_NAME+(FXuint)(FXuval)ptr),SEL_COMMAND),NULL);
   return 1;
   }
 
@@ -907,7 +860,7 @@ FXint FXFileList::cmpFName(const FXIconItem* pa,const FXIconItem* pb){
     if(tolower(*p) > tolower(*q)) return 1;
     if(tolower(*p) < tolower(*q)) return -1;
 #endif
-    if(*p<='\t') return 0;
+    if(*p<='\t') break;
     p++;
     q++;
     }
@@ -931,11 +884,11 @@ FXint FXFileList::cmpFType(const FXIconItem* pa,const FXIconItem* pb){
     if(tolower(*p) > tolower(*q)) return 1;
     if(tolower(*p) < tolower(*q)) return -1;
 #endif
-    if(*p<='\t') return 0;
+    if(*p<='\t') break;
     p++;
     q++;
     }
-  return 0;
+  return cmpFName(pa,pb);
   }
 
 
@@ -945,7 +898,9 @@ FXint FXFileList::cmpFSize(const FXIconItem* pa,const FXIconItem* pb){
   register const FXFileItem *b=(FXFileItem*)pb;
   register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
   if(diff) return diff;
-  return (long)a->size - (long)b->size;
+  register long l=(long)a->size - (long)b->size;
+  if(l) return l;
+  return cmpFName(pa,pb);
   }
 
 
@@ -955,7 +910,9 @@ FXint FXFileList::cmpFTime(const FXIconItem* pa,const FXIconItem* pb){
   register const FXFileItem *b=(FXFileItem*)pb;
   register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
   if(diff) return diff;
-  return (long)a->date - (long)b->date;
+  register long l=(long)a->date - (long)b->date;
+  if(l) return l;
+  return cmpFName(pa, pb);
   }
 
 
@@ -977,11 +934,11 @@ FXint FXFileList::cmpFUser(const FXIconItem* pa,const FXIconItem* pb){
     if(tolower(*p) > tolower(*q)) return 1;
     if(tolower(*p) < tolower(*q)) return -1;
 #endif
-    if(*p<='\t') return 0;
+    if(*p<='\t') break;
     p++;
     q++;
     }
-  return 0;
+  return cmpFName(pa,pb);
   }
 
 
@@ -1003,13 +960,14 @@ FXint FXFileList::cmpFGroup(const FXIconItem* pa,const FXIconItem* pb){
     if(tolower(*p) > tolower(*q)) return 1;
     if(tolower(*p) < tolower(*q)) return -1;
 #endif
-    if(*p<='\t') return 0;
+    if(*p<='\t') break;
     p++;
     q++;
     }
-  return 0;
+  return cmpFName(pa,pb);
   }
 
+
 // Reversed compare file name
 FXint FXFileList::cmpRName(const FXIconItem* pa,const FXIconItem* pb){
   return -FXFileList::cmpFName(pa,pb);
@@ -1049,6 +1007,18 @@ FXint FXFileList::cmpRGroup(const FXIconItem* pa,const FXIconItem* pb){
 /*******************************************************************************/
 
 
+
+//HANDLE FindFirstChangeNotification(
+// LPCTSTR lpPathName,  // directory name
+// BOOL bWatchSubtree,  // monitoring option
+// DWORD dwNotifyFilter  // filter conditions
+//);
+//
+//The HANDLE can be passed to FXApp::addInput(), and you'll be notified
+//for all events as specified in dwNotifyFilter.
+
+
+
 // Got refresh timeout
 long FXFileList::onRefreshTimer(FXObject*,FXSelector,void*){
   FXuint interval=REFRESHINTERVAL;
@@ -1095,6 +1065,8 @@ void FXFileList::setCurrentFile(const FXString& pathname){    // FIXME notify ar
     FXTRACE((100,"%s::setCurrentFile(%s)\n",getClassName(),pathname.text()));
     setDirectory(FXFile::directory(pathname));
     setCurrentItem(findItem(FXFile::name(pathname)));
+    setAnchorItem(getCurrentItem());
+    if(0<=getCurrentItem()) selectItem(getCurrentItem());
     }
   }
 
@@ -1213,25 +1185,27 @@ FXIconItem *FXFileList::createItem(const FXString& text,FXIcon *big,FXIcon* mini
 
 #ifndef WIN32
 
+
 // List directory
 void FXFileList::listDirectory(){
-  FXchar grpid[64],usrid[64],atts[16],mod[32];
+  FXchar grpid[64],usrid[64],atts[16],mod[32],ext[128];
   FXchar pathname[MAXPATHLEN];
+  FXFileItem *curitem=NULL;
   FXFileItem *item;
-  FXchar *pathtail,*name;
+  FXchar *pathtail,*name,*qq,*pp;
   FXFileAssoc *fileassoc;
   const FXchar *extension;
   FXIcon *big,*mini;
   FXFileItem **hashlist;
   int nhashlist,i,x,p,h;
-  long filetime,modtime;
+  time_t filetime;
   struct stat info;
   struct dirent *dp;
   DIR *dirp;
   int islink;
 
-  // Unfocus
-  if(hasFocus() && 0<=current){ ((FXFileItem*)items[current])->state&=~FXFileItem::FOCUS; }
+  // Remember current item
+  if(0<=current){ curitem=(FXFileItem*)items[current]; }
 
   // Place existing items into a hash table
   for(nhashlist=32; nhashlist<=nitems; nhashlist<<=1);
@@ -1250,7 +1224,7 @@ void FXFileList::listDirectory(){
   if(stat(directory.text(),&info)==0){
 
     // Need latest change no matter what actually changed!
-    timestamp=(FXTime)FXMAX(info.st_mtime,info.st_ctime);
+    timestamp=FXMAX(info.st_mtime,info.st_ctime);
 
     // Set path to stat with
     strcpy(pathname,directory.text());
@@ -1267,7 +1241,12 @@ void FXFileList::listDirectory(){
     if(dirp){
 
       // Loop over directory entries
+#ifdef FOX_THREAD_SAFE
+      struct fxdirent dirresult;
+      while(readdir_r(dirp,&dirresult,&dp)==0 && dp){
+#else
       while((dp=readdir(dirp))!=NULL){
+#endif
         name=dp->d_name;
 
         // Hidden file (.xxx) or directory (. or .yyy) normally not shown,
@@ -1291,10 +1270,8 @@ void FXFileList::listDirectory(){
         if(!S_ISDIR(info.st_mode) && !fxfilematch(pattern.text(),name,matchmode)) continue;
 
         // Anything about file has changed
-        filetime=FXMAX(info.st_mtime,info.st_ctime);
-
-        // File mod time
-        modtime=info.st_mtime;
+        filetime=info.st_mtime;
+        if(filetime<0) filetime=0;
 
         // Take item from hash table if found
         h=hash(name);
@@ -1311,83 +1288,87 @@ void FXFileList::listDirectory(){
 
         // Append item in list
 fnd:    FXRESIZE(&items,FXIconItem*,nitems+1);
+        if(item==curitem) current=nitems;
         items[nitems]=item;
         nitems++;
 
-        // Anything about the file changed?
-        if((item->date!=modtime) || (timestamp<filetime) || (filetime==0)){
+        // Obtain user name
+        fxgetusername(usrid,info.st_uid);
 
-          // Obtain user name
-          fxgetusername(usrid,info.st_uid);
+        // Obtain group name
+        fxgetgroupname(grpid,info.st_gid);
 
-          // Obtain group name
-          fxgetgroupname(grpid,info.st_gid);
+        // Permissions
+        fxgetpermissions(atts,info.st_mode);
 
-          // Permissions
-          fxgetpermissions(atts,info.st_mode);
+        // Mod time
+        strftime(mod,sizeof(mod),TIMEFORMAT,localtime(&filetime));
 
-          // Mod time
-          strftime(mod,sizeof(mod),TIMEFORMAT,localtime(&info.st_mtime));
-
-          // Item flags
-          if(S_ISDIR(info.st_mode)) item->state|=FXFileItem::FOLDER; else item->state&=~FXFileItem::FOLDER;
-          if(S_ISLNK(info.st_mode)) item->state|=FXFileItem::SYMLINK; else item->state&=~FXFileItem::SYMLINK;
-          if(S_ISCHR(info.st_mode)) item->state|=FXFileItem::CHARDEV; else item->state&=~FXFileItem::CHARDEV;
-          if(S_ISBLK(info.st_mode)) item->state|=FXFileItem::BLOCKDEV; else item->state&=~FXFileItem::BLOCKDEV;
-          if(S_ISFIFO(info.st_mode)) item->state|=FXFileItem::FIFO; else item->state&=~FXFileItem::FIFO;
-          if(S_ISSOCK(info.st_mode)) item->state|=FXFileItem::SOCK; else item->state&=~FXFileItem::SOCK;
-          if((info.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH)) && !(S_ISDIR(info.st_mode)||S_ISCHR(info.st_mode)||S_ISBLK(info.st_mode)||S_ISFIFO(info.st_mode)||S_ISSOCK(info.st_mode))){
-            item->state|=FXFileItem::EXECUTABLE;
-            }
-          else{
-            item->state&=~FXFileItem::EXECUTABLE;
-            }
-
-          // We can drag items
-          item->state|=FXFileItem::DRAGGABLE;
-
-          // Assume no associations
-          fileassoc=NULL;
-
-          // Determine icons and type
-          if(item->state&FXFileItem::FOLDER){
-            big=big_folder;
-            mini=mini_folder;
-            extension="Folder";
-            if(associations) fileassoc=associations->findDirBinding(pathname);
-            }
-          else if(item->state&FXFileItem::EXECUTABLE){
-            big=big_app;
-            mini=mini_app;
-            extension="Application";
-            if(associations) fileassoc=associations->findExecBinding(pathname);
-            }
-          else{
-            big=big_doc;
-            mini=mini_doc;
-            extension="Document";
-            if(associations) fileassoc=associations->findFileBinding(pathname);
-            }
-
-          // If association is found, use it
-          if(fileassoc){
-            extension=fileassoc->extension.text();
-            if(fileassoc->bigicon) big=fileassoc->bigicon;
-            if(fileassoc->miniicon) mini=fileassoc->miniicon;
-            // FIXME use open icons also, useful when dragging over directory
-            }
-
-          // Update item information
-          item->label.format("%s\t%s\t%lu\t%s\t%s\t%s\t%s",name,extension,(unsigned long)info.st_size,mod,usrid,grpid,atts);
-          item->bigIcon=big;
-          item->miniIcon=mini;
-          item->size=(unsigned long)info.st_size;
-          item->assoc=fileassoc;
-          item->date=modtime;
-
-          // Create item
-          if(id()) item->create();
+        // Item flags
+        if(S_ISDIR(info.st_mode)) item->state|=FXFileItem::FOLDER; else item->state&=~FXFileItem::FOLDER;
+        if(S_ISLNK(info.st_mode)) item->state|=FXFileItem::SYMLINK; else item->state&=~FXFileItem::SYMLINK;
+        if(S_ISCHR(info.st_mode)) item->state|=FXFileItem::CHARDEV; else item->state&=~FXFileItem::CHARDEV;
+        if(S_ISBLK(info.st_mode)) item->state|=FXFileItem::BLOCKDEV; else item->state&=~FXFileItem::BLOCKDEV;
+        if(S_ISFIFO(info.st_mode)) item->state|=FXFileItem::FIFO; else item->state&=~FXFileItem::FIFO;
+        if(S_ISSOCK(info.st_mode)) item->state|=FXFileItem::SOCK; else item->state&=~FXFileItem::SOCK;
+        if((info.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH)) && !(S_ISDIR(info.st_mode)||S_ISCHR(info.st_mode)||S_ISBLK(info.st_mode)||S_ISFIFO(info.st_mode)||S_ISSOCK(info.st_mode))){
+          item->state|=FXFileItem::EXECUTABLE;
           }
+        else{
+          item->state&=~FXFileItem::EXECUTABLE;
+          }
+
+        // We can drag items
+        item->state|=FXFileItem::DRAGGABLE;
+
+        // Assume no associations
+        fileassoc=NULL;
+
+        // Determine icons and type
+        if(item->state&FXFileItem::FOLDER){
+          big=big_folder;
+          mini=mini_folder;
+          extension="File Folder";
+          if(associations) fileassoc=associations->findDirBinding(pathname);
+          }
+        else if(item->state&FXFileItem::EXECUTABLE){
+          big=big_app;
+          mini=mini_app;
+          extension="Application";
+          if(associations) fileassoc=associations->findExecBinding(pathname);
+          }
+        else{
+          big=big_doc;
+          mini=mini_doc;
+          extension="";
+          qq=strrchr(name,'.');
+          if(qq){
+            qq++;
+            extension=pp=ext;
+            while(*qq){ *pp++=toupper((FXuchar)*qq++); }
+            pp[0]=' '; pp[1]='F'; pp[2]='i'; pp[3]='l'; pp[4]='e'; pp[5]='\0';
+            }
+          if(associations) fileassoc=associations->findFileBinding(pathname);
+          }
+
+        // If association is found, use it
+        if(fileassoc){
+          extension=fileassoc->extension.text();
+          if(fileassoc->bigicon) big=fileassoc->bigicon;
+          if(fileassoc->miniicon) mini=fileassoc->miniicon;
+          // FIXME use open icons also, useful when dragging over directory
+          }
+
+        // Update item information
+        item->label.format("%s\t%s\t%lu\t%s\t%s\t%s\t%s",name,extension,(unsigned long)info.st_size,mod,usrid,grpid,atts);
+        item->bigIcon=big;
+        item->miniIcon=mini;
+        item->size=(unsigned long)info.st_size;
+        item->assoc=fileassoc;
+        item->date=filetime;
+
+        // Create item
+        if(id()) item->create();
         }
       closedir(dirp);
       }
@@ -1406,14 +1387,12 @@ fnd:    FXRESIZE(&items,FXIconItem*,nitems+1);
   if(anchor>=nitems) anchor=-1;
   if(extent>=nitems) extent=-1;
 
-  // Refocus
-  if(hasFocus() && 0<=current){ ((FXFileItem*)items[current])->state|=FXFileItem::FOCUS; }
-
   // Gotta recalc size of content
   recalc();
   }
 
 
+
 /********************************************************************************
 *                                   MS-Windows                                  *
 ********************************************************************************/
@@ -1424,7 +1403,7 @@ fnd:    FXRESIZE(&items,FXIconItem*,nitems+1);
 // Convert FILETIME (# 100ns since 01/01/1601) to time_t (# s since 01/01/1970)
 static time_t fxfiletime(const FILETIME& ft){
   FXlong ll=(((FXlong)ft.dwHighDateTime)<<32) | (FXlong)ft.dwLowDateTime;
-#if defined(__CYGWIN__) || defined(__MINGW32__)
+#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__SC__)
   ll=ll-116444736000000000LL;
 #else
   ll=ll-116444736000000000L;
@@ -1436,10 +1415,11 @@ static time_t fxfiletime(const FILETIME& ft){
 
 // List directory
 void FXFileList::listDirectory(){
-  FXchar grpid[64],usrid[64],atts[16],mod[32];
+  FXchar grpid[64],usrid[64],atts[16],mod[32],ext[128];
   FXchar pathname[MAXPATHLEN];
+  FXFileItem *curitem=NULL;
   FXFileItem *item;
-  FXchar *pathtail,*name;
+  FXchar *pathtail,*name,*qq,*pp;
   FXFileAssoc *fileassoc;
   const FXchar *extension;
   FXIcon *big,*mini;
@@ -1449,10 +1429,10 @@ void FXFileList::listDirectory(){
   WIN32_FIND_DATA ffData;
   HANDLE hFindFile;
   SHFILEINFO sfi;
-  time_t filetime,modtime;
+  time_t filetime;
 
-  // Unfocus
-  if(hasFocus() && 0<=current){ ((FXFileItem*)items[current])->state&=~FXFileItem::FOCUS; }
+  // Remember current item
+  if(0<=current){ curitem=(FXFileItem*)items[current]; }
 
   // Place existing items into a hash table
   for(nhashlist=32; nhashlist<=nitems; nhashlist<<=1);
@@ -1509,9 +1489,7 @@ void FXFileList::listDirectory(){
 
         // Convert it
         filetime=fxfiletime(ffData.ftLastWriteTime);
-
-        // Modified time
-        modtime=filetime;
+        if(filetime<0) filetime=0;
 
         // Take item from hash table if found
         h=hash(name);
@@ -1528,92 +1506,95 @@ void FXFileList::listDirectory(){
 
         // Append item in list
 fnd:    FXRESIZE(&items,FXIconItem*,nitems+1);
+        if(item==curitem) current=nitems;
         items[nitems]=item;
         nitems++;
 
-        // Anything about the file changed?
-        if((item->date!=modtime) || (timestamp<filetime) || (filetime==0)){
+        // Obtain user name (no Win95 equivalent?)
+        fxgetusername(usrid,0);
 
-          // Obtain user name (no Win95 equivalent?)
-          fxgetusername(usrid,0);
+        // Obtain group name (no Win95 equivalent?)
+        fxgetgroupname(grpid,0);
 
-          // Obtain group name (no Win95 equivalent?)
-          fxgetgroupname(grpid,0);
+        // Permissions
+        fxgetpermissions(atts,0);
 
-          // Permissions
-          fxgetpermissions(atts,0);
+        // Mod time
+        strftime(mod,sizeof(mod),TIMEFORMAT,localtime(&filetime));
 
-          // Mod time
-          strftime(mod,sizeof(mod),TIMEFORMAT,localtime(&filetime));
-
-          // Is it a directory?
-          if(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY){
-            atts[0]='d';
-            }
-
-          // Check if file is read-only
-          if(ffData.dwFileAttributes&FILE_ATTRIBUTE_READONLY){
-            atts[2]='-'; atts[5]='-'; atts[8]='-';
-            }
-
-          // Is it an executable file?
-          // GetBinaryType() is not supported for Win95/98! Try SHGetFileInfo() instead.
-          if(SHGetFileInfo(pathname,0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)==0){
-            atts[3]='-'; atts[6]='-'; atts[9]='-';
-            item->state&=~FXFileItem::EXECUTABLE;
-            }
-          else{
-            item->state|=FXFileItem::EXECUTABLE;
-            }
-
-          // Flags
-          if(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) item->state|=FXFileItem::FOLDER; else item->state&=~FXFileItem::FOLDER;
-
-          // We can drag items
-          item->state|=FXFileItem::DRAGGABLE;
-
-          // Assume no associations
-          fileassoc=NULL;
-
-          // Determine icons and type
-          if(item->state&FXFileItem::FOLDER){
-            big=big_folder;
-            mini=mini_folder;
-            extension="Folder";
-            if(associations) fileassoc=associations->findDirBinding(pathname);
-            }
-          else if(item->state&FXFileItem::EXECUTABLE){
-            big=big_app;
-            mini=mini_app;
-            extension="Application";
-            if(associations) fileassoc=associations->findExecBinding(pathname);
-            }
-          else{
-            big=big_doc;
-            mini=mini_doc;
-            extension="Document";
-            if(associations) fileassoc=associations->findFileBinding(pathname);
-            }
-
-          // If association is found, use it
-          if(fileassoc){
-            extension=fileassoc->extension.text();
-            if(fileassoc->bigicon) big=fileassoc->bigicon;
-            if(fileassoc->miniicon) mini=fileassoc->miniicon;
-            // FIXME use open icons also, useful when dragging over directory
-            }
-
-          // Update item information
-          item->label.format("%s\t%s\t%ul\t%s\t%s\t%s\t%s",name,extension,(unsigned long)ffData.nFileSizeLow,mod,usrid,grpid,atts);
-          item->bigIcon=big;
-          item->miniIcon=mini;
-          item->size=(unsigned long)ffData.nFileSizeLow;
-          item->assoc=fileassoc;
-          item->date=modtime;
-
-          // Create item
-          if(id()) item->create();
+        // Is it a directory?
+        if(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY){
+          atts[0]='d';
           }
+
+        // Check if file is read-only
+        if(ffData.dwFileAttributes&FILE_ATTRIBUTE_READONLY){
+          atts[2]='-'; atts[5]='-'; atts[8]='-';
+          }
+
+        // Is it an executable file?
+        if(SHGetFileInfo(pathname,0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)==0){
+          atts[3]='-'; atts[6]='-'; atts[9]='-';
+          item->state&=~FXFileItem::EXECUTABLE;
+          }
+        else{
+          item->state|=FXFileItem::EXECUTABLE;
+          }
+
+        // Flags
+        if(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) item->state|=FXFileItem::FOLDER; else item->state&=~FXFileItem::FOLDER;
+
+        // We can drag items
+        item->state|=FXFileItem::DRAGGABLE;
+
+        // Assume no associations
+        fileassoc=NULL;
+
+        // Determine icons and type
+        if(item->state&FXFileItem::FOLDER){
+          big=big_folder;
+          mini=mini_folder;
+          extension="File Folder";
+          if(associations) fileassoc=associations->findDirBinding(pathname);
+          }
+        else if(item->state&FXFileItem::EXECUTABLE){
+          big=big_app;
+          mini=mini_app;
+          extension="Application";
+          if(associations) fileassoc=associations->findExecBinding(pathname);
+          }
+        else{
+          big=big_doc;
+          mini=mini_doc;
+          extension="";
+          qq=strrchr(name,'.');
+          if(qq){
+            qq++;
+            extension=pp=ext;
+            while(*qq){ *pp++=toupper((FXuchar)*qq++); }
+            pp[0]=' '; pp[1]='F'; pp[2]='i'; pp[3]='l'; pp[4]='e'; pp[5]='\0';
+            }
+          if(associations) fileassoc=associations->findFileBinding(pathname);
+          }
+
+        // If association is found, use it
+        if(fileassoc){
+          extension=fileassoc->extension.text();
+          if(fileassoc->bigicon) big=fileassoc->bigicon;
+          if(fileassoc->miniicon) mini=fileassoc->miniicon;
+          // FIXME use open icons also, useful when dragging over directory
+          }
+
+        // Update item information
+        item->label.format("%s\t%s\t%u\t%s\t%s\t%s\t%s",name,extension,(unsigned)ffData.nFileSizeLow,mod,usrid,grpid,atts);
+        item->bigIcon=big;
+        item->miniIcon=mini;
+        item->size=(unsigned long)ffData.nFileSizeLow;
+        item->assoc=fileassoc;
+        item->date=filetime;
+
+        // Create item
+        if(id()) item->create();
         }
       while(FindNextFile(hFindFile,&ffData));
       FindClose(hFindFile);
@@ -1633,13 +1614,11 @@ fnd:    FXRESIZE(&items,FXIconItem*,nitems+1);
   if(anchor>=nitems) anchor=-1;
   if(extent>=nitems) extent=-1;
 
-  // Refocus
-  if(hasFocus() && 0<=current){ ((FXFileItem*)items[current])->state|=FXFileItem::FOCUS; }
-
   // Gotta recalc size of content
   recalc();
   }
 
+
 #endif
 
 
