@@ -3,7 +3,7 @@
 *                             I m a g e    O b j e c t                          *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXImage.cpp,v 1.60 2001/08/21 22:37:55 jeroen Exp $                      *
+* $Id: FXImage.cpp,v 1.67.4.1 2002/05/13 21:10:32 fox Exp $                      *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -29,16 +29,10 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
-#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXApp.h"
-#include "FXId.h"
 #include "FXVisual.h"
-#include "FXDrawable.h"
 #include "FXImage.h"
-#include "FXDC.h"
 #include "FXDCWindow.h"
 
 
@@ -259,11 +253,11 @@ void FXImage::destroy(){
     if(getApp()->initialized){
       FXTRACE((100,"%s::destroy %p\n",getClassName(),this));
 #ifndef WIN32
-      
+
       // Delete pixmap
       XFreePixmap(DISPLAY(getApp()),xid);
 #else
-      
+
       // Delete bitmap
       DeleteObject(xid);
 #endif
@@ -291,191 +285,192 @@ static inline FXPixel lowbit(FXPixel mask){
 
 // Restore client-side pixel buffer from image
 void FXImage::restore(){
+  if(xid){
+    register FXPixel red,green,blue;
+    register FXPixel red1,green1,blue1;
+    register FXPixel pixel;
+    register FXuint  redshift,greenshift,blueshift;
+    register FXPixel redmask,greenmask,bluemask;
+    register int size,dd,i;
+    register FXbool shmi=FALSE;
+    register XImage *xim=NULL;
+    register Visual *vis;
+    register FXint x,y;
+    register FXuchar *img;
+    register FXuint r,g,b;
+    FXuchar rtab[MAX_MAPSIZE];
+    FXuchar gtab[MAX_MAPSIZE];
+    FXuchar btab[MAX_MAPSIZE];
 #ifdef HAVE_XSHM
-  XShmSegmentInfo shminfo;
-#endif
-  register FXPixel red,green,blue;
-  register FXPixel red1,green1,blue1;
-  register FXPixel pixel;
-  register FXuint  redshift,greenshift,blueshift;
-  register FXPixel redmask,greenmask,bluemask;
-  register int size,dd,i;
-  register FXbool shmi=FALSE;
-  register XImage *xim=NULL;
-  register Visual *vis;
-  register FXint x,y;
-  register FXuchar *img;
-  register FXuint r,g,b;
-  FXuchar rtab[MAX_MAPSIZE];
-  FXuchar gtab[MAX_MAPSIZE];
-  FXuchar btab[MAX_MAPSIZE];
-
-  FXTRACE((100,"%s::restore image %p\n",getClassName(),this));
-
-  // Can not restore before creation
-  if(!xid){ fxerror("%s::restore: trying to restore image before it has been created.\n",getClassName()); }
-
-  // Check for legal size
-  if(width<1 || height<1){ fxerror("%s::restore: illegal image size %dx%d.\n",getClassName(),width,height); }
-
-  // Get Visual
-  vis=(Visual*)visual->visual;
-  dd=visual->getDepth();
-  redmask=vis->red_mask;
-  greenmask=vis->green_mask;
-  bluemask=vis->blue_mask;
-  redshift=findshift(redmask);
-  greenshift=findshift(greenmask);
-  blueshift=findshift(bluemask);
-
-  // Just in case you're on a high-end system
-  FXASSERT(vis->map_entries<=MAX_MAPSIZE);
-
-  // Make array for data if needed
-  if(!data || !(options&IMAGE_OWNED)){
-    size=width*height*channels;
-    FXMALLOC(&data,FXuchar,size);
-    options|=IMAGE_OWNED;
-    }
-
-  // Got local buffer to receive into
-  if(data){
-
-    // Turn it on iff both supported and desired
-#ifdef HAVE_XSHM
-    if(options&IMAGE_SHMI) shmi=getApp()->shmi;
+    XShmSegmentInfo shminfo;
 #endif
 
-  // First try XShm
+    FXTRACE((100,"%s::restore image %p\n",getClassName(),this));
+
+    // Check for legal size
+    if(width<1 || height<1){ fxerror("%s::restore: illegal image size %dx%d.\n",getClassName(),width,height); }
+
+    // Get Visual
+    vis=(Visual*)visual->visual;
+    dd=visual->getDepth();
+
+    // Just in case you're on a high-end system
+    FXASSERT(vis->map_entries<=MAX_MAPSIZE);
+
+    // Make array for data if needed
+    if(!data || !(options&IMAGE_OWNED)){
+      size=width*height*channels;
+      FXMALLOC(&data,FXuchar,size);
+      options|=IMAGE_OWNED;
+      }
+
+    // Got local buffer to receive into
+    if(data){
+
+      // Turn it on iff both supported and desired
+#ifdef HAVE_XSHM
+      if(options&IMAGE_SHMI) shmi=getApp()->shmi;
+#endif
+
+      // First try XShm
 #ifdef HAVE_XSHM
-    if(shmi){
-      xim=XShmCreateImage(DISPLAY(getApp()),vis,dd,(dd==1)?XYPixmap:ZPixmap,NULL,&shminfo,width,height);
-      if(!xim){ shmi=0; }
       if(shmi){
-        shminfo.shmid=shmget(IPC_PRIVATE,xim->bytes_per_line*xim->height,IPC_CREAT|0777);
-        if(shminfo.shmid==-1){ xim->data=NULL; XDestroyImage(xim); xim=NULL; shmi=0; }
+        xim=XShmCreateImage(DISPLAY(getApp()),vis,dd,(dd==1)?XYPixmap:ZPixmap,NULL,&shminfo,width,height);
+        if(!xim){ shmi=0; }
         if(shmi){
-          shminfo.shmaddr=xim->data=(char*)shmat(shminfo.shmid,0,0);
-          shminfo.readOnly=FALSE;
-          XShmAttach(DISPLAY(getApp()),&shminfo);
-          FXTRACE((150,"RGBPixmap XSHM attached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
-          XShmGetImage(DISPLAY(getApp()),xid,xim,0,0,AllPlanes);
-          XSync(DISPLAY(getApp()),False);
+          shminfo.shmid=shmget(IPC_PRIVATE,xim->bytes_per_line*xim->height,IPC_CREAT|0777);
+          if(shminfo.shmid==-1){ xim->data=NULL; XDestroyImage(xim); xim=NULL; shmi=0; }
+          if(shmi){
+            shminfo.shmaddr=xim->data=(char*)shmat(shminfo.shmid,0,0);
+            shminfo.readOnly=FALSE;
+            XShmAttach(DISPLAY(getApp()),&shminfo);
+            FXTRACE((150,"RGBPixmap XSHM attached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
+            XShmGetImage(DISPLAY(getApp()),xid,xim,0,0,AllPlanes);
+            XSync(DISPLAY(getApp()),False);
+            }
           }
         }
-      }
 #endif
 
-    // Try the old fashioned way
-    if(!shmi){
-      xim=XGetImage(DISPLAY(getApp()),xid,0,0,width,height,AllPlanes,ZPixmap);
-      if(!xim){ fxerror("%s::restore: unable to restore image.\n",getClassName()); }
-      }
-
-    // Should have succeeded
-    FXASSERT(xim);
-
-    FXTRACE((150,"im width = %d\n",xim->width));
-    FXTRACE((150,"im height = %d\n",xim->height));
-    FXTRACE((150,"im format = %s\n",xim->format==XYBitmap?"XYBitmap":xim->format==XYPixmap?"XYPixmap":"ZPixmap"));
-    FXTRACE((150,"im byte_order = %s\n",(xim->byte_order==MSBFirst)?"MSBFirst":"LSBFirst"));
-    FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
-    FXTRACE((150,"im bitmap_bit_order = %s\n",(xim->bitmap_bit_order==MSBFirst)?"MSBFirst":"LSBFirst"));
-    FXTRACE((150,"im bitmap_pad = %d\n",xim->bitmap_pad));
-    FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
-    FXTRACE((150,"im depth = %d\n",xim->depth));
-    FXTRACE((150,"im bytes_per_line = %d\n",xim->bytes_per_line));
-    FXTRACE((150,"im bits_per_pixel = %d\n",xim->bits_per_pixel));
-
-
-    {
-    XColor colors[MAX_MAPSIZE];
-
-    // Read back the colormap and convert to more usable form
-    if(vis->c_class!=TrueColor && vis->c_class!=DirectColor){
-      for(i=0 ; i<vis->map_entries; i++){
-        colors[i].pixel=i;
-        colors[i].flags=DoRed|DoGreen|DoBlue;
+      // Try the old fashioned way
+      if(!shmi){
+        xim=XGetImage(DISPLAY(getApp()),xid,0,0,width,height,AllPlanes,ZPixmap);
+        if(!xim){ fxerror("%s::restore: unable to restore image.\n",getClassName()); }
         }
-      }
-    else{
-      red=green=blue=0;
-      red1=lowbit(vis->red_mask);
-      green1=lowbit(vis->green_mask);
-      blue1=lowbit(vis->blue_mask);
+
+      // Should have succeeded
+      FXASSERT(xim);
+
+      FXTRACE((150,"im width = %d\n",xim->width));
+      FXTRACE((150,"im height = %d\n",xim->height));
+      FXTRACE((150,"im format = %s\n",xim->format==XYBitmap?"XYBitmap":xim->format==XYPixmap?"XYPixmap":"ZPixmap"));
+      FXTRACE((150,"im byte_order = %s\n",(xim->byte_order==MSBFirst)?"MSBFirst":"LSBFirst"));
+      FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
+      FXTRACE((150,"im bitmap_bit_order = %s\n",(xim->bitmap_bit_order==MSBFirst)?"MSBFirst":"LSBFirst"));
+      FXTRACE((150,"im bitmap_pad = %d\n",xim->bitmap_pad));
+      FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
+      FXTRACE((150,"im depth = %d\n",xim->depth));
+      FXTRACE((150,"im bytes_per_line = %d\n",xim->bytes_per_line));
+      FXTRACE((150,"im bits_per_pixel = %d\n",xim->bits_per_pixel));
+
+
+      {
+      XColor colors[MAX_MAPSIZE];
+      
+      // Get masks
+      redmask=vis->red_mask;
+      greenmask=vis->green_mask;
+      bluemask=vis->blue_mask;
+
+      // Read back the colormap and convert to more usable form
+      if(vis->c_class!=TrueColor && vis->c_class!=DirectColor){
+        for(i=0 ; i<vis->map_entries; i++){
+          colors[i].pixel=i;
+          colors[i].flags=DoRed|DoGreen|DoBlue;
+          }
+        }
+      else{
+        red=green=blue=0;
+        red1=lowbit(redmask);
+        green1=lowbit(greenmask);
+        blue1=lowbit(bluemask);
+        for(i=0; i<vis->map_entries; i++){
+          colors[i].pixel=red|green|blue;
+          colors[i].flags=DoRed|DoGreen|DoBlue;
+          if(red<redmask) red+=red1;
+          if(green<greenmask) green+=green1;
+          if(blue<bluemask) blue+=blue1;
+          }
+        }
+      XQueryColors(DISPLAY(getApp()),visual->colormap,colors,vis->map_entries);
       for(i=0; i<vis->map_entries; i++){
-        colors[i].pixel=red|green|blue;
-        colors[i].flags=DoRed|DoGreen|DoBlue;
-        if(red<vis->red_mask) red+=red1;
-        if(green<vis->green_mask) green+=green1;
-        if(blue<vis->blue_mask) blue+=blue1;
+        rtab[i]=colors[i].red >> 8;
+        gtab[i]=colors[i].green >> 8;
+        btab[i]=colors[i].blue >> 8;
         }
       }
-    XQueryColors(DISPLAY(getApp()),visual->colormap,colors,vis->map_entries);
-    for(i=0; i<vis->map_entries; i++){
-      rtab[i]=colors[i].red >> 8;
-      gtab[i]=colors[i].green >> 8;
-      btab[i]=colors[i].blue >> 8;
-      }
-    }
 
-    // Now we convert the pixels back to color
-    img=data;
-    switch(xim->bits_per_pixel){
-      case 0:
-      case 1:
-      case 2:
-      case 3:
-      case 4:
-      case 5:
-      case 6:
-      case 7:
-      case 8:
-        for(y=0; y<height; y++){
-          for(x=0; x<width; x++){
-            pixel=XGetPixel(xim,x,y);
-            img[0]=rtab[pixel];
-            img[1]=gtab[pixel];
-            img[2]=btab[pixel];
-            img+=channels;
+      // Now we convert the pixels back to color
+      img=data;
+      switch(xim->bits_per_pixel){
+        case 0:
+        case 1:
+        case 2:
+        case 3:
+        case 4:
+        case 5:
+        case 6:
+        case 7:
+        case 8:
+          for(y=0; y<height; y++){
+            for(x=0; x<width; x++){
+              pixel=XGetPixel(xim,x,y);
+              img[0]=rtab[pixel];
+              img[1]=gtab[pixel];
+              img[2]=btab[pixel];
+              img+=channels;
+              }
             }
-          }
-        break;
-      case 15:
-      case 16:
-      case 24:
-      case 32:
-      default:
-        FXASSERT(vis->c_class==TrueColor || vis->c_class==DirectColor);
-        for(y=0; y<height; y++){
-          for(x=0; x<width; x++){
-            pixel=XGetPixel(xim,x,y);
-            r=(pixel&redmask)>>redshift;
-            g=(pixel&greenmask)>>greenshift;
-            b=(pixel&bluemask)>>blueshift;
-            img[0]=rtab[r];
-            img[1]=gtab[g];
-            img[2]=btab[b];
-            img+=channels;
+          break;
+        case 15:
+        case 16:
+        case 24:
+        case 32:
+        default:
+          FXASSERT(vis->c_class==TrueColor || vis->c_class==DirectColor);
+          redshift=findshift(redmask);
+          greenshift=findshift(greenmask);
+          blueshift=findshift(bluemask);
+          for(y=0; y<height; y++){
+            for(x=0; x<width; x++){
+              pixel=XGetPixel(xim,x,y);
+              r=(pixel&redmask)>>redshift;
+              g=(pixel&greenmask)>>greenshift;
+              b=(pixel&bluemask)>>blueshift;
+              img[0]=rtab[r];
+              img[1]=gtab[g];
+              img[2]=btab[b];
+              img+=channels;
+              }
             }
-          }
-        break;
-      }
+          break;
+        }
 
-    // Destroy image
+      // Destroy image
 #ifdef HAVE_XSHM
-    if(shmi){
-      FXTRACE((150,"RGBPixmap XSHM detached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
-      XShmDetach(DISPLAY(getApp()),&shminfo);
-      XDestroyImage(xim);
-      shmdt(shminfo.shmaddr);
-      shmctl(shminfo.shmid,IPC_RMID,0);
-      }
+      if(shmi){
+        FXTRACE((150,"RGBPixmap XSHM detached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
+        XShmDetach(DISPLAY(getApp()),&shminfo);
+        XDestroyImage(xim);
+        shmdt(shminfo.shmaddr);
+        shmctl(shminfo.shmid,IPC_RMID,0);
+        }
 #endif
 
-    // Destroy image
-    if(!shmi){
-      XDestroyImage(xim);
+      // Destroy image
+      if(!shmi){
+        XDestroyImage(xim);
+        }
       }
     }
   }
@@ -486,70 +481,69 @@ void FXImage::restore(){
 
 // Restore client-side pixel buffer from image
 void FXImage::restore(){
-  register FXint size,bytes_per_line,skip;
-  register FXint x,y;
-  register FXuchar *pixels,*pix,*img;
-  BITMAPINFO bmi;
-  HDC hdcmem;
+  if(xid){
+    register FXint size,bytes_per_line,skip;
+    register FXint x,y;
+    register FXuchar *pixels,*pix,*img;
+    BITMAPINFO bmi;
+    HDC hdcmem;
 
-  FXTRACE((100,"%s::restore image %p\n",getClassName(),this));
+    FXTRACE((100,"%s::restore image %p\n",getClassName(),this));
 
-  // Can not restore before creation
-  if(!xid){ fxerror("%s::restore: trying to restore image before it has been created.\n",getClassName()); }
+    // Check for legal size
+    if(width<1 || height<1){ fxerror("%s::restore: illegal image size %dx%d.\n",getClassName(),width,height); }
 
-  // Check for legal size
-  if(width<1 || height<1){ fxerror("%s::restore: illegal image size %dx%d.\n",getClassName(),width,height); }
-
-  // Make array for data if needed
-  if(!data || !(options&IMAGE_OWNED)){
-    size=width*height*channels;
-    FXMALLOC(&data,FXuchar,size);
-    options|=IMAGE_OWNED;
-    }
-
-  // Got local buffer to receive into
-  if(data){
-
-    // Set up the bitmap info
-    bytes_per_line=(width*3+3)/4*4;
-    skip=bytes_per_line-width*3;
-
-    bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
-    bmi.bmiHeader.biWidth=width;
-    bmi.bmiHeader.biHeight=-height; // Negative heights means upside down!
-    bmi.bmiHeader.biPlanes=1;
-    bmi.bmiHeader.biBitCount=24;
-    bmi.bmiHeader.biCompression=BI_RGB;
-    bmi.bmiHeader.biSizeImage=0;
-    bmi.bmiHeader.biXPelsPerMeter=0;
-    bmi.bmiHeader.biYPelsPerMeter=0;
-    bmi.bmiHeader.biClrUsed=0;
-    bmi.bmiHeader.biClrImportant=0;
-
-    // DIB format pads to multiples of 4 bytes...
-    FXMALLOC(&pixels,FXuchar,bytes_per_line*height);
-
-    // Make device context
-    hdcmem=::CreateCompatibleDC(NULL);
-    if(!GetDIBits(hdcmem,(HBITMAP)xid,0,height,pixels,&bmi,DIB_RGB_COLORS)){
-      fxerror("%s::render: unable to restore pixels\n",getClassName());
+    // Make array for data if needed
+    if(!data || !(options&IMAGE_OWNED)){
+      size=width*height*channels;
+      FXMALLOC(&data,FXuchar,size);
+      options|=IMAGE_OWNED;
       }
 
-    // Stuff it into our own data structure
-    img=data;
-    pix=pixels;
-    for(y=0; y<height; y++){
-      for(x=0; x<width; x++){
-        img[0]=pix[2];
-        img[1]=pix[1];
-        img[2]=pix[0];
-        img+=channels;
-        pix+=3;
+    // Got local buffer to receive into
+    if(data){
+
+      // Set up the bitmap info
+      bytes_per_line=(width*3+3)/4*4;
+      skip=bytes_per_line-width*3;
+
+      bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+      bmi.bmiHeader.biWidth=width;
+      bmi.bmiHeader.biHeight=-height; // Negative heights means upside down!
+      bmi.bmiHeader.biPlanes=1;
+      bmi.bmiHeader.biBitCount=24;
+      bmi.bmiHeader.biCompression=BI_RGB;
+      bmi.bmiHeader.biSizeImage=0;
+      bmi.bmiHeader.biXPelsPerMeter=0;
+      bmi.bmiHeader.biYPelsPerMeter=0;
+      bmi.bmiHeader.biClrUsed=0;
+      bmi.bmiHeader.biClrImportant=0;
+
+      // DIB format pads to multiples of 4 bytes...
+      FXMALLOC(&pixels,FXuchar,bytes_per_line*height);
+
+      // Make device context
+      hdcmem=::CreateCompatibleDC(NULL);
+      if(!GetDIBits(hdcmem,(HBITMAP)xid,0,height,pixels,&bmi,DIB_RGB_COLORS)){
+        fxerror("%s::render: unable to restore pixels\n",getClassName());
         }
-      pix+=skip;
+
+      // Stuff it into our own data structure
+      img=data;
+      pix=pixels;
+      for(y=0; y<height; y++){
+        for(x=0; x<width; x++){
+          img[0]=pix[2];
+          img[1]=pix[1];
+          img[2]=pix[0];
+          img+=channels;
+          pix+=3;
+          }
+        pix+=skip;
+        }
+      FXFREE(&pixels);
+      ::DeleteDC(hdcmem);
       }
-    FXFREE(&pixels);
-    ::DeleteDC(hdcmem);
     }
   }
 
@@ -1373,189 +1367,188 @@ void FXImage::render_mono_1_dither(void *xim,FXuchar *img){
 
 // Render into pixmap
 void FXImage::render(){
+  if(xid){
+    register FXbool shmi=FALSE;
+    register XImage *xim=NULL;
+    register Visual *vis;
+    register int dd;
+    XGCValues values;
+    GC gc;
 #ifdef HAVE_XSHM
-  XShmSegmentInfo shminfo;
-#endif
-  register FXbool shmi=FALSE;
-  register XImage *xim=NULL;
-  register Visual *vis;
-  register int dd;
-  XGCValues values;
-  GC gc;
-
-  FXTRACE((100,"%s::render image %p\n",getClassName(),this));
-
-  // Can not render before creation
-  if(!xid){ fxerror("%s::render: trying to render image before it has been created.\n",getClassName()); }
-
-  // Check for legal size
-  if(width<1 || height<1){ fxerror("%s::render: illegal image size %dx%d.\n",getClassName(),width,height); }
-
-  // Just leave if black if no data
-  if(data){
-
-    // Make GC
-    values.foreground=BlackPixel(DISPLAY(getApp()),DefaultScreen(DISPLAY(getApp())));
-    values.background=WhitePixel(DISPLAY(getApp()),DefaultScreen(DISPLAY(getApp())));
-    gc=XCreateGC(DISPLAY(getApp()),xid,GCForeground|GCBackground,&values);
-
-    // Get Visual
-    vis=(Visual*)visual->visual;
-
-    dd=visual->getDepth();
-
-    // Turn it on iff both supported and desired
-#ifdef HAVE_XSHM
-    if(options&IMAGE_SHMI) shmi=getApp()->shmi;
+    XShmSegmentInfo shminfo;
 #endif
 
-    // First try XShm
+    FXTRACE((100,"%s::render image %p\n",getClassName(),this));
+
+    // Check for legal size
+    if(width<1 || height<1){ fxerror("%s::render: illegal image size %dx%d.\n",getClassName(),width,height); }
+
+    // Just leave if black if no data
+    if(data){
+
+      // Make GC
+      values.foreground=BlackPixel(DISPLAY(getApp()),DefaultScreen(DISPLAY(getApp())));
+      values.background=WhitePixel(DISPLAY(getApp()),DefaultScreen(DISPLAY(getApp())));
+      gc=XCreateGC(DISPLAY(getApp()),xid,GCForeground|GCBackground,&values);
+
+      // Get Visual
+      vis=(Visual*)visual->visual;
+
+      dd=visual->getDepth();
+
+      // Turn it on iff both supported and desired
+#ifdef HAVE_XSHM
+      if(options&IMAGE_SHMI) shmi=getApp()->shmi;
+#endif
+
+      // First try XShm
 #ifdef HAVE_XSHM
-    if(shmi){
-      xim=XShmCreateImage(DISPLAY(getApp()),vis,dd,(dd==1)?XYPixmap:ZPixmap,NULL,&shminfo,width,height);
-      if(!xim){ shmi=0; }
       if(shmi){
-        shminfo.shmid=shmget(IPC_PRIVATE,xim->bytes_per_line*xim->height,IPC_CREAT|0777);
-        if(shminfo.shmid==-1){ xim->data=NULL; XDestroyImage(xim); xim=NULL; shmi=0; }
+        xim=XShmCreateImage(DISPLAY(getApp()),vis,dd,(dd==1)?XYPixmap:ZPixmap,NULL,&shminfo,width,height);
+        if(!xim){ shmi=0; }
         if(shmi){
-          shminfo.shmaddr=xim->data=(char*)shmat(shminfo.shmid,0,0);
-          shminfo.readOnly=FALSE;
-          XShmAttach(DISPLAY(getApp()),&shminfo);
-          FXTRACE((150,"RGBPixmap XSHM attached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
+          shminfo.shmid=shmget(IPC_PRIVATE,xim->bytes_per_line*xim->height,IPC_CREAT|0777);
+          if(shminfo.shmid==-1){ xim->data=NULL; XDestroyImage(xim); xim=NULL; shmi=0; }
+          if(shmi){
+            shminfo.shmaddr=xim->data=(char*)shmat(shminfo.shmid,0,0);
+            shminfo.readOnly=FALSE;
+            XShmAttach(DISPLAY(getApp()),&shminfo);
+            FXTRACE((150,"RGBPixmap XSHM attached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
+            }
           }
         }
-      }
 #endif
 
-    // Try the old fashioned way
-    if(!shmi){
-      xim=XCreateImage(DISPLAY(getApp()),vis,dd,(dd==1)?XYPixmap:ZPixmap,0,NULL,width,height,32,0);
-      if(!xim){ fxerror("%s::render: unable to render image.\n",getClassName()); }
+      // Try the old fashioned way
+      if(!shmi){
+        xim=XCreateImage(DISPLAY(getApp()),vis,dd,(dd==1)?XYPixmap:ZPixmap,0,NULL,width,height,32,0);
+        if(!xim){ fxerror("%s::render: unable to render image.\n",getClassName()); }
 
-      // Try create temp pixel store
-      if(!FXMALLOC(&xim->data,char,xim->bytes_per_line*height)){ fxerror("%s::render: unable to allocate memory.\n",getClassName()); }
-      }
+        // Try create temp pixel store
+        if(!FXMALLOC(&xim->data,char,xim->bytes_per_line*height)){ fxerror("%s::render: unable to allocate memory.\n",getClassName()); }
+        }
 
-    // Should have succeeded
-    FXASSERT(xim);
+      // Should have succeeded
+      FXASSERT(xim);
 
-    FXTRACE((150,"im width = %d\n",xim->width));
-    FXTRACE((150,"im height = %d\n",xim->height));
-    FXTRACE((150,"im format = %s\n",xim->format==XYBitmap?"XYBitmap":xim->format==XYPixmap?"XYPixmap":"ZPixmap"));
-    FXTRACE((150,"im byte_order = %s\n",(xim->byte_order==MSBFirst)?"MSBFirst":"LSBFirst"));
-    FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
-    FXTRACE((150,"im bitmap_bit_order = %s\n",(xim->bitmap_bit_order==MSBFirst)?"MSBFirst":"LSBFirst"));
-    FXTRACE((150,"im bitmap_pad = %d\n",xim->bitmap_pad));
-    FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
-    FXTRACE((150,"im depth = %d\n",xim->depth));
-    FXTRACE((150,"im bytes_per_line = %d\n",xim->bytes_per_line));
-    FXTRACE((150,"im bits_per_pixel = %d\n",xim->bits_per_pixel));
+      FXTRACE((150,"im width = %d\n",xim->width));
+      FXTRACE((150,"im height = %d\n",xim->height));
+      FXTRACE((150,"im format = %s\n",xim->format==XYBitmap?"XYBitmap":xim->format==XYPixmap?"XYPixmap":"ZPixmap"));
+      FXTRACE((150,"im byte_order = %s\n",(xim->byte_order==MSBFirst)?"MSBFirst":"LSBFirst"));
+      FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
+      FXTRACE((150,"im bitmap_bit_order = %s\n",(xim->bitmap_bit_order==MSBFirst)?"MSBFirst":"LSBFirst"));
+      FXTRACE((150,"im bitmap_pad = %d\n",xim->bitmap_pad));
+      FXTRACE((150,"im bitmap_unit = %d\n",xim->bitmap_unit));
+      FXTRACE((150,"im depth = %d\n",xim->depth));
+      FXTRACE((150,"im bytes_per_line = %d\n",xim->bytes_per_line));
+      FXTRACE((150,"im bits_per_pixel = %d\n",xim->bits_per_pixel));
 
-    // Determine what to do
-    switch(visual->getType()){
-      case VISUALTYPE_TRUE:
-        switch(xim->bits_per_pixel){
-          case 32:
-            render_true_32(xim,data);
-            break;
-          case 24:
-            render_true_24(xim,data);
-            break;
-          case 15:
-          case 16:
-            if(options&IMAGE_NEAREST)
-              render_true_16_fast(xim,data);
-            else
-              render_true_16_dither(xim,data);
-            break;
-          case 8:
-            if(options&IMAGE_NEAREST)
-              render_true_8_fast(xim,data);
-            else
-              render_true_8_dither(xim,data);
-            break;
-          default:
-            if(options&IMAGE_NEAREST)
-              render_true_N_fast(xim,data);
-            else
-              render_true_N_dither(xim,data);
-            break;
-          }
-        break;
-      case VISUALTYPE_GRAY:
-        switch(xim->bits_per_pixel){
-          case 1:
-            if(options&IMAGE_NEAREST)
-              render_mono_1_fast(xim,data);
-            else
-              render_mono_1_dither(xim,data);
-            break;
-          case 8:
-            if(options&IMAGE_NEAREST)
-              render_gray_8_fast(xim,data);
-            else
-              render_gray_8_dither(xim,data);
-            break;
-          default:
-            if(options&IMAGE_NEAREST)
-              render_gray_N_fast(xim,data);
-            else
-              render_gray_N_dither(xim,data);
-            break;
-          }
-        break;
-      case VISUALTYPE_INDEX:
-        switch(xim->bits_per_pixel){
-          case 4:
-            if(options&IMAGE_NEAREST)
-              render_index_4_fast(xim,data);
-            else
-              render_index_4_dither(xim,data);
-            break;
-          case 8:
-            if(options&IMAGE_NEAREST)
-              render_index_8_fast(xim,data);
-            else
-              render_index_8_dither(xim,data);
-            break;
-          default:
-            if(options&IMAGE_NEAREST)
-              render_index_N_fast(xim,data);
-            else
-              render_index_N_dither(xim,data);
-            break;
-          }
-        break;
-      case VISUALTYPE_MONO:
-        if(options&IMAGE_NEAREST)
-          render_mono_1_fast(xim,data);
-        else
-          render_mono_1_dither(xim,data);
-      case VISUALTYPE_UNKNOWN:
-        break;
-      }
+      // Determine what to do
+      switch(visual->getType()){
+        case VISUALTYPE_TRUE:
+          switch(xim->bits_per_pixel){
+            case 32:
+              render_true_32(xim,data);
+              break;
+            case 24:
+              render_true_24(xim,data);
+              break;
+            case 15:
+            case 16:
+              if(options&IMAGE_NEAREST)
+                render_true_16_fast(xim,data);
+              else
+                render_true_16_dither(xim,data);
+              break;
+            case 8:
+              if(options&IMAGE_NEAREST)
+                render_true_8_fast(xim,data);
+              else
+                render_true_8_dither(xim,data);
+              break;
+            default:
+              if(options&IMAGE_NEAREST)
+                render_true_N_fast(xim,data);
+              else
+                render_true_N_dither(xim,data);
+              break;
+            }
+          break;
+        case VISUALTYPE_GRAY:
+          switch(xim->bits_per_pixel){
+            case 1:
+              if(options&IMAGE_NEAREST)
+                render_mono_1_fast(xim,data);
+              else
+                render_mono_1_dither(xim,data);
+              break;
+            case 8:
+              if(options&IMAGE_NEAREST)
+                render_gray_8_fast(xim,data);
+              else
+                render_gray_8_dither(xim,data);
+              break;
+            default:
+              if(options&IMAGE_NEAREST)
+                render_gray_N_fast(xim,data);
+              else
+                render_gray_N_dither(xim,data);
+              break;
+            }
+          break;
+        case VISUALTYPE_INDEX:
+          switch(xim->bits_per_pixel){
+            case 4:
+              if(options&IMAGE_NEAREST)
+                render_index_4_fast(xim,data);
+              else
+                render_index_4_dither(xim,data);
+              break;
+            case 8:
+              if(options&IMAGE_NEAREST)
+                render_index_8_fast(xim,data);
+              else
+                render_index_8_dither(xim,data);
+              break;
+            default:
+              if(options&IMAGE_NEAREST)
+                render_index_N_fast(xim,data);
+              else
+                render_index_N_dither(xim,data);
+              break;
+            }
+          break;
+        case VISUALTYPE_MONO:
+          if(options&IMAGE_NEAREST)
+            render_mono_1_fast(xim,data);
+          else
+            render_mono_1_dither(xim,data);
+        case VISUALTYPE_UNKNOWN:
+          break;
+        }
 
-    // Transfer image with shared memory
+      // Transfer image with shared memory
 #ifdef HAVE_XSHM
-    if(shmi){
-      XShmPutImage(DISPLAY(getApp()),xid,gc,xim,0,0,0,0,width,height,False);
-      XSync(DISPLAY(getApp()),False);
-      FXTRACE((150,"RGBPixmap XSHM detached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
-      XShmDetach(DISPLAY(getApp()),&shminfo);
-      xim->data=NULL;
-      XDestroyImage(xim);
-      shmdt(shminfo.shmaddr);
-      shmctl(shminfo.shmid,IPC_RMID,0);
-      }
+      if(shmi){
+        XShmPutImage(DISPLAY(getApp()),xid,gc,xim,0,0,0,0,width,height,False);
+        XSync(DISPLAY(getApp()),False);
+        FXTRACE((150,"RGBPixmap XSHM detached at memory=%p (%d bytes)\n",xim->data,xim->bytes_per_line*xim->height));
+        XShmDetach(DISPLAY(getApp()),&shminfo);
+        xim->data=NULL;
+        XDestroyImage(xim);
+        shmdt(shminfo.shmaddr);
+        shmctl(shminfo.shmid,IPC_RMID,0);
+        }
 #endif
 
-    // Transfer the image old way
-    if(!shmi){
-      XPutImage(DISPLAY(getApp()),xid,gc,xim,0,0,0,0,width,height);
-      FXFREE(&xim->data);
-      XDestroyImage(xim);
+      // Transfer the image old way
+      if(!shmi){
+        XPutImage(DISPLAY(getApp()),xid,gc,xim,0,0,0,0,width,height);
+        FXFREE(&xim->data);
+        XDestroyImage(xim);
+        }
+      XFreeGC(DISPLAY(getApp()),gc);
       }
-    XFreeGC(DISPLAY(getApp()),gc);
     }
   }
 
@@ -1564,72 +1557,71 @@ void FXImage::render(){
 
 
 void FXImage::render(){
-  register FXuint bytes_per_line,skip;
-  register FXuchar *src,*dst;
-  register FXint h,w;
-  BITMAPINFO bmi;
-  FXuchar *pixels;
-  HDC hdcmem;
+  if(xid){
+    register FXuint bytes_per_line,skip;
+    register FXuchar *src,*dst;
+    register FXint h,w;
+    BITMAPINFO bmi;
+    FXuchar *pixels;
+    HDC hdcmem;
 
-  FXTRACE((100,"%s::render %p\n",getClassName(),this));
+    FXTRACE((100,"%s::render %p\n",getClassName(),this));
 
-  // Can not render before creation
-  if(!xid){ fxerror("%s::render: trying to render image before it has been created.\n",getClassName()); }
-
-  // Check for legal size
-  if(width<1 || height<1){ fxerror("%s::render: illegal image size %dx%d.\n",getClassName(),width,height); }
+    // Check for legal size
+    if(width<1 || height<1){ fxerror("%s::render: illegal image size %dx%d.\n",getClassName(),width,height); }
 
 
-  // Just leave if black if no data
-  if(data){
+    // Just leave if black if no data
+    if(data){
 
-    // Set up the bitmap info
-    bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
-    bmi.bmiHeader.biWidth=width;
-    bmi.bmiHeader.biHeight=height;
-    bmi.bmiHeader.biPlanes=1;
-    bmi.bmiHeader.biBitCount=24;
-    bmi.bmiHeader.biCompression=BI_RGB;
-    bmi.bmiHeader.biSizeImage=0;
-    bmi.bmiHeader.biXPelsPerMeter=0;
-    bmi.bmiHeader.biYPelsPerMeter=0;
-    bmi.bmiHeader.biClrUsed=0;
-    bmi.bmiHeader.biClrImportant=0;
+      // Set up the bitmap info
+      bmi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
+      bmi.bmiHeader.biWidth=width;
+      bmi.bmiHeader.biHeight=height;
+      bmi.bmiHeader.biPlanes=1;
+      bmi.bmiHeader.biBitCount=24;
+      bmi.bmiHeader.biCompression=BI_RGB;
+      bmi.bmiHeader.biSizeImage=0;
+      bmi.bmiHeader.biXPelsPerMeter=0;
+      bmi.bmiHeader.biYPelsPerMeter=0;
+      bmi.bmiHeader.biClrUsed=0;
+      bmi.bmiHeader.biClrImportant=0;
 
-    // DIB format pads to multiples of 4 bytes...
-    bytes_per_line=(width*3+3)&~3;
-    skip=bytes_per_line+width*3;
-    FXMALLOC(&pixels,FXuchar,bytes_per_line*height);
-    h=height-1;
-    src=data;
-    dst=pixels+h*bytes_per_line;
-    do{
-      w=width-1;
+      // DIB format pads to multiples of 4 bytes...
+      bytes_per_line=(width*3+3)&~3;
+      skip=bytes_per_line+width*3;
+      FXMALLOC(&pixels,FXuchar,bytes_per_line*height);
+      h=height-1;
+      src=data;
+      dst=pixels+h*bytes_per_line;
       do{
-        dst[0]=src[2];
-        dst[1]=src[1];
-        dst[2]=src[0];
-        src+=channels;
-        dst+=3;
+        w=width-1;
+        do{
+          dst[0]=src[2];
+          dst[1]=src[1];
+          dst[2]=src[0];
+          src+=channels;
+          dst+=3;
+          }
+        while(--w>=0);
+        dst-=skip;
         }
-      while(--w>=0);
-      dst-=skip;
-      }
-    while(--h>=0);
+      while(--h>=0);
 
-    // The MSDN documentation for SetDIBits() states that "the device context
-    // identified by the (first) parameter is used only if the DIB_PAL_COLORS
-    // constant is set for the (last) parameter". This may be true, but under
-    // Win95 you must pass in a non-NULL hdc for the first parameter; otherwise
-    // this call to SetDIBits() will fail (in contrast, it works fine under
-    // Windows NT if you pass in a NULL hdc).
-    hdcmem=::CreateCompatibleDC(NULL);
-    if(!SetDIBits(hdcmem,(HBITMAP)xid,0,height,pixels,&bmi,DIB_RGB_COLORS)){
-      fxerror("%s::render: unable to render pixels\n",getClassName());
+      // The MSDN documentation for SetDIBits() states that "the device context
+      // identified by the (first) parameter is used only if the DIB_PAL_COLORS
+      // constant is set for the (last) parameter". This may be true, but under
+      // Win95 you must pass in a non-NULL hdc for the first parameter; otherwise
+      // this call to SetDIBits() will fail (in contrast, it works fine under
+      // Windows NT if you pass in a NULL hdc).
+      hdcmem=::CreateCompatibleDC(NULL);
+      if(!SetDIBits(hdcmem,(HBITMAP)xid,0,height,pixels,&bmi,DIB_RGB_COLORS)){
+        fxerror("%s::render: unable to render pixels\n",getClassName());
+        }
+      GdiFlush();
+      FXFREE(&pixels);
+      ::DeleteDC(hdcmem);
       }
-    GdiFlush();
-    FXFREE(&pixels);
-    ::DeleteDC(hdcmem);
     }
   }
 
@@ -1637,6 +1629,25 @@ void FXImage::render(){
 
 
 
+// Get pixel at x,y
+FXColor FXImage::getPixel(FXint x,FXint y) const {
+  return (options&IMAGE_ALPHA) ? ((FXColor*)data)[y*width+x] : FXRGB(data[(y*width+x)*3],data[(y*width+x)*3+1],data[(y*width+x)*3+2]);
+  }
+
+
+// Change pixel at x,y
+void FXImage::setPixel(FXint x,FXint y,FXColor color){
+  if(options&IMAGE_ALPHA){
+    ((FXColor*)data)[y*width+x]=color;
+    }
+  else{
+    data[(y*width+x)*3+0]=FXREDVAL(color);
+    data[(y*width+x)*3+1]=FXGREENVAL(color);
+    data[(y*width+x)*3+2]=FXBLUEVAL(color);
+    }
+  }
+
+
 // Resize pixmap to the specified width and height; the data
 // array is resized also, but its contents will be undefined.
 void FXImage::resize(FXint w,FXint h){
@@ -1870,7 +1881,7 @@ void FXImage::scale(FXint w,FXint h){
 
       // Free interim buffer
       FXFREE(&interim);
-      if(xid) render();
+      render();
       }
     else{
       resize(w,h);
@@ -1953,7 +1964,7 @@ void FXImage::mirror(FXbool horizontal,FXbool vertical){
         }
       if(options&IMAGE_OWNED){FXFREE(&olddata);}
       options|=IMAGE_OWNED;
-      if(xid) render();
+      render();
       }
     }
   }
@@ -2081,7 +2092,7 @@ void FXImage::rotate(FXint degrees){
           break;
         }
       FXFREE(&olddata);
-      if(xid) render();
+      render();
       }
     else{
       switch(degrees){
@@ -2158,7 +2169,7 @@ void FXImage::crop(FXint x,FXint y,FXint w,FXint h){
       while(paa<end);
       }
     FXFREE(&olddata);
-    if(xid) render();
+    render();
     }
   else{
     resize(w,h);
