@@ -3,7 +3,7 @@
 *                            O b j e c t   L i s t                              *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXObjectList.cpp,v 1.14 2001/08/24 18:53:39 jeroen Exp $                 *
+* $Id: FXObjectList.cpp,v 1.16 2002/01/18 22:43:01 jeroen Exp $                 *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -127,8 +127,8 @@ void FXObjectList::remove(const FXObject* p){
   register FXint i;
   for(i=0; i<num; i++){
     if(data[i]==p){
-      while(i<num-1){ data[i]=data[i+1]; i++; }
       num--;
+      while(i<num){ data[i]=data[i+1]; i++; }
       return;
       }
     }
@@ -140,7 +140,7 @@ FXint FXObjectList::findf(const FXObject* obj,FXint pos) const {
   register const FXObject *o=obj;
   register FXint p=pos;
   if(p<0) return -1;
-  while(p<num){ if(data[p]==o){ return p; } p++; } 
+  while(p<num){ if(data[p]==o){ return p; } p++; }
   return -1;
   }
 
