@@ -3,7 +3,7 @@
 *                P a c k e r   C o n t a i n e r   O b j e c t                  *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXPacker.cpp,v 1.18 2001/08/28 22:52:27 jeroen Exp $                     *
+* $Id: FXPacker.cpp,v 1.24.4.1 2002/07/17 01:58:32 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -29,18 +29,9 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
-#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDC.h"
 #include "FXDCWindow.h"
-#include "FXDrawable.h"
-#include "FXWindow.h"
-#include "FXFrame.h"
-#include "FXComposite.h"
 #include "FXPacker.h"
 
 
@@ -152,10 +143,10 @@ void FXPacker::drawGrooveRectangle(FXDCWindow& dc,FXint x,FXint y,FXint w,FXint
   dc.fillRectangle(x+1,y+h-2,w-2,1);
   dc.fillRectangle(x+w-2,y+1,1,h-2);
   dc.setForeground(hiliteColor);
-  dc.fillRectangle(x+1,y+1,w-2,1);
-  dc.fillRectangle(x+1,y+1,1,h-2);
-  dc.fillRectangle(x+1,y+h-1,w,1);
-  dc.fillRectangle(x+w-1,y+1,1,h);
+  dc.fillRectangle(x+1,y+1,w-3,1);
+  dc.fillRectangle(x+1,y+1,1,h-3);
+  dc.fillRectangle(x,y+h-1,w,1);
+  dc.fillRectangle(x+w-1,y,1,h);
   }
 
 
@@ -250,29 +241,37 @@ FXuint FXPacker::getPackingHints() const {
 
 // Set base color
 void FXPacker::setBaseColor(FXColor clr){
-  baseColor=clr;
-  update();
+  if(baseColor!=clr){
+    baseColor=clr;
+    update();
+    }
   }
 
 
 // Set highlight color
 void FXPacker::setHiliteColor(FXColor clr){
-  hiliteColor=clr;
-  update();
+  if(hiliteColor!=clr){
+    hiliteColor=clr;
+    update();
+    }
   }
 
 
 // Set shadow color
 void FXPacker::setShadowColor(FXColor clr){
-  shadowColor=clr;
-  update();
+  if(shadowColor!=clr){
+    shadowColor=clr;
+    update();
+    }
   }
 
 
 // Set border color
 void FXPacker::setBorderColor(FXColor clr){
-  borderColor=clr;
-  update();
+  if(borderColor!=clr){
+    borderColor=clr;
+    update();
+    }
   }
 
 
@@ -340,7 +339,6 @@ void FXPacker::setVSpacing(FXint vs){
 long FXPacker::onFocusUp(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint cury,childy;
-  FXTRACE((100,"%s::onFocusUp %p\n",getClassName(),this));
   if(getFocus()){
     cury=getFocus()->getY();
     while(1){
@@ -367,8 +365,8 @@ long FXPacker::onFocusUp(FXObject*,FXSelector sel,void* ptr){
           return 1;
           }
         if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
-        child=child->getPrev();
         }
+      child=child->getPrev();
       }
     }
   return 0;
@@ -379,7 +377,6 @@ long FXPacker::onFocusUp(FXObject*,FXSelector sel,void* ptr){
 long FXPacker::onFocusDown(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint cury,childy;
-  FXTRACE((100,"%s::onFocusDown %p\n",getClassName(),this));
   if(getFocus()){
     cury=getFocus()->getY();
     while(1){
@@ -418,7 +415,6 @@ long FXPacker::onFocusDown(FXObject*,FXSelector sel,void* ptr){
 long FXPacker::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint curx,childx;
-  FXTRACE((100,"%s::onFocusLeft %p\n",getClassName(),this));
   if(getFocus()){
     curx=getFocus()->getX();
     while(1){
@@ -457,7 +453,6 @@ long FXPacker::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
 long FXPacker::onFocusRight(FXObject*,FXSelector sel,void* ptr){
   FXWindow *child,*c;
   FXint curx,childx;
-  FXTRACE((100,"%s::onFocusRight %p\n",getClassName(),this));
   if(getFocus()){
     curx=getFocus()->getX();
     while(1){
