@@ -3,7 +3,7 @@
 *                         S t a t u s b a r   W i d g e t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXStatusbar.cpp,v 1.12 2001/04/27 14:30:34 jeroen Exp $                  *
+* $Id: FXStatusbar.cpp,v 1.14 2002/01/18 22:43:04 jeroen Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -29,24 +29,14 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
-#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDC.h"
 #include "FXDCWindow.h"
 #include "FXFont.h"
-#include "FXDrawable.h"
-#include "FXImage.h"
 #include "FXIcon.h"
 #include "FXWindow.h"
 #include "FXFrame.h"
-#include "FXComposite.h"
-#include "FXPacker.h"
-#include "FXHorizontalFrame.h"
 #include "FXDragCorner.h"
 #include "FXStatusline.h"
 #include "FXStatusbar.h"
