@@ -3,7 +3,7 @@
 *                        T o o l b a r   W i d g e t                            *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXToolbar.cpp,v 1.58 2001/08/07 14:13:53 jeroen Exp $                    *
+* $Id: FXToolbar.cpp,v 1.61.4.1 2002/10/03 06:08:48 fox Exp $                    *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -29,14 +29,9 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
-#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDC.h"
 #include "FXDCWindow.h"
 #include "FXDrawable.h"
 #include "FXWindow.h"
@@ -477,7 +472,7 @@ void FXToolbar::setDockingSide(FXuint side){
 
 // Get toolbar orientation
 FXuint FXToolbar::getDockingSide() const {
-  return (options&LAYOUT_SIDE_MASK)!=0;
+  return (options&LAYOUT_SIDE_MASK);
   }
 
 
@@ -627,8 +622,7 @@ long FXToolbar::onUpdDockRight(FXObject* sender,FXSelector,void*){
 
 
 // Toolbar grip drag started
-long FXToolbar::onBeginDragGrip(FXObject* sender,FXSelector,void* ptr){
-  FXToolbarGrip *grip=(FXToolbarGrip*)sender;
+long FXToolbar::onBeginDragGrip(FXObject*,FXSelector,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
   FXint x,y;
   FXDCWindow dc(getRoot());
