@@ -3,7 +3,7 @@
 *                         T o p   W i n d o w   O b j e c t                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXTopWindow.cpp,v 1.63 2001/08/15 06:33:40 jeroen Exp $                  *
+* $Id: FXTopWindow.cpp,v 1.72.4.1 2002/08/06 19:33:31 fox Exp $                  *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -30,25 +30,12 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
-#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXAccelTable.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDC.h"
 #include "FXDCWindow.h"
 #include "FXCursor.h"
-#include "FXDrawable.h"
-#include "FXImage.h"
 #include "FXIcon.h"
-#include "FXWindow.h"
-#include "FXFrame.h"
-#include "FXComposite.h"
-#include "FXPacker.h"
-#include "FXRootWindow.h"
-#include "FXShell.h"
 #include "FXTopWindow.h"
 #include "FXToolbar.h"
 #include "FXToolbarGrip.h"
@@ -102,9 +89,6 @@
             wmBottom = (sy - msy - wmTop);
             printf ("WM Borders l:%d r:%d t:%d b:%d\n", wmLeft, wmRight, wmTop, wmBottom);
 
-
-
-
       One thing that I would find useful, if you do eventually add code
       to fox to determine these sizes, that you make the values accessible
       to the API in the rare case that someone needs them.
@@ -120,6 +104,8 @@
       |      bar3          |///////|
       +--------------------+-------+
 
+  - Need to have some way to start window initially as "iconified".
+
 */
 
 
@@ -138,6 +124,10 @@
 // Map
 FXDEFMAP(FXTopWindow) FXTopWindowMap[]={
   FXMAPFUNC(SEL_CLOSE,0,FXTopWindow::onClose),
+  FXMAPFUNC(SEL_FOCUS_UP,0,FXTopWindow::onFocusUp),
+  FXMAPFUNC(SEL_FOCUS_DOWN,0,FXTopWindow::onFocusDown),
+  FXMAPFUNC(SEL_FOCUS_LEFT,0,FXTopWindow::onFocusLeft),
+  FXMAPFUNC(SEL_FOCUS_RIGHT,0,FXTopWindow::onFocusRight),
   FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE,FXTopWindow::onCmdSetStringValue),
   FXMAPFUNC(SEL_COMMAND,FXTopWindow::ID_ICONIFY,FXTopWindow::onCmdIconify),
   FXMAPFUNC(SEL_COMMAND,FXTopWindow::ID_DEICONIFY,FXTopWindow::onCmdDeiconify),
@@ -195,49 +185,52 @@ void FXTopWindow::create(){
   if(icon) icon->create();
   if(miniIcon) miniIcon->create();
 
-  // Catch delete window
+  if(xid){
+    if(getApp()->initialized){
 #ifndef WIN32
-  Atom protocols[2];
-  protocols[0]=getApp()->wmDeleteWindow;
-  protocols[1]=getApp()->wmTakeFocus;
-  XSetWMProtocols(DISPLAY(getApp()),xid,protocols,2);
+      // Catch delete window
+      Atom protocols[2];
+      protocols[0]=getApp()->wmDeleteWindow;
+      protocols[1]=getApp()->wmTakeFocus;
+      XSetWMProtocols(DISPLAY(getApp()),xid,protocols,2);
+
+      // Set position for Window Manager
+      XSizeHints size;
+
+      size.flags=USPosition|PPosition|USSize|PSize|PWinGravity;
+      size.min_width=0;
+      size.min_height=0;
+      size.max_width=0;
+      size.max_height=0;
+      size.x=xpos;
+      size.y=ypos;
+      size.width=width;
+      size.height=height;
+      size.width_inc=0;
+      size.height_inc=0;
+      size.min_aspect.x=0;
+      size.min_aspect.y=0;
+      size.max_aspect.x=0;
+      size.max_aspect.y=0;
+      size.base_width=0;
+      size.base_height=0;
+      size.win_gravity=StaticGravity;     // Account for border (ICCCM)
+      size.win_gravity=NorthWestGravity;  // Tim Alexeevsky <realtim@mail.ru>
+
+      // Set hints
+      XSetWMNormalHints(DISPLAY(getApp()),xid,&size);
 #endif
 
-  // Set position for Window Manager
-#ifndef WIN32
-  XSizeHints size;
+      // Set title
+      settitle();
 
-  size.flags=USPosition|PPosition|USSize|PSize|PWinGravity;
-  size.min_width=0;
-  size.min_height=0;
-  size.max_width=0;
-  size.max_height=0;
-  size.x=xpos;
-  size.y=ypos;
-  size.width=width;
-  size.height=height;
-  size.width_inc=0;
-  size.height_inc=0;
-  size.min_aspect.x=0;
-  size.min_aspect.y=0;
-  size.max_aspect.x=0;
-  size.max_aspect.y=0;
-  size.base_width=0;
-  size.base_height=0;
-  size.win_gravity=StaticGravity; // Account for border (ICCCM)
+      // Set decorations
+      setdecorations();
 
-  // Set hints
-  XSetWMNormalHints(DISPLAY(getApp()),xid,&size);
-#endif
-
-  // Set title
-  settitle();
-
-  // Set decorations
-  setdecorations();
-
-  // Set icon for X-Windows
-  seticons();
+      // Set icon for X-Windows
+      seticons();
+      }
+    }
   }
 
 
@@ -269,7 +262,7 @@ void FXTopWindow::killFocus(){
   FXShell::killFocus();
   if(xid){
 #ifndef WIN32
-    Window win; 
+    Window win;
     int    dum;
     XGetInputFocus(DISPLAY(getApp()),&win,&dum);
     if(win==xid){
@@ -330,6 +323,82 @@ void FXTopWindow::hide(){
     }
   }
 
+// FIXME
+// Unify placement API's under show, also perhaps
+// move to FXWindow, and maybe same in FXMDIChild also
+// for simplicity.
+
+// void FXTopWindow::show(FXuint placement)
+// {
+//
+// #ifdef WIN32
+//   if (placement == PLACEMENT_MAXIMIZED)
+//   {
+//                 place(PLACEMENT_DEFAULT);
+//                 //Replacement of FXShell::show()
+//                 if(!(flags&FLAG_SHOWN))
+//                 {
+//                         flags|=FLAG_SHOWN;
+//                 }
+//                 if (xid)
+//                 {
+//                         ShowWindow((HWND)xid,SW_MAXIMIZE);
+//                 }
+//                 //end of replacement
+//                 //FXShell::show();
+//                 raise();
+//
+//         return;
+//   }
+//
+//   //proposed minimized functionality...
+//   if (placement == PLACEMENT_MINIMIZED)
+//   {
+//                 place(PLACEMENT_DEFAULT);
+//                 //Replacement of FXShell::show()
+//                 if(!(flags&FLAG_SHOWN))
+//                 {
+//                         flags|=FLAG_SHOWN;
+//                 }
+//                 if (xid)
+//                 {
+//                         ShowWindow((HWND)xid,SW_MINIMIZE);
+//                 }
+//                 //end of replacement
+//                 //FXShell::show();
+//                 raise();
+//
+//         return;
+//   }
+//
+// #endif
+//   place(placement);
+//   FXShell::show();
+//   raise();
+// }
+
+// void FXTopWindow::maximize(){
+//   if(xid){
+// #ifndef WIN32
+//     place(PLACEMENT_MAXIMIZED); // I thought it works on Unix until this
+// e-mail
+// #else
+//     ShowWindow((HWND)xid, SW_MAXIMIZE);
+// #endif
+//     }
+//   }
+//
+// FXbool FXTopWindow::isMaximized() const {
+//   FXbool maximized = false;
+//   if(xid){
+// #ifdef WIN32
+//     maximized=IsZoomed((HWND)xid);
+// #endif
+//     }
+//   return maximized;
+//   }
+
+
 
 // Position the window based on placement
 void FXTopWindow::place(FXuint placement){
@@ -599,14 +668,14 @@ void FXTopWindow::setdecorations(){
   SetWindowLong((HWND)xid,GWL_STYLE,dwStyle);
 
   // Patch from Stephane Ancelot <sancelot@wanadoo.fr> and Sander Jansen <sxj@cfdrc.com>
-  HMENU sysmenu=GetSystemMenu((HWND)xid,FALSE); 
-  if(sysmenu){ 
+  HMENU sysmenu=GetSystemMenu((HWND)xid,FALSE);
+  if(sysmenu){
     if(options&DECOR_CLOSE)
       EnableMenuItem(sysmenu,SC_CLOSE,MF_ENABLED);
     else
       EnableMenuItem(sysmenu,SC_CLOSE,MF_GRAYED);
     }
-  
+
   // Adjust non-client area size based on new style
   SetRect(&rect,0,0,width,height);
   AdjustWindowRectEx(&rect,dwStyle,FALSE,dwExStyle);
@@ -619,7 +688,7 @@ void FXTopWindow::setdecorations(){
 
 // Change decorations
 void FXTopWindow::setDecorations(FXuint decorations){
-  FXuint opts=(decorations&~DECOR_ALL) | (decorations&DECOR_ALL);
+  FXuint opts=(options&~DECOR_ALL) | (decorations&DECOR_ALL);
   if(options!=opts){
     options=opts;
     if(xid) setdecorations();
@@ -698,7 +767,7 @@ void FXTopWindow::move(FXint x,FXint y){
       RECT rect;
       SetRect(&rect,xpos,ypos,0,0);
       DWORD dwStyle=GetWindowLong((HWND)xid,GWL_STYLE);
-      BOOL bMenu=FALSE;	// No, we always account for the menu bar ourselves
+      BOOL bMenu=FALSE; // No, we always account for the menu bar ourselves
       DWORD dwExStyle=GetWindowLong((HWND)xid,GWL_EXSTYLE);
       AdjustWindowRectEx(&rect,dwStyle,bMenu,dwExStyle);
       SetWindowPos((HWND)xid,NULL,rect.left,rect.top,0,0,SWP_NOSIZE|SWP_NOZORDER);
@@ -725,7 +794,7 @@ void FXTopWindow::resize(FXint w,FXint h){
       RECT rect;
       SetRect(&rect,0,0,width,height);
       DWORD dwStyle=GetWindowLong((HWND)xid,GWL_STYLE);
-      BOOL bMenu=FALSE;	// No, we always account for the menu bar ourselves
+      BOOL bMenu=FALSE; // No, we always account for the menu bar ourselves
       DWORD dwExStyle=GetWindowLong((HWND)xid,GWL_EXSTYLE);
       AdjustWindowRectEx(&rect,dwStyle,bMenu,dwExStyle);
       SetWindowPos((HWND)xid,NULL,0,0,FXMAX(rect.right-rect.left,1),FXMAX(rect.bottom-rect.top,1),SWP_NOMOVE|SWP_NOZORDER);
@@ -757,7 +826,7 @@ void FXTopWindow::position(FXint x,FXint y,FXint w,FXint h){
       RECT rect;
       SetRect(&rect,xpos,ypos,xpos+width,ypos+height);
       DWORD dwStyle=GetWindowLong((HWND)xid,GWL_STYLE);
-      BOOL bMenu=FALSE;	// No, we always account for the menu bar ourselves
+      BOOL bMenu=FALSE; // No, we always account for the menu bar ourselves
       DWORD dwExStyle=GetWindowLong((HWND)xid,GWL_EXSTYLE);
       AdjustWindowRectEx(&rect,dwStyle,bMenu,dwExStyle);
       SetWindowPos((HWND)xid,NULL,rect.left,rect.top,FXMAX(rect.right-rect.left,1),FXMAX(rect.bottom-rect.top,1),SWP_NOZORDER);
@@ -963,6 +1032,158 @@ long FXTopWindow::onClose(FXObject*,FXSelector,void*){
   }
 
 
+// Focus moved up
+long FXTopWindow::onFocusUp(FXObject*,FXSelector sel,void* ptr){
+  FXWindow *child,*c;
+  FXint cury,childy;
+  if(getFocus()){
+    cury=getFocus()->getY();
+    while(1){
+      child=NULL;
+      childy=-10000000;
+      for(c=getFirst(); c; c=c->getNext()){
+        if(c->shown() && c->getY()<cury && childy<c->getY()){ childy=c->getY(); child=c; }
+        }
+      if(!child) return 0;
+      if(child->isEnabled() && child->canFocus()){
+        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+        return 1;
+        }
+      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      cury=childy;
+      }
+    }
+  else{
+    child=getLast();
+    while(child){
+      if(child->shown()){
+        if(child->isEnabled() && child->canFocus()){
+          child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+          return 1;
+          }
+        if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+        }
+      child=child->getPrev();
+      }
+    }
+  return 0;
+  }
+
+
+// Focus moved down
+long FXTopWindow::onFocusDown(FXObject*,FXSelector sel,void* ptr){
+  FXWindow *child,*c;
+  FXint cury,childy;
+  if(getFocus()){
+    cury=getFocus()->getY();
+    while(1){
+      child=NULL;
+      childy=10000000;
+      for(c=getFirst(); c; c=c->getNext()){
+        if(c->shown() && cury<c->getY() && c->getY()<childy){ childy=c->getY(); child=c; }
+        }
+      if(!child) return 0;
+      if(child->isEnabled() && child->canFocus()){
+        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+        return 1;
+        }
+      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      cury=childy;
+      }
+    }
+  else{
+    child=getFirst();
+    while(child){
+      if(child->shown()){
+        if(child->isEnabled() && child->canFocus()){
+          child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+          return 1;
+          }
+        if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+        }
+      child=child->getNext();
+      }
+    }
+  return 0;
+  }
+
+
+// Focus moved to left
+long FXTopWindow::onFocusLeft(FXObject*,FXSelector sel,void* ptr){
+  FXWindow *child,*c;
+  FXint curx,childx;
+  if(getFocus()){
+    curx=getFocus()->getX();
+    while(1){
+      child=NULL;
+      childx=-10000000;
+      for(c=getFirst(); c; c=c->getNext()){
+        if(c->shown() && c->getX()<curx && childx<c->getX()){ childx=c->getX(); child=c; }
+        }
+      if(!child) return 0;
+      if(child->isEnabled() && child->canFocus()){
+        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+        return 1;
+        }
+      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      curx=childx;
+      }
+    }
+  else{
+    child=getLast();
+    while(child){
+      if(child->shown()){
+        if(child->isEnabled() && child->canFocus()){
+          child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+          return 1;
+          }
+        if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+        }
+      child=child->getPrev();
+      }
+    }
+  return 0;
+  }
+
+
+// Focus moved to right
+long FXTopWindow::onFocusRight(FXObject*,FXSelector sel,void* ptr){
+  FXWindow *child,*c;
+  FXint curx,childx;
+  if(getFocus()){
+    curx=getFocus()->getX();
+    while(1){
+      child=NULL;
+      childx=10000000;
+      for(c=getFirst(); c; c=c->getNext()){
+        if(c->shown() && curx<c->getX() && c->getX()<childx){ childx=c->getX(); child=c; }
+        }
+      if(!child) return 0;
+      if(child->isEnabled() && child->canFocus()){
+        child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+        return 1;
+        }
+      if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+      curx=childx;
+      }
+    }
+  else{
+    child=getFirst();
+    while(child){
+      if(child->shown()){
+        if(child->isEnabled() && child->canFocus()){
+          child->handle(this,MKUINT(0,SEL_FOCUS_SELF),ptr);
+          return 1;
+          }
+        if(child->isComposite() && child->handle(this,sel,ptr)) return 1;
+        }
+      child=child->getNext();
+      }
+    }
+  return 0;
+  }
+
+
 
 // Change regular icon
 void FXTopWindow::setIcon(FXIcon* ic){
@@ -1094,5 +1315,3 @@ FXTopWindow::~FXTopWindow(){
   icon=(FXIcon*)-1;
   miniIcon=(FXIcon*)-1;
   }
-
-
