@@ -3,7 +3,7 @@
 *                 V e r t i c a l   C o n t a i n e r   O b j e c t             *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1997,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1997,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXVerticalFrame.cpp,v 1.11 2001/08/14 06:17:28 jeroen Exp $              *
+* $Id: FXVerticalFrame.cpp,v 1.13 2002/01/18 22:43:07 jeroen Exp $              *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -29,17 +29,8 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
-#include "FXObject.h"
-#include "FXDict.h"
-#include "FXSettings.h"
 #include "FXRegistry.h"
 #include "FXApp.h"
-#include "FXId.h"
-#include "FXDrawable.h"
-#include "FXWindow.h"
-#include "FXFrame.h"
-#include "FXComposite.h"
-#include "FXPacker.h"
 #include "FXVerticalFrame.h"
 
 
