@@ -3,7 +3,7 @@
 *                      J P E G    I n p u t / O u t p u t                       *
 *                                                                               *
 *********************************************************************************
-* Author: David Tyree   Email: celer@ipro.lug.usf.edu                           *
+* Copyright (C) 2000,2002 by David Tyree.   All Rights Reserved.                *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxjpegio.cpp,v 1.19 2001/07/24 21:49:58 jeroen Exp $                     *
+* $Id: fxjpegio.cpp,v 1.25 2002/01/21 19:28:28 jeroen Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -48,11 +48,11 @@
 /*******************************************************************************/
 
 /// Load a jpeg from a stream
-extern FXAPI FXbool fxloadJPEG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint& quality);
+extern FXAPI FXbool fxloadJPG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint& quality);
 
 
 /// Save a jpeg to a stream
-extern FXAPI FXbool fxsaveJPEG(FXStream& store,const FXuchar* data,FXColor transp,FXint width,FXint height,FXint quality);
+extern FXAPI FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor transp,FXint width,FXint height,FXint quality);
 
 
 /*******************************************************************************/
@@ -62,7 +62,7 @@ extern FXAPI FXbool fxsaveJPEG(FXStream& store,const FXuchar* data,FXColor trans
 extern "C" {
 
 /* Theo Veenker <Theo.Veenker@let.uu.nl> says this is needed for CYGWIN */
-#if defined(__CYGWIN__) && !defined(XMD_H)
+#if (defined(__CYGWIN__) || defined(__MINGW32__) || defined(_MSC_VER)) && !defined(XMD_H)
 #define XMD_H
 typedef short INT16;
 typedef int INT32;
@@ -146,7 +146,7 @@ static void term_source(j_decompress_ptr){
 
 
 // Load a JPEG image
-FXbool fxloadJPEG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint&){
+FXbool fxloadJPG(FXStream& store,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint&){
   struct jpeg_decompress_struct srcinfo;
   struct jpeg_error_mgr jerr;
   FOX_jpeg_source_mgr src;
@@ -237,7 +237,7 @@ static void term_destination(j_compress_ptr cinfo){
 
 
 // Save a JPEG image
-FXbool fxsaveJPEG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint height,FXint quality){
+FXbool fxsaveJPG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint height,FXint quality){
   struct jpeg_compress_struct dstinfo;
   struct jpeg_error_mgr jerr;
   FOX_jpeg_dest_mgr dst;
@@ -293,18 +293,33 @@ FXbool fxsaveJPEG(FXStream& store,const FXuchar* data,FXColor,FXint width,FXint
 
 
 // Stub routine
-FXbool fxloadJPEG(FXStream&,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint& quality){
-  data=NULL;
+FXbool fxloadJPG(FXStream&,FXuchar*& data,FXColor& transp,FXint& width,FXint& height,FXint& quality){
+  static const FXuchar jpeg_bits[] = {
+   0xff, 0xff, 0xff, 0xff, 0x01, 0x00, 0x00, 0x80, 0xfd, 0xff, 0xff, 0xbf,
+   0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
+   0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
+   0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
+   0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
+   0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0xf5, 0x3d, 0x9f, 0xa3,
+   0x05, 0x45, 0x41, 0xa4, 0x05, 0x45, 0x41, 0xa0, 0x05, 0x45, 0x47, 0xa0,
+   0x05, 0x3d, 0x41, 0xa6, 0x05, 0x05, 0x41, 0xa4, 0x15, 0x05, 0x41, 0xa4,
+   0xe5, 0x04, 0x9f, 0xa3, 0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0,
+   0x05, 0x00, 0x00, 0xa0, 0x05, 0x00, 0x00, 0xa0, 0xfd, 0xff, 0xff, 0xbf,
+   0x01, 0x00, 0x00, 0x80, 0xff, 0xff, 0xff, 0xff};
+  register FXint p,q;
+  FXCALLOC(&data,FXuchar,32*32*3);
+  for(p=q=0; p<32*32*3; p+=3,q+=1){
+    if(!(jpeg_bits[q>>3]&(1<<(q&7)))) data[p+0]=data[p+1]=data[p+2]=255;
+    }
   transp=0;
-  width=0;
-  height=0;
-  quality=75;
+  width=32;
+  height=32;
   return FALSE;
   }
 
 
 // Stub routine
-FXbool fxsaveJPEG(FXStream&,const FXuchar*,FXColor transp,FXint,FXint,FXint){
+FXbool fxsaveJPG(FXStream&,const FXuchar*,FXColor,FXint,FXint,FXint){
   return FALSE;
   }
 
