@@ -3,7 +3,7 @@
 *                          U t i l i t y   F u n c t i o n s                    *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2001 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2002 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxutils.cpp,v 1.57 2001/07/27 22:20:58 jeroen Exp $                      *
+* $Id: fxutils.cpp,v 1.71.4.4 2003/05/02 17:36:58 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -44,6 +44,14 @@
 unsigned int fxTraceLevel=0;
 
 
+// Version number that the library has been compiled with
+const FXuchar fxversion[3]={FOX_MAJOR,FOX_MINOR,FOX_LEVEL};
+
+
+// Allows GNU autoconfigure to find FOX
+extern "C" FXAPI void fxfindfox(void){ }
+
+
 /*******************************************************************************/
 
 
@@ -56,55 +64,66 @@ FXuint fxrandom(FXuint& seed){
 
 // Allocate memory
 FXint fxmalloc(void** ptr,unsigned long size){
-  FXASSERT(ptr);
-  *ptr=malloc(size);
-  return *ptr!=NULL;
+  *ptr=NULL;
+  if(size!=0){
+    if((*ptr=malloc(size))==NULL) return FALSE;
+    }
+  return TRUE;
   }
 
 
 // Allocate cleaned memory
 FXint fxcalloc(void** ptr,unsigned long size){
-  FXASSERT(ptr);
-  *ptr=calloc(size,1);
-  return *ptr!=NULL;
+  *ptr=NULL;
+  if(size!=0){
+    if((*ptr=calloc(size,1))==NULL) return FALSE;
+    }
+  return TRUE;
   }
 
 
 // Resize memory
 FXint fxresize(void** ptr,unsigned long size){
-  FXASSERT(ptr);
-  void *p=realloc(*ptr,size);
-  if(!size || p){ *ptr=p; return TRUE; }
-  return FALSE;
+  register void *p=NULL;
+  if(size!=0){
+    if((p=realloc(*ptr,size))==NULL) return FALSE;
+    }
+  else{
+    if(*ptr) free(*ptr);
+    }
+  *ptr=p;
+  return TRUE;
   }
 
 
 // Allocate and initialize memory
 FXint fxmemdup(void** ptr,unsigned long size,const void* src){
-  FXASSERT(ptr);
   *ptr=NULL;
-  if(src){
-    *ptr=malloc(size);
-    if(*ptr){ memcpy(*ptr,src,size); return TRUE; }
+  if(size!=0 && src!=NULL){
+    if((*ptr=malloc(size))==NULL) return FALSE;
+    memcpy(*ptr,src,size);
     }
-  return FALSE;
-  }
-
-
-// Free memory, resets ptr to NULL afterward
-void fxfree(void** ptr){
-  FXASSERT(ptr);
-  if(*ptr) free(*ptr);
-  *ptr=NULL;
+  return TRUE;
   }
 
 
 // String duplicate
 FXchar *fxstrdup(const FXchar* str){
-  FXchar *copy;
-  FXMALLOC(&copy,FXchar,strlen(str)+1);
-  strcpy(copy,str);
-  return copy;
+  register FXchar *copy;
+  if(str!=NULL && (copy=(FXchar*)malloc(strlen(str)+1))!=NULL){
+    strcpy(copy,str);
+    return copy;
+    }
+  return NULL;
+  }
+
+
+// Free memory, resets ptr to NULL afterward
+void fxfree(void** ptr){
+  if(*ptr){
+    free(*ptr);
+    *ptr=NULL;
+    }
   }
 
 #ifdef WIN32
@@ -118,6 +137,7 @@ FXchar *fxstrdup(const FXchar* str){
 #define IMAGE_SUBSYSTEM_WINDOWS_CE_GUI 9
 #endif
 
+// getmodulefilename
 
 FXbool fxisconsole(const FXchar *path){
   IMAGE_OPTIONAL_HEADER optional_header;
@@ -358,35 +378,6 @@ void fxsleep(unsigned int n){
   }
 
 
-// Character value of keysym+control state
-FXchar fxkeyval(FXuint keysym,FXuint state){
-  if((state&CONTROLMASK) && 0x60<=keysym && keysym<=0x7f) return keysym-0x60;
-  if(state&0x2000){   // Patch from mohme@hgb-leipzig.de
-    switch(keysym){
-      case 'q': return '@';
-      case '7': return '{';
-      case '8': return '[';
-      case '9': return ']';
-      case '0': return '}';
-      case 0xdf: return '\\';
-      }
-    }
-  if(KEY_space<=keysym && keysym<=255) return keysym;
-  if(KEY_BackSpace<=keysym && keysym<=KEY_Return) return keysym&255;
-  if(KEY_KP_0<=keysym && keysym<=KEY_KP_9) return '0'-KEY_KP_0+keysym;
-  if(KEY_Escape==keysym) return '\033';
-  if(KEY_KP_Space==keysym) return ' ';
-  if(KEY_KP_Tab==keysym) return '\t';
-  if(KEY_KP_Enter==keysym) return '\015';
-  if(KEY_KP_Add==keysym) return '+';
-  if(KEY_KP_Subtract==keysym) return '-';
-  if(KEY_KP_Multiply==keysym) return '*';
-  if(KEY_KP_Divide==keysym) return '/';
-  if(KEY_KP_Equal==keysym) return '=';
-  if(KEY_KP_Decimal==keysym) return '.';
-  return 0;
-  }
-
 #define ISSEP(ch) ((ch)=='+' || (ch)=='-' || (ch)==' ')
 
 
@@ -524,13 +515,22 @@ FXColor makeShadowColor(FXColor clr){
 FXchar* fxgetusername(FXchar* result,FXuint uid){
   if(!result){fxerror("fxgetusername: NULL result argument.\n");}
 #ifndef WIN32
+#ifdef FOX_THREAD_SAFE
+  struct passwd pwdresult,*pwd;
+  char buffer[1024];
+  if(getpwuid_r(uid,&pwdresult,buffer,sizeof(buffer),&pwd)==0 && pwd)
+    strcpy(result,pwd->pw_name);
+  else
+    sprintf(result,"%d",uid);
+#else
   struct passwd *pwd;
   if((pwd=getpwuid(uid))!=NULL)
     strcpy(result,pwd->pw_name);
   else
     sprintf(result,"%d",uid);
+#endif
 #else
-//  sprintf(result,"%d",uid);
+//sprintf(result,"%d",uid);
   strcpy(result,"user");
 #endif
   return result;
@@ -541,13 +541,22 @@ FXchar* fxgetusername(FXchar* result,FXuint uid){
 FXchar* fxgetgroupname(FXchar* result,FXuint gid){
   if(!result){fxerror("fxgetgroupname: NULL result argument.\n");}
 #ifndef WIN32
+#ifdef FOX_THREAD_SAFE
+  struct group grpresult,*grp;
+  char buffer[1024];
+  if(getgrgid_r(gid,&grpresult,buffer,sizeof(buffer),&grp)==0 && grp)
+    strcpy(result,grp->gr_name);
+  else
+    sprintf(result,"%d",gid);
+#else
   struct group *grp;
   if((grp=getgrgid(gid))!=NULL)
     strcpy(result,grp->gr_name);
   else
     sprintf(result,"%d",gid);
+#endif
 #else
-  //sprintf(result,"%d",gid);
+//sprintf(result,"%d",gid);
   strcpy(result,"group");
 #endif
   return result;
@@ -571,6 +580,15 @@ FXchar* fxgetpermissions(FXchar* result,FXuint mode){
   result[10]=0;
 #else
   result[0]='-';
+#ifdef _S_IFDIR
+  if(mode&_S_IFDIR) result[0]='d';
+#endif
+#ifdef _S_IFCHR
+  if(mode&_S_IFCHR) result[0]='c';
+#endif
+#ifdef _S_IFIFO
+  if(mode&_S_IFIFO) result[0]='p';
+#endif
   result[1]='r';
   result[2]='w';
   result[3]='x';
@@ -586,400 +604,6 @@ FXchar* fxgetpermissions(FXchar* result,FXuint mode){
   }
 
 
-// Return TRUE iff s is a prefix of t
-FXbool fxprefix(const FXchar* s,const FXchar* t){
-  const FXchar *p,*q;
-  for(p=s,q=t; *p && *q && *p==*q; p++,q++);
-  return !*p;
-  }
-
-
-// Return TRUE iff s is a suffix of t
-FXbool fxsuffix(const FXchar* s, const FXchar* t){
-  const FXchar *p,*q;
-  for(p=s+strlen(s),q=t+strlen(t); p>s && q>t && *p==*q; p--,q--);
-  return p==s && *p==*q;
-  }
-
-
-// Expand ~ in filenames; using HOME environment variable
-// Note "~name" expands to "~name", "~/name" expands to "$HOME/name",
-// and "~" expands to "$HOME"
-FXchar* fxexpand(FXchar* result,const FXchar* name){
-  if(!result){fxerror("fxexpand: NULL result argument.\n");}
-  *result='\0';
-  if(name){
-    if(name[0]=='~'){
-      if(name[1]=='\0' || name[1]==PATHSEP){
-        fxgethomedir(result);
-        name++;
-        }
-      }
-    strcat(result,name);
-    }
-  return result;
-  }
-
-
-// Construct a full pathname from the given directory and file name
-FXchar* fxpathname(FXchar* result,const FXchar* dirname,const FXchar* filename){
-  int l;
-  if(!result){fxerror("fxpathname: NULL result argument.\n");}
-  fxexpand(result,dirname);
-  if((l=strlen(result)) && result[l-1]!=PATHSEP) result[l++]=PATHSEP;
-  strcpy(result+l,filename);
-  return result;
-  }
-
-
-// Return the directory part of pathname (incl drive name)
-FXchar* fxdirpart(FXchar* result,const FXchar* pathname){
-  FXchar *p;
-  if(!result){fxerror("fxdirpart: NULL result argument.\n");}
-  strcpy(result,pathname);
-  if((p=strrchr(result,PATHSEP))){      // Fix by sancelot@usa.net
-    if(!fxisdir(result)){
-      if((p==result) || (p==result+2 && isalpha(result[0]) && result[1]==':')) p++;
-      *p='\0';
-      }
-    }
-  else{
-    fxgetcurrentdir(result);
-    }
-  return result;
-  }
-
-
-// Return the file title, i.e. filename less extension part of pathname
-FXchar* fxfiletitle(FXchar* result,const FXchar* pathname){
-  FXchar *p;
-  if(!result){fxerror("fxfiletitle: NULL result argument.\n");}
-  result[0]='\0';
-  if(pathname){
-    if(isalpha(pathname[0]) && pathname[1]==':') pathname+=2;   // Skip drive name part
-    p=(FXchar*)strrchr(pathname,PATHSEP);                       // Borland C++ compiler says strrchr() returns a const char*
-    if(p) pathname=p+1;
-    strcpy(result,pathname);
-    p=strrchr(result,'.');
-    if(p) *p=0;
-    }
-  return result;
-  }
-
-
-// Return the filename part of pathname
-FXchar* fxfilepart(FXchar* result,const FXchar* pathname){
-  const FXchar *p; // Borland C++ compiler says strrchr() returns const char*
-  if(!result){fxerror("fxfilepart: NULL result argument.\n");}
-  if(isalpha(pathname[0]) && pathname[1]==':') pathname+=2;   // Skip drive name part
-  p=strrchr(pathname,PATHSEP);
-  if(p) pathname=p+1;
-  strcpy(result,pathname);
-  return result;
-  }
-
-
-// Return the extension part of pathname
-FXchar* fxfileext(FXchar* result,const FXchar* pathname){
-  const FXchar* p; // Borland C++ compiler says strrchr() returns const char*
-  if(!result){fxerror("fxfileext: NULL result argument.\n");}
-  *result='\0';
-  p=strrchr(pathname,PATHSEP);
-  if(p) pathname=p+1;
-  if((p=strrchr(pathname,'.'))) return strcpy(result,p+1);
-  return result;
-  }
-
-
-// Check whether a file exists
-FXbool fxexists(const FXchar *name){
-  if(!name){fxerror("fxexists: NULL name argument.\n");}
-#ifndef WIN32
-  struct stat info;
-  return stat(name,&info)==0;
-#else
-  DWORD atts=GetFileAttributes(name);
-  return (atts!=0xFFFFFFFF);
-#endif
-  }
-
-
-// Check whether its a directory
-FXbool fxisdir(const FXchar *name){
-  if(!name){fxerror("fxisdir: NULL name argument.\n");}
-#ifndef WIN32
-  struct stat info;
-  return (stat(name,&info)==0) && S_ISDIR(info.st_mode);
-#else
-  DWORD atts=GetFileAttributes(name);
-  return (atts!=0xFFFFFFFF) && (atts&FILE_ATTRIBUTE_DIRECTORY);
-#endif
-  }
-
-
-// Check whether its a file
-FXbool fxisfile(const FXchar *name){
-  if(!name){fxerror("fxisdir: NULL name argument.\n");}
-#ifndef WIN32
-  struct stat info;
-  return (stat(name,&info)==0) && S_ISREG(info.st_mode);
-#else
-  DWORD atts=GetFileAttributes(name);
-  return (atts!=0xFFFFFFFF) && !(atts&FILE_ATTRIBUTE_DIRECTORY);
-#endif
-  }
-
-
-// Create a unique numbered backup file name for the given pathname
-FXchar* fxbakname(FXchar* result,const FXchar* pathname){
-  FXchar* p;
-  int n=0;
-  if(!result){fxerror("fxbakname: NULL result argument.\n");}
-  if(!pathname){fxerror("fxbakname: NULL pathname argument.\n");}
-  p=result+strlen(pathname);
-  strcpy(result,pathname);
-  do{
-    sprintf(p,".~%d~",++n);
-    }
-  while(fxexists(result));
-  return result;
-  }
-
-
-
-// Check whether two files are identical (refer to the same inode)
-FXbool fxidentical(const FXchar *name1,const FXchar *name2){
-  if(!name1){fxerror("fxidentical: NULL name1 argument.\n");}
-  if(!name2){fxerror("fxidentical: NULL name2 argument.\n");}
-#ifndef WIN32
-  struct stat info1,info2;
-  return !lstat(name1,&info1) && !lstat(name2,&info2) && info1.st_ino==info2.st_ino;
-#else
-  HANDLE hFile1=CreateFile(name1,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
-  if(hFile1==INVALID_HANDLE_VALUE) return FALSE;
-  HANDLE hFile2=CreateFile(name2,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
-  if(hFile2==INVALID_HANDLE_VALUE){
-    CloseHandle(hFile1);
-    return FALSE;
-    }
-  BY_HANDLE_FILE_INFORMATION info1;
-  BY_HANDLE_FILE_INFORMATION info2;
-  FXbool same=FALSE;
-  if(GetFileInformationByHandle(hFile1,&info1) && GetFileInformationByHandle(hFile2,&info2)){
-    if(info1.nFileIndexLow==info2.nFileIndexLow &&
-       info1.nFileIndexHigh==info2.nFileIndexHigh &&
-       info1.dwVolumeSerialNumber==info2.dwVolumeSerialNumber) same=TRUE;
-    }
-  CloseHandle(hFile1);
-  CloseHandle(hFile2);
-  return same;
-#endif
-  }
-
-
-// Split a string into substrings delimited by a given character
-FXchar *fxsplit(FXchar*& s,FXchar c){
-  FXchar *t=s;
-  if(s && (s=strchr(s,c))) *s++='\0';
-  return t;
-  }
-
-
-// Return the shortest path equivalent to pathname (remove . and ..)
-FXchar *fxshortestpath(FXchar *result,const FXchar *pathname){
-  FXchar path[MAXPATHLEN+1],*cur,*last,*part,*tmp;
-  if(!result){fxerror("fxshortestpath: NULL result argument.\n");}
-  if(!pathname){fxerror("fxshortestpath: NULL pathname argument.\n");}
-  strcpy(path,pathname);
-  cur=result;
-  *cur='\0';
-  tmp=path;
-  last=NULL;
-  if(*tmp==PATHSEP){
-    *cur++=PATHSEP;
-    *cur='\0';
-    tmp++;
-    }
-  while((part=fxsplit(tmp,PATHSEP))){
-    if(strcmp(part,".")==0)
-      ;
-    else if(strcmp(part,"..")==0 && (last=strrchr(result,PATHSEP))){
-      if(last>result)
-        cur=last;
-      else
-        cur=last+1;
-      *cur='\0';
-      }
-    else{
-      if(cur>result && *(cur-1)!=PATHSEP) *cur++=PATHSEP;
-      strcpy(cur,part);
-      cur+=strlen(part);
-      }
-    }
-
-  // Remove trailing backslash(es)
-  while(result<cur-1 && *(cur-1)==PATHSEP && *(cur-2)!=':'){
-    *--cur='\0';
-    }
-
-#ifdef WIN32
-  // Use GetFullPathName() to ensure that the correct drive letter is used
-  DWORD dwSize=GetFullPathName(result,0,NULL,NULL);
-  if(dwSize>0){
-    FXchar *buffer;
-    if(FXMALLOC(&buffer,FXchar,dwSize+1)){
-      if(0!=GetFullPathName(result,dwSize,buffer,NULL))
-        strcpy(result,buffer);
-      FXFREE(&buffer);
-      }
-    }
-#endif
-
-  return result;
-  }
-
-
-// Return directory one level above given one
-FXchar* fxupdir(FXchar* result,const FXchar *dirname){
-  FXchar *ptr;
-  if(!result){fxerror("fxupdir: NULL result argument.\n");}
-  if(!dirname){fxerror("fxupdir: NULL dirname argument.\n");}
-  strcpy(result,dirname);
-  if(!fxistopdir(result)){
-    ptr=strrchr(result,PATHSEP);
-    if(ptr && *(ptr+1)=='\0'){ *ptr='\0'; ptr=strrchr(result,PATHSEP); }
-    if(ptr){
-      if((result[0]==PATHSEP && ptr==result) || (isalpha(result[0]) && result[1]==':' && result[2]==PATHSEP && ptr==result+2)){
-        *(ptr+1)='\0';
-        }
-      else{
-        *ptr='\0';
-        }
-      }
-    }
-  return result;
-  }
-
-
-// Test if pathname is the toplevel directory
-FXbool fxistopdir(const FXchar* path){
-  return (path[0]==PATHSEP && path[1]=='\0') || (isalpha(path[0]) && path[1]==':' && path[2]==PATHSEP && path[3]=='\0');
-  }
-
-
-// Get current working directory
-FXchar* fxgetcurrentdir(FXchar *result){
-  if(!result){fxerror("fxgetcurrentdir: NULL result argument.\n");}
-#ifndef WIN32
-  if(getcwd(result,MAXPATHLEN)==NULL){fxerror("fxgetcurrentdir: failed.\n");}
-#else
-  GetCurrentDirectory(MAXPATHLEN,result);
-#endif
-  return result;
-  }
-
-
-// Make a directory
-FXint fxmkdir(const FXchar* pathname,FXuint mode){
-#if defined(WIN32) && !defined(__CYGWIN__)
-  return _mkdir(pathname);
-#else
-  return mkdir(pathname,mode);
-#endif
-  }
-
-
-// Get home directory
-FXchar* fxgethomedir(FXchar *result){
-  static const FXchar* home=NULL;
-  if(!result){fxerror("fxgethomedir: NULL result argument.\n");}
-  if(!home) {
-#ifdef WIN32
-    HKEY hKey;
-    char    szEnv[MAXPATHLEN+5];     // "HOME="
-    DWORD   dwSize=MAXPATHLEN;
-    if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",0,KEY_READ,&hKey)==ERROR_SUCCESS){
-      strcpy(szEnv,"HOME=");
-      if(RegQueryValueEx(hKey,"Personal",NULL,NULL,(LPBYTE)szEnv+5,&dwSize)==ERROR_SUCCESS){
-#ifdef _MSC_VER
-        _putenv(szEnv);
-#else
-        putenv(szEnv);
-#endif
-        }
-      RegCloseKey(hKey);
-      }
-#endif
-    home=getenv("HOME");
-    if(!home) home="";
-    }
-  strcpy(result,home);
-  return result;
-  }
-
-
-// Translate filename to an absolute pathname; ~ in filename is expanded,
-// and if the resulting pathname is still relative, basename is prepended
-FXchar *fxabspath(FXchar *result,const FXchar *basename,const FXchar *filename){
-  char s[MAXPATHLEN+1],t[MAXPATHLEN+1],cwd[MAXPATHLEN+1];
-
-  // Test
-  if(!result){fxerror("fxabspath: NULL result argument.\n");}
-
-  // Expand tilde's
-  fxexpand(s,filename);
-
-  // Absolute path may be preceded by a drive letter as in C:\blabla\blabla
-  // or just \blabla\blabla or on unix just /blabla/blabla
-  if(!(s[0]==PATHSEP || (isalpha(s[0]) && s[1]==':' && s[2]==PATHSEP))){
-    if(basename==NULL || *basename=='\0'){
-      fxgetcurrentdir(cwd);
-      basename=cwd;
-      }
-    fxpathname(t,basename,s);
-    return fxshortestpath(result,t);
-    }
-  return fxshortestpath(result,s);
-  }
-
-
-// Search path for file name. A ~ in path is expanded. If name is absolute
-// it is returned unchanged. Otherwise the absolute name is returned in
-// result. If name is not found on path, NULL is returned.
-FXchar *fxsearchpath(FXchar *result,const FXchar *path,const FXchar *name){
-  FXchar fullname[MAXPATHLEN+1];
-  FXchar basedir[MAXPATHLEN+1];
-  const FXchar *s,*t;
-  if(!result){fxerror("fxsearchpath: NULL result argument.\n");}
-  if(!path){fxerror("fxsearchpath: NULL path argument.\n");}
-  if(!name){fxerror("fxsearchpath: NULL name argument.\n");}
-  if(name[0]==PATHSEP || (isalpha(name[0]) && name[1]==':' && name[2]==PATHSEP)){
-    strcpy(result,name);
-    if(fxexists(result)) return result;
-    return NULL;
-    }
-  fxgetcurrentdir(basedir);
-  for(s=path; *s; s=t){
-    int l;
-    t=strchr(s,PATHLISTSEP);
-    if(!t) t=s+strlen(s);
-    if(s!=t){
-      if(s[0]=='.'){
-        if(t==s+1) s=t; else if(s[1]==PATHSEP) s+=2;
-        }
-      l=t-s;
-      strncpy(fullname,s,l);
-      if(l>0 && fullname[l-1]!=PATHSEP){ fullname[l]=PATHSEP; l++; }
-      strcpy(fullname+l,name);
-      fxabspath(result,basedir,fullname);
-      if(fxexists(result)) return result;
-      }
-    if(*t) t++;
-    }
-  return NULL;
-  }
-
-
 // Get process id
 FXint fxgetpid(){
 #ifndef WIN32
@@ -993,8 +617,8 @@ FXint fxgetpid(){
 // Convert RGB to HSV
 void fxrgb_to_hsv(FXfloat& h,FXfloat& s,FXfloat& v,FXfloat r,FXfloat g,FXfloat b){
   FXfloat t,delta;
-  v=r; if(g>v) v=g; if(b>v) v=b;
-  t=r; if(g<t) t=g; if(b<t) t=b;
+  v=FXMAX3(r,g,b);
+  t=FXMIN3(r,g,b);
   delta=v-t;
   if(v!=0.0)
     s=delta/v;
@@ -1110,22 +734,6 @@ FXint fxieeedoubleclass(FXdouble number){
 
 #ifdef WIN32
 
-// Return the current state of the modifier keys and mouse buttons
-unsigned int fxmodifierkeys(){
-  FXuint state=0;
-  if(GetKeyState(VK_SHIFT)<0) state|=SHIFTMASK;
-  if(GetKeyState(VK_CAPITAL)>0) state|=CAPSLOCKMASK;
-  if(GetKeyState(VK_CONTROL)<0) state|=CONTROLMASK;
-  if(GetKeyState(VK_MENU)<0) state|=ALTMASK;
-  if(GetKeyState(VK_NUMLOCK)>0) state|=NUMLOCKMASK;
-  if(GetKeyState(VK_SCROLL)>0) state|=SCROLLLOCKMASK;
-  if(GetKeyState(VK_LBUTTON)<0) state|=LEFTBUTTONMASK;
-  if(GetKeyState(VK_MBUTTON)<0) state|=MIDDLEBUTTONMASK;
-  if(GetKeyState(VK_RBUTTON)<0) state|=RIGHTBUTTONMASK;
-  return state;
-  }
-
-
 // Convert font size (in decipoints) to device logical units
 int fxpointsize_to_height(HDC hdc,unsigned size){
   // The calls to SetGraphicsMode() and ModifyWorldTransform() have no effect on Win95/98
@@ -1168,3 +776,5 @@ unsigned fxheight_to_pointsize(HDC hdc,int height){
   }
 
 #endif
+
+
