@@ -5,7 +5,7 @@
 *********************************************************************************
 * Copyright (C) 1998 by Jeroen van der Zijp.   All Rights Reserved.             *
 *********************************************************************************
-* $Id: dirlist.cpp,v 1.10 2001/02/21 21:05:40 jeroen Exp $                       *
+* $Id: dirlist.cpp,v 1.11 2001/11/02 06:07:19 jeroen Exp $                      *
 ********************************************************************************/
 #include "fx.h"
 #include <stdio.h>
@@ -65,7 +65,7 @@ DirListWindow::DirListWindow(FXApp* a):FXMainWindow(a,"Directory List",NULL,NULL
   // Make menu bar
   menubar=new FXMenubar(this,LAYOUT_FILL_X);
   filemenu=new FXMenuPane(this);
-    new FXMenuCommand(filemenu,"&Quit\tCtl-Q",NULL,getApp(),FXApp::ID_QUIT,MENU_DEFAULT);
+    new FXMenuCommand(filemenu,"&Quit\tCtl-Q",NULL,getApp(),FXApp::ID_QUIT);
     new FXMenuTitle(menubar,"&File",NULL,filemenu);
   helpmenu=new FXMenuPane(this);
     new FXMenuCommand(helpmenu,"&About FOX...",NULL,this,ID_ABOUT,0);
