@@ -5,7 +5,7 @@
 *********************************************************************************
 * Copyright (C) 1997 by Jeroen van der Zijp.   All Rights Reserved.             *
 *********************************************************************************
-* $Id: groupbox.cpp,v 1.41 2001/06/28 22:38:25 jeroen Exp $                     *
+* $Id: groupbox.cpp,v 1.47 2002/02/26 17:20:12 fox Exp $                     *
 ********************************************************************************/
 #include "fx.h"
 #include <stdio.h>
@@ -20,7 +20,7 @@
 class GroupWindow : public FXMainWindow {
   FXDECLARE(GroupWindow)
 protected:
-    
+
   // Member data
   FXTooltip*         tooltip;
   FXMenubar*         menubar;
@@ -36,17 +36,18 @@ protected:
   FXGroupBox*        group2;
   FXGroupBox*        group3;
   FXuint             choice;
-  
+
 protected:
   GroupWindow(){}
-  
+
 public:
-  
+
   // Message handlers
   long onCmdFileDlgAny(FXObject*,FXSelector,void*);
   long onCmdFileDlgExisting(FXObject*,FXSelector,void*);
   long onCmdFileDlgMultiple(FXObject*,FXSelector,void*);
   long onCmdFileDlgDirectory(FXObject*,FXSelector,void*);
+  long onCmdFileDlgMultipleAll(FXObject*,FXSelector,void*);
   long onCmdDirDlg(FXObject*,FXSelector,void*);
   long onCmdDownSize(FXObject*,FXSelector,void*);
   long onCmdDelete(FXObject*,FXSelector,void*);
@@ -55,9 +56,9 @@ public:
   long onCmdPopup(FXObject*,FXSelector,void*);
   long onUpdRadio(FXObject*,FXSelector,void*);
   long onCmdOption(FXObject*,FXSelector,void*);
-  
+
 public:
-  
+
   // Messages
   enum {
     ID_DOWNSIZE=FXMainWindow::ID_LAST,
@@ -68,6 +69,7 @@ public:
     ID_FILEDLG_EXISTING,
     ID_FILEDLG_MULTIPLE,
     ID_FILEDLG_DIRECTORY,
+    ID_FILEDLG_MULTIPLE_ALL,
     ID_DIRDLG,
     ID_OPTION1,
     ID_OPTION2,
@@ -77,7 +79,7 @@ public:
     ID_RADIO2,
     ID_RADIO3
     };
-    
+
 public:
   GroupWindow(FXApp* a);
   virtual void create();
@@ -85,9 +87,9 @@ public:
   };
 
 
-  
+
 /*******************************************************************************/
-  
+
 // Map
 FXDEFMAP(GroupWindow) GroupWindowMap[]={
 
@@ -101,6 +103,7 @@ FXDEFMAP(GroupWindow) GroupWindowMap[]={
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_ANY,                       GroupWindow::onCmdFileDlgAny),
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_EXISTING,                  GroupWindow::onCmdFileDlgExisting),
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_MULTIPLE,                  GroupWindow::onCmdFileDlgMultiple),
+  FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_MULTIPLE_ALL,              GroupWindow::onCmdFileDlgMultipleAll),
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_FILEDLG_DIRECTORY,                 GroupWindow::onCmdFileDlgDirectory),
   FXMAPFUNC(SEL_COMMAND,  GroupWindow::ID_DIRDLG,                            GroupWindow::onCmdDirDlg),
   FXMAPFUNCS(SEL_COMMAND, GroupWindow::ID_OPTION1,GroupWindow::ID_OPTION4,   GroupWindow::onCmdOption),
@@ -110,8 +113,8 @@ FXDEFMAP(GroupWindow) GroupWindowMap[]={
 // Object implementation
 FXIMPLEMENT(GroupWindow,FXMainWindow,GroupWindowMap,ARRAYNUMBER(GroupWindowMap))
 
-  
-  
+
+
 /*******************************************************************************/
 
 const unsigned char minidoc1[]={
@@ -145,12 +148,12 @@ const unsigned char minifolderclosed[]={
   0x48,0x05,0x20,0x38,0x9f,0x50,0xe8,0x66,0x4a,0x75,0x24,0x00,0x00,0x3b
   };
 
-  
+
 // Make some windows
 GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DECOR_ALL,0,0,0,0){
-  //tooltip=new FXTooltip(getApp(),0,100,100);
+  tooltip=new FXTooltip(getApp(),0,100,100);
+
 
-  
   FXIcon *doc=new FXGIFIcon(getApp(),minidoc1);
   FXIcon *folder_open=new FXGIFIcon(getApp(),minifolderopen);
   FXIcon *folder_closed=new FXGIFIcon(getApp(),minifolderclosed);
@@ -161,6 +164,7 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
     new FXMenuCommand(filemenu,"Open any",folder_open,this,ID_FILEDLG_ANY);
     new FXMenuCommand(filemenu,"Open existing",folder_open,this,ID_FILEDLG_EXISTING);
     new FXMenuCommand(filemenu,"Open multiple",folder_open,this,ID_FILEDLG_MULTIPLE);
+    new FXMenuCommand(filemenu,"Open multiple all",folder_open,this,ID_FILEDLG_MULTIPLE_ALL);
     new FXMenuCommand(filemenu,"Open directory",folder_open,this,ID_FILEDLG_DIRECTORY);
     new FXMenuCommand(filemenu,"Open directory dialog",folder_open,this,ID_DIRDLG);
     new FXMenuCommand(filemenu,"Radio&1",NULL,this,ID_RADIO1,0);
@@ -184,8 +188,8 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
     new FXMenuCascade(filemenu,"&Edit",NULL,editmenu);
     new FXMenuCommand(filemenu,"&Quit\tCtl-Q",NULL,getApp(),FXApp::ID_QUIT,0);
   new FXMenuTitle(menubar,"&File",NULL,filemenu);
-  
-  
+
+
   helpmenu=new FXMenuPane(this);
     new FXMenuCommand(helpmenu,"&About FOX...",NULL,this,ID_ABOUT,0);
   new FXMenuTitle(menubar,"&Help",NULL,helpmenu,LAYOUT_RIGHT);
@@ -193,11 +197,11 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   popupmenu=new FXMenuPane(this);
     FXTextField* poptext=new FXTextField(popupmenu,10,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP,0,0,0,0);
     poptext->setText("Popup with text");
-  
-    
+
+
   // Status bar
   FXStatusbar *status=new FXStatusbar(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER);
-  
+
   new FXLabel(status,"10:15 PM",NULL,LAYOUT_FILL_Y|LAYOUT_RIGHT|FRAME_SUNKEN);
 
   // Content
@@ -206,27 +210,27 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   group1=new FXGroupBox(contents,"Title Left",GROUPBOX_TITLE_LEFT|FRAME_RIDGE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
   group2=new FXGroupBox(contents,"Slider Tests",GROUPBOX_TITLE_CENTER|FRAME_RIDGE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
   group3=new FXGroupBox(contents,"Title Right",GROUPBOX_TITLE_RIGHT|FRAME_RIDGE|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-  
+
   FXLabel *testlabel=new FXLabel(group1,"&This is a multi-line\nlabel widget\nwith a big font",NULL,LAYOUT_CENTER_X|JUSTIFY_CENTER_X);
   testlabel->setFont(new FXFont(getApp(),"helvetica",24,FONTWEIGHT_BOLD,FONTSLANT_ITALIC,FONTENCODING_DEFAULT));
-  
+
   new FXButton(group1,"Small &Button",NULL,NULL,0,FRAME_RAISED|FRAME_THICK);
   new FXButton(group1,"Big Fat Wide Button\nComprising\nthree lines",NULL,NULL,0,FRAME_RAISED|FRAME_THICK);
   new FXToggleButton(group1,"C&losed\tTooltip for closed\tHelp for closed","O&pen\nState\tTooltip for open\tHelp for open",folder_closed,folder_open,NULL,0,ICON_BEFORE_TEXT|JUSTIFY_LEFT|FRAME_RAISED|FRAME_THICK);
 
   pop=new FXPopup(this);
-  
+
   new FXOption(pop,"&First\tTip #1\tHelp first",NULL,this,ID_OPTION1,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
   new FXOption(pop,"&Second\tTip #2\tHelp second",NULL,this,ID_OPTION2,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
   new FXOption(pop,"Th&ird\tTip #3\tHelp third",NULL,this,ID_OPTION3,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
   new FXOption(pop,"F&ourth\tTip #4\tHelp fourth",NULL,this,ID_OPTION4,JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
-  
+
   new FXOptionMenu(group1,pop,LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|JUSTIFY_HZ_APART|ICON_AFTER_TEXT);
 
   new FXLabel(group1,"Te&kstje",NULL,LAYOUT_TOP|JUSTIFY_LEFT);
   new FXButton(group1,"Add an `&&' by doubling\tTooltip\tHelp text for status",NULL,NULL,0,LAYOUT_TOP|FRAME_RAISED|FRAME_THICK);
   new FXButton(group1,"Te&kstje",NULL,this,ID_POPUP,LAYOUT_TOP|FRAME_RAISED|FRAME_THICK);
-  
+
   new FXMenuButton(group1,"&Menu",NULL,filemenu,MENUBUTTON_ATTACH_BOTH|MENUBUTTON_DOWN|JUSTIFY_HZ_APART|LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|ICON_AFTER_TEXT);
   new FXMenuButton(group1,"&Menu",NULL,filemenu,MENUBUTTON_UP|LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|ICON_AFTER_TEXT);
 
@@ -236,48 +240,55 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   new FXButton(coolpop,"C\tTipC",NULL,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,30,30);
   new FXButton(coolpop,"D\tTipD",NULL,NULL,0,FRAME_THICK|FRAME_RAISED|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,30,30);
   new FXMenuButton(group1,"&S\tSideways",NULL,coolpop,MENUBUTTON_ATTACH_BOTH|MENUBUTTON_LEFT|MENUBUTTON_NOARROWS|LAYOUT_TOP|FRAME_RAISED|FRAME_THICK|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,0,0,30,30);
-  
+
   FXMatrix* matrix=new FXMatrix(group1,3,FRAME_RAISED|LAYOUT_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
-  
+
   new FXButton(matrix,"A",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FILL_ROW);
   new FXButton(matrix,"&Wide button",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X);
   new FXButton(matrix,"A",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X);
-  
+
   new FXButton(matrix,"BBBB",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_FILL_ROW|LAYOUT_FILL_COLUMN);
   new FXButton(matrix,"B",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN);
   new FXButton(matrix,"BB",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_COLUMN);
-  
+
   new FXButton(matrix,"C",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_CENTER_Y|LAYOUT_CENTER_X|LAYOUT_FILL_ROW);
   new FXButton(matrix,"&wide",NULL,NULL,0,FRAME_RAISED|FRAME_THICK);
   new FXButton(matrix,"CC",NULL,NULL,0,FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT);
-  
+
+  FXSlider *slider;
   new FXLabel(group2,"No Arrow");
-  new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL,0,0,200,30);
-  
+  slider=new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL,0,0,200,30);
+
   new FXLabel(group2,"Up Arrow");
-  new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_ARROW_UP,0,0,200,30);
-  
+  slider=new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_ARROW_UP|SLIDER_TICKS_TOP,0,0,200,30);
+  slider->setRange(0,10);
+
   new FXLabel(group2,"Down Arrow");
-  new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_ARROW_DOWN,0,0,200,30);
-  
+  slider=new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_ARROW_DOWN|SLIDER_TICKS_BOTTOM,0,0,200,30);
+  slider->setRange(0,10);
+
   new FXLabel(group2,"Inside Bar");
-  FXSlider *slider=new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR,0,0,200,20);  
+  slider=new FXSlider(group2,NULL,0,LAYOUT_TOP|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|SLIDER_HORIZONTAL|SLIDER_INSIDE_BAR|SLIDER_TICKS_BOTTOM,0,0,200,20);
   slider->setRange(0,3);
-  
+
   FXHorizontalFrame *frame=new FXHorizontalFrame(group2,LAYOUT_FILL_X|LAYOUT_FILL_Y);
-  
-  new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL,0,0,30,200);
-  new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_ARROW_RIGHT,0,0,30,200);  
-  new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_ARROW_LEFT,0,0,30,200);
-  new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_INSIDE_BAR,0,0,20,200);
+
+  slider=new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_TICKS_LEFT|SLIDER_TICKS_RIGHT,0,0,30,200);
+  slider->setRange(0,10);
+  slider=new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_ARROW_RIGHT|SLIDER_TICKS_RIGHT,0,0,30,200);
+  slider->setRange(0,10);
+  slider=new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_ARROW_LEFT|SLIDER_TICKS_LEFT,0,0,30,200);
+  slider->setRange(0,10);
+  slider=new FXSlider(frame,NULL,0,LAYOUT_FIX_HEIGHT|SLIDER_VERTICAL|SLIDER_INSIDE_BAR|SLIDER_TICKS_LEFT,0,0,20,200);
+  slider->setRange(0,10);
   new FXScrollbar(frame,NULL,0,SCROLLBAR_VERTICAL|LAYOUT_FIX_HEIGHT|LAYOUT_FIX_WIDTH,0,0,20,300);
 
-  FXVerticalFrame *vframe1=new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y);  
+  FXVerticalFrame *vframe1=new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y);
   new FXArrowButton(vframe1,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_UP);
   new FXArrowButton(vframe1,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_DOWN);
   new FXArrowButton(vframe1,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_LEFT);
   new FXArrowButton(vframe1,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_RIGHT);
-  FXVerticalFrame *vframe2=new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y);  
+  FXVerticalFrame *vframe2=new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y);
   new FXArrowButton(vframe2,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_UP|ARROW_TOOLBAR);
   new FXArrowButton(vframe2,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_DOWN|ARROW_TOOLBAR);
   new FXArrowButton(vframe2,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_RAISED|FRAME_THICK|ARROW_LEFT|ARROW_TOOLBAR);
@@ -288,16 +299,18 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   new FXRadioButton(gp,"Hilversum &2",NULL,0,ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
   new FXRadioButton(gp,"One multi-line\nRadiobox Widget",NULL,0,JUSTIFY_LEFT|JUSTIFY_TOP|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
   new FXRadioButton(gp,"Radio Stad Amsterdam",NULL,0,ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
-  
+
   FXPacker *vv=new FXGroupBox(group3,"Group Box",LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0,0,0,0);
   new FXCheckButton(vv,"Hilversum 1",NULL,0,ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
   new FXCheckButton(vv,"Hilversum 2",NULL,0,ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
   new FXCheckButton(vv,"One multi-line\nCheckbox Widget",NULL,0,JUSTIFY_LEFT|JUSTIFY_TOP|ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
   new FXCheckButton(vv,"Radio Stad Amsterdam",NULL,0,ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP);
-  
+
   FXSpinner *spinner=new FXSpinner(group3,20,NULL,0,SPIN_NORMAL|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP);
   spinner->setRange(1,20);
-  
+  spinner->setTipText("tip");
+  spinner->setHelpText("help");
+
   FXComboBox* combobox=new FXComboBox(group3,5,5,NULL,0,COMBOBOX_INSERT_LAST|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP);
   combobox->appendItem("Very Wide Item");
   for(int i=0; i<3; i++){
@@ -305,7 +318,9 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
     sprintf(name,"%04d",i);
     combobox->appendItem(name);
     }
-  
+
+  //FXDebugTarget *debugtarget=new FXDebugTarget;
+
   FXTreeListBox *treebox=new FXTreeListBox(group3,10,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP,0,0,200,0);
   FXTreeItem *branch,*twig,*leaf,*topmost,*topmost2;
 
@@ -331,7 +346,7 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
       treebox->addItemLast(twig,"Fourth-Fourth-First",doc,doc);
       treebox->addItemLast(twig,"Fourth-Fourth-Second",doc,doc);
       treebox->addItemLast(twig,"Fourth-Fourth-Third",doc,doc);
-  
+
   new FXLabel(group3,"H&it the hotkey",NULL,LAYOUT_CENTER_X|JUSTIFY_CENTER_X|FRAME_RAISED);
   FXTextField* textfield1=new FXTextField(group3,20,NULL,0,JUSTIFY_RIGHT|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP);
   textfield1->setText("Normal Text Field");
@@ -342,15 +357,15 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   FXTextField* textfield4=new FXTextField(group3,20,NULL,0,TEXTFIELD_READONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP);
   textfield4->setText("Grayed out");
   textfield4->disable();
-  
+
   FXTextField* realnumber=new FXTextField(group3,20,NULL,0,TEXTFIELD_REAL|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FIX_HEIGHT,0,0,0,30);
   realnumber->setText("1.0E+3");
   FXTextField* intnumber=new FXTextField(group3,20,NULL,0,TEXTFIELD_INTEGER|FRAME_SUNKEN|FRAME_THICK|LAYOUT_SIDE_TOP|LAYOUT_FIX_HEIGHT,0,0,0,30);
   intnumber->setText("1000");
-  
+
   FXDial *dial2=new FXDial(group3,NULL,0,DIAL_CYCLIC|DIAL_HAS_NOTCH|DIAL_HORIZONTAL|LAYOUT_FILL_X|FRAME_RAISED|FRAME_THICK,0,0,120,0);
   new FXScrollbar(group3,NULL,0,SCROLLBAR_HORIZONTAL|LAYOUT_FIX_HEIGHT|LAYOUT_FIX_WIDTH,0,0,300,20);
-  
+
   FXProgressBar *pbar=new FXProgressBar(group3,NULL,0,LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|PROGRESSBAR_PERCENTAGE);
   pbar->setProgress(48);
   pbar->setTotal(360);
@@ -363,9 +378,9 @@ GroupWindow::GroupWindow(FXApp* a):FXMainWindow(a,"Group Box Test",NULL,NULL,DEC
   dial2->setTarget(pbar);
   dial2->setSelector(FXWindow::ID_SETVALUE);
   choice=0;
-  
+
   }
-  
+
 GroupWindow::~GroupWindow(){
   delete filemenu;
   delete helpmenu;
@@ -399,7 +414,7 @@ long GroupWindow::onCmdFileDlgExisting(FXObject*,FXSelector,void*){
 
 // Open multiple
 long GroupWindow::onCmdFileDlgMultiple(FXObject*,FXSelector,void*){
-  FXString* files=FXFileDialog::getOpenFilenames(this,"Open file","../tests/groupbox.cpp",sourcefiles);
+  FXString* files=FXFileDialog::getOpenFilenames(this,"Open files","../tests/groupbox.cpp",sourcefiles);
   if(files){
     for(int i=0; !files[i].empty(); i++){
       fxmessage("Files=\"%s\"\n",files[i].text());
@@ -410,6 +425,23 @@ long GroupWindow::onCmdFileDlgMultiple(FXObject*,FXSelector,void*){
   }
 
 
+// Open multiple all
+long GroupWindow::onCmdFileDlgMultipleAll(FXObject*,FXSelector,void*){
+  FXFileDialog open(this,"Open files or directories");
+  open.setSelectMode(SELECTFILE_MULTIPLE_ALL);
+  if(open.execute(PLACEMENT_CURSOR)){
+    FXString* files=open.getFilenames();
+    if(files){
+      for(int i=0; !files[i].empty(); i++){
+        fxmessage("Files=\"%s\"\n",files[i].text());
+        }
+      delete [] files;
+      }
+    }
+  return 1;
+  }
+
+
 // Open existing directory
 long GroupWindow::onCmdFileDlgDirectory(FXObject*,FXSelector,void*){
   FXString dir=FXFileDialog::getOpenDirectory(this,"Open directory","../tests");
@@ -493,19 +525,19 @@ void GroupWindow::create(){
 
 // Start the whole thing
 int main(int argc,char *argv[]){
-  
+
   // Make application
   FXApp application("Groupbox","FoxTest");
-  
+
   // Open display
   application.init(argc,argv);
-  
+
   // Make window
   new GroupWindow(&application);
-  
+
   // Create app
   application.create();
-  
+
   // Run
   return application.run();
   }
